Package: ReliaPlotR
Type: Package
Title: Interactive Reliability Probability Plots
Version: 0.7
Authors@R: person("Paul", "Govan", email = "paul.govan2@gmail.com", role = c("aut", "cre", "cph"), comment = c(ORCID = "0000-0002-1821-8492"))
Description: Build interactive Reliability Probability Plots with 'plotly' by Carson Sievert (2020) <https://plotly.com/r/>, an interactive web-based graphing library.
URL: https://paulgovan.github.io/ReliaPlotR/,
        https://github.com/paulgovan/ReliaPlotR
BugReports: https://github.com/paulgovan/ReliaPlotR/issues
License: Apache License
Imports: plotly, ReliaGrowR, WeibullR, WeibullR.ALT
Suggests: ellmer, jsonlite, knitr, mcptools, rmarkdown, spelling,
        testthat (>= 3.0.0)
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Config/testthat/edition: 3
Language: en-US
NeedsCompilation: no
Packaged: 2026-06-03 20:43:49 UTC; paulgovan
Author: Paul Govan [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-1821-8492>)
Maintainer: Paul Govan <paul.govan2@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-04 05:11:36 UTC
Built: R 4.6.0; ; 2026-06-04 08:22:54 UTC; unix

# Generated by roxygen2: do not edit by hand

export(plotly_alt)
export(plotly_contour)
export(plotly_duane)
export(plotly_exposure)
export(plotly_mcf)
export(plotly_nhpp)
export(plotly_rel)
export(plotly_rga)
export(plotly_wblr)
export(reliapltr_mcp_server)
export(tidy_alt)
export(tidy_rga)
export(tidy_wblr)
import(ReliaGrowR)
import(WeibullR)
import(WeibullR.ALT)
import(plotly)
importFrom(graphics,text)
importFrom(plotly,add_trace)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotly,toRGB)
importFrom(stats,coef)
importFrom(stats,plnorm)
importFrom(stats,pweibull)
importFrom(stats,qlnorm)
importFrom(stats,qnorm)
importFrom(stats,qweibull)
importFrom(stats,runif)

# ReliaPlotR

<!-- badges: start -->

[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![CRAN
status](https://www.r-pkg.org/badges/version/ReliaPlotR)](https://CRAN.R-project.org/package=ReliaPlotR)
[![R-CMD-check](https://github.com/paulgovan/ReliaPlotR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/paulgovan/ReliaPlotR/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/paulgovan/ReliaPlotR/graph/badge.svg)](https://app.codecov.io/gh/paulgovan/ReliaPlotR)
[![](http://cranlogs.r-pkg.org/badges/last-month/ReliaPlotR)](https://cran.r-project.org/package=ReliaPlotR)
[![](http://cranlogs.r-pkg.org/badges/grand-total/ReliaPlotR)](https://cran.r-project.org/package=ReliaPlotR)

<!-- badges: end -->

Build interactive Reliability Probability Plots with `plotly`, an
interactive web-based graphing library.

## Getting Started

To install `ReliaPlotR` in R:

``` r
install.packages("ReliaPlotR")
```

Or install the development version:

``` r
devtools::install_github("paulgovan/ReliaPlotR")
```

## Basic Examples

To build a probability plot, first fit a `wblr` object using the
`WeibullR` package and then use `plotly_wblr` to build the plot.

``` r
library(WeibullR)
library(ReliaPlotR)
failures <- c(30, 49, 82, 90, 96)
obj <- wblr.conf(wblr.fit(wblr(failures)))
plotly_wblr(obj)
```

![](ReadMe_files/figure-gfm/unnamed-chunk-3-1.png)<!-- -->

To build a contour plot, use the `plotly_contour` function. Note that
contour plots are only available where `method.fit='mle'` and
`method.conf='lrb'`.

``` r
obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
plotly_contour(obj)
```

![](ReadMe_files/figure-gfm/unnamed-chunk-4-1.png)<!-- -->

## Customization

`ReliaPlotR` has several customization options.

``` r
plotly_wblr(obj, main = "Weibull Probability Plot", xlab = "Years", ylab = "Failure Probability", confCol = "blue", signif = 4, grid = FALSE)
```

![](ReadMe_files/figure-gfm/unnamed-chunk-5-1.png)<!-- -->

``` r
plotly_contour(obj, main = "Weibull Contour Plot", col = "red", signif = 4, grid = FALSE)
```

![](ReadMe_files/figure-gfm/unnamed-chunk-6-1.png)<!-- -->

## Code of Conduct

Please note that the ReliaPlotR project is released with a [Contributor
Code of
Conduct](https://github.com/paulgovan/ReliaPlotR/blob/f919aeb72a1d4dd3a64e55221eb1ae214b3480f5/CODE_OF_CONDUCT.md).
By contributing to this project, you agree to abide by its terms.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
has_ellmer <- requireNamespace("ellmer", quietly = TRUE)

## ----setup--------------------------------------------------------------------
library(ReliaPlotR)

## ----install, eval = FALSE----------------------------------------------------
# pak::pak(c("mcptools", "ellmer"))

## ----wblr-example, eval = has_ellmer------------------------------------------
# Five machines that failed at these times (hours)
tool <- ReliaPlotR:::.make_wblr_tool()
result <- tool(failures = c(30, 49, 82, 90, 96))
str(result)

## ----wblr-susp, eval = has_ellmer---------------------------------------------
result_susp <- tool(
  failures    = c(30, 49, 82, 90, 96),
  suspensions = c(100, 120, 150)
)
cat("Beta:", result_susp$param1, "\n")
cat("Eta: ", result_susp$param2, "\n")

## ----alt-example, eval = has_ellmer, results = "hide"-------------------------
tool_alt <- ReliaPlotR:::.make_alt_tool()
result_alt <- tool_alt(
  stresses      = c(300, 350, 400),
  failures_json = "[[248,456,528,731,813,537],[164,176,289],[88,112,152]]",
  dist          = "weibull",
  alt_model     = "arrhenius"
)

## ----alt-show, eval = has_ellmer----------------------------------------------
# Per-stress-level parameter estimates
result_alt$parallel

# Global Arrhenius life-stress relationship coefficients
result_alt$relationship

## ----plot-wblr-example, eval = has_ellmer-------------------------------------
ptool <- ReliaPlotR:::.make_plot_wblr_tool()
json_str <- ptool(failures = c(30, 49, 82, 90, 96))
nchar(json_str)  # plotly JSON string length

## ----plot-alt-example, eval = has_ellmer, results = "hide"--------------------
ptool_alt <- ReliaPlotR:::.make_plot_alt_tool()
result_plots <- ptool_alt(
  stresses      = c(300, 350, 400),
  failures_json = "[[248,456,528,731,813,537],[164,176,289],[88,112,152]]",
  dist          = "weibull",
  alt_model     = "arrhenius"
)

## ----plot-alt-show, eval = has_ellmer-----------------------------------------
nchar(result_plots$probability_plot)
nchar(result_plots$life_stress_plot)

## ----plot-rga-example, eval = has_ellmer--------------------------------------
ptool_rga <- ReliaPlotR:::.make_plot_rga_tool()
json_rga <- ptool_rga(
  times    = c(100, 200, 300, 400, 500),
  failures = c(  2,   1,   3,   1,   2)
)
nchar(json_rga)

#' Interactive Reliability Growth Plot.
#'
#' The function creates an interactive reliability growth plot for one or more `rga`
#' objects. When a list of objects is provided the models are overlaid on the same
#' plot, each rendered in a distinct color.
#' The plot includes cumulative failures over time, the model fit, and optional confidence bounds.
#' Vertical lines indicate change points if breakpoints are specified in the rga object.
#'
#'
#' @param rga_obj An object of class 'rga', or a list of such objects. Each object
#' is created using the `rga()` function from the `ReliaGrowR` package.
#' @param showConf Show the confidence bounds (TRUE) or not (FALSE). Default is TRUE.
#' @param showGrid Show grid (TRUE) or hide grid (FALSE). Default is TRUE.
#' @param main Main title. Default is "Reliability Growth Plot".
#' @param xlab X-axis label. Default is "Cumulative Time".
#' @param ylab Y-axis label. Default is "Cumulative Failures".
#' @param pointCol Color of the point values. Default is "black". Used only for a
#' single rga object; ignored when `cols` is provided or multiple objects are supplied.
#' @param fitCol Color of the model fit. Default is "black". Used only for a single
#' rga object; ignored when `cols` is provided or multiple objects are supplied.
#' @param confCol Color of the confidence bounds. Default is "black". Used only for a
#' single rga object; ignored when `cols` is provided or multiple objects are supplied.
#' @param gridCol Color of the grid. Default is "lightgray".
#' @param breakCol Color of the breakpoints. Default is "black". Used only for a
#' single rga object; ignored when `cols` is provided or multiple objects are supplied.
#' @param signif Significant digits of results. Default is 3. Must be a positive integer.
#' @param cols Optional character vector of colors, one per rga object. When provided,
#' each object's points, fit line, confidence bounds, and breakpoints are all drawn in
#' the corresponding color. Recycled if shorter than the number of objects.
#' @return A `plotly` object representing the interactive reliability growth plot.
#' @details
#' The Crow-AMSAA (NHPP Power Law) model fits a Non-Homogeneous Poisson
#' Process where the expected cumulative failures follow
#' \eqn{E[N(t)] = \lambda t^\beta}. A \eqn{\beta < 1} indicates reliability
#' growth (decreasing failure rate), \eqn{\beta > 1} indicates reliability
#' degradation. For piecewise models, a separate set of parameters is fit
#' within each interval defined by \code{breaks}, and vertical lines mark
#' the change-point times.
#' @seealso [plotly_duane()] for the Duane MTBF growth model on the same data.
#' @examples
#' library(ReliaGrowR)
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' rga <- rga(times, failures)
#' plotly_rga(rga)
#'
#' times <- c(100, 200, 300, 400, 500, 600, 700, 800, 900, 1000)
#' failures <- c(1, 2, 1, 1, 1, 2, 3, 1, 2, 4)
#' breakpoints <- 400
#' rga2 <- rga(times, failures, model_type = "Piecewise NHPP", breaks = breakpoints)
#' plotly_rga(rga2, fitCol = "blue", confCol = "blue", breakCol = "red")
#'
#' # Overlay two models
#' rga3 <- rga(c(50, 150, 250, 350, 450), c(2, 1, 3, 1, 2))
#' plotly_rga(list(rga, rga3))
#' @import ReliaGrowR plotly
#' @importFrom graphics text
#' @importFrom stats runif qnorm
#' @export

plotly_rga <- function(rga_obj,
                       showConf = TRUE,
                       showGrid = TRUE,
                       main = "Reliability Growth Plot",
                       xlab = "Cumulative Time",
                       ylab = "Cumulative Failures",
                       pointCol = "black",
                       fitCol = "black",
                       confCol = "black",
                       gridCol = "lightgray",
                       breakCol = "black",
                       signif = 3,
                       cols = NULL) {

  # Normalize to list
  if (!is.list(rga_obj) || inherits(rga_obj, "rga")) {
    rga_obj <- list(rga_obj)
  }
  n <- length(rga_obj)

  # Validate inputs
  lapply(rga_obj, function(obj) {
    if (!inherits(obj, "rga")) {
      stop("All inputs must be of class 'rga'.")
    }
  })

  # Set up per-object colors
  default_palette <- c(
    "#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd",
    "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"
  )
  if (is.null(cols)) {
    if (n == 1) {
      obj_colors <- list(list(point = pointCol, fit = fitCol, conf = confCol, brk = breakCol))
    } else {
      base_cols <- rep(default_palette, length.out = n)
      obj_colors <- lapply(base_cols, function(col) {
        list(point = col, fit = col, conf = col, brk = col)
      })
    }
  } else {
    base_cols <- rep(cols, length.out = n)
    obj_colors <- lapply(base_cols, function(col) {
      list(point = col, fit = col, conf = col, brk = col)
    })
  }

  xgrid <- ifelse(is.null(showGrid) || isTRUE(showGrid), TRUE, FALSE)

  vline <- function(x = 0, color = "black") {
    list(
      type = "line",
      y0 = 0, y1 = 1, yref = "paper",
      x0 = x, x1 = x,
      line = list(color = color, dash = "dot")
    )
  }

  # Build plot
  p <- plot_ly()
  all_shapes <- list()

  for (i in seq_along(rga_obj)) {
    obj <- rga_obj[[i]]
    clrs <- obj_colors[[i]]
    fillcolor <- plotly::toRGB(clrs$conf, 0.2)

    times <- exp(obj$model$model$log_times)
    cum_failures <- exp(obj$model$model$log_cum_failures)
    fitted <- obj$fitted_values

    label_suffix <- if (n > 1) paste0(" ", i) else ""
    show_in_legend <- n > 1

    p <- p %>%
      add_trace(
        x = times, y = cum_failures, type = "scatter", mode = "markers",
        marker = list(color = clrs$point),
        showlegend = show_in_legend,
        legendgroup = as.character(i),
        name = paste0("Data", label_suffix),
        text = ~ paste0("Failures: (", round(times, signif), ", ", round(cum_failures, signif), ")"),
        hoverinfo = "text"
      ) %>%
      add_trace(
        x = times, y = fitted, mode = "markers+lines",
        marker = list(color = "transparent"), line = list(color = clrs$fit),
        showlegend = show_in_legend,
        legendgroup = as.character(i),
        name = paste0("Fit", label_suffix),
        text = ~ paste0("Fit: (", round(times, signif), ", ", round(fitted, signif), ")"),
        hoverinfo = "text"
      )

    if (isTRUE(showConf)) {
      p <- p %>%
        add_trace(
          x = times, y = obj$lower_bounds, mode = "markers+lines",
          marker = list(color = "transparent"), line = list(color = clrs$conf),
          showlegend = FALSE,
          legendgroup = as.character(i),
          text = ~ paste0("Lower: (", round(times, signif), ", ", round(obj$lower_bounds, signif), ")"),
          hoverinfo = "text"
        ) %>%
        add_trace(
          x = times, y = obj$upper_bounds, mode = "markers+lines",
          fill = "tonexty", fillcolor = fillcolor,
          marker = list(color = "transparent"), line = list(color = clrs$conf),
          showlegend = FALSE,
          legendgroup = as.character(i),
          text = ~ paste0("Upper: (", round(times, signif), ", ", round(obj$upper_bounds, signif), ")"),
          hoverinfo = "text"
        )
    }

    # Collect breakpoint shapes
    if (!is.null(obj$breakpoints) && length(obj$breakpoints) > 0) {
      bp_times <- exp(as.numeric(obj$breakpoints))
      for (bp in bp_times) {
        all_shapes <- c(all_shapes, list(vline(bp, color = clrs$brk)))
      }
    }
  }

  p <- p %>%
    layout(
      title = main,
      xaxis = list(
        title = xlab, showline = TRUE, mirror = "ticks",
        showgrid = xgrid, gridcolor = gridCol
      ),
      yaxis = list(
        title = ylab, showline = TRUE, mirror = "ticks",
        showgrid = xgrid, gridcolor = gridCol
      ),
      shapes = all_shapes
    )

  return(p)
}

# Create a mock 'rga' class object for testing
mock_rga_obj <- list(
  model = list(model = list(
    log_times = log(c(100, 200, 300, 400, 500)),
    log_cum_failures = log(c(1, 3, 6, 10, 15))
  )),
  fitted_values = c(1.5, 3.5, 6.5, 10.5, 15.5),
  lower_bounds = c(1.2, 3.2, 6.2, 10.2, 15.2),
  upper_bounds = c(1.8, 3.8, 6.8, 10.8, 15.8),
  breakpoints = c(200, 400)
)
class(mock_rga_obj) <- "rga"

# Unit tests
test_that("plotly_rga throws an error for incorrect input class", {
  incorrect_obj <- list(model = list(model = list()))
  expect_error(plotly_rga(incorrect_obj), "All inputs must be of class 'rga'.")
})

test_that("plotly_rga returns a plotly object", {
  plot <- plotly_rga(mock_rga_obj)
  expect_s3_class(plot, "plotly")
})

test_that("plotly_rga works with default parameters", {
  plot <- plotly_rga(mock_rga_obj)
  expect_true(!is.null(plot))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_rga works with custom color parameters", {
  plot <- plotly_rga(mock_rga_obj, pointCol = "red", fitCol = "blue", confCol = "green", gridCol = "yellow", breakCol = "purple")
  expect_s3_class(plot, "plotly")
})

test_that("plotly_rga respects the showGrid parameter", {
  plot_with_grid <- plotly_rga(mock_rga_obj, showGrid = TRUE)
  plot_without_grid <- plotly_rga(mock_rga_obj, showGrid = FALSE)

  # Check if grid lines are reflected in the plot layout
  expect_equal(plot_with_grid$x$layoutAttrs[[1]]$xaxis$showgrid, TRUE)
  expect_equal(plot_with_grid$x$layoutAttrs[[1]]$yaxis$showgrid, TRUE)
  expect_equal(plot_without_grid$x$layoutAttrs[[1]]$xaxis$showgrid, FALSE)
  expect_equal(plot_without_grid$x$layoutAttrs[[1]]$yaxis$showgrid, FALSE)
})

test_that("plotly_rga includes hover text for data points", {
  plot <- plotly_rga(mock_rga_obj)
  hover_data <- plot$x$data[[1]]$text
  expect_true(all(grepl("Failures: \\(", hover_data)))
})

test_that("plotly_rga respects showConf = FALSE", {
  plot_with_conf <- plotly_rga(mock_rga_obj, showConf = TRUE)
  plot_without_conf <- plotly_rga(mock_rga_obj, showConf = FALSE)

  # With confidence: point + fit + lower + upper = 4 traces
  # Without confidence: point + fit = 2 traces
  expect_gt(length(plot_with_conf$x$attrs), length(plot_without_conf$x$attrs))
})

test_that("plotly_rga accepts a list of rga objects and returns a plotly object", {
  plot <- plotly_rga(list(mock_rga_obj, mock_rga_obj))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_rga overlay produces more traces than single object", {
  single <- plotly_rga(mock_rga_obj)
  overlay <- plotly_rga(list(mock_rga_obj, mock_rga_obj))
  expect_gt(length(overlay$x$attrs), length(single$x$attrs))
})

test_that("plotly_rga overlay respects cols parameter", {
  plot <- plotly_rga(list(mock_rga_obj, mock_rga_obj), cols = c("red", "blue"))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_rga throws error for non-rga input in list", {
  expect_error(
    plotly_rga(list(mock_rga_obj, list())),
    "All inputs must be of class 'rga'."
  )
})

test_that("plotly_rga works when breakpoints is NULL", {
  mock_no_bp <- mock_rga_obj
  mock_no_bp$breakpoints <- NULL
  plot <- plotly_rga(mock_no_bp)
  expect_s3_class(plot, "plotly")
})

test_that("plotly_rga respects custom main, xlab, ylab", {
  plot <- plotly_rga(mock_rga_obj, main = "My RGA", xlab = "Time", ylab = "Failures")
  expect_equal(plot$x$layoutAttrs[[1]]$title, "My RGA")
  expect_equal(plot$x$layoutAttrs[[1]]$xaxis$title, "Time")
  expect_equal(plot$x$layoutAttrs[[1]]$yaxis$title, "Failures")
})

#' Interactive Mean Cumulative Function Plot.
#'
#' The function creates an interactive Mean Cumulative Function (MCF) plot for
#' one or more `mcf` objects. When a list of objects is provided the models are
#' overlaid on the same plot, each rendered in a distinct color.
#' The MCF is rendered as a step function. Optional confidence bounds are shown
#' as a shaded band around the estimate.
#'
#' @param mcf_obj An object of class 'mcf', or a list of such objects. Each
#' object is created using the `mcf()` function from the `ReliaGrowR` package.
#' @param showConf Show the confidence bounds (TRUE) or not (FALSE). Default is TRUE.
#' @param showGrid Show grid (TRUE) or hide grid (FALSE). Default is TRUE.
#' @param main Main title. Default is "Mean Cumulative Function Plot".
#' @param xlab X-axis label. Default is "Time".
#' @param ylab Y-axis label. Default is "Mean Cumulative Function".
#' @param fitCol Color of the MCF step function. Default is "black". Used only
#' for a single mcf object; ignored when `cols` is provided or multiple objects
#' are supplied.
#' @param confCol Color of the confidence bounds. Default is "black". Used only
#' for a single mcf object; ignored when `cols` is provided or multiple objects
#' are supplied.
#' @param gridCol Color of the grid. Default is "lightgray".
#' @param signif Significant digits of results. Default is 3. Must be a positive integer.
#' @param cols Optional character vector of colors, one per mcf object. When
#' provided, each object's step function and confidence bounds are drawn in the
#' corresponding color. Recycled if shorter than the number of objects.
#' @return A `plotly` object representing the interactive MCF plot.
#' @details
#' The MCF is a nonparametric estimate of the expected cumulative number of
#' events per system by time \eqn{t}. It is computed from recurrence data
#' (multiple events per system) and rendered as a step function. The slope
#' of the MCF at any point approximates the current event rate (repairs per
#' unit time). Confidence bounds are computed using the Nelson variance
#' estimator.
#' @seealso [plotly_nhpp()] to overlay a parametric Power Law model on the MCF;
#' [plotly_exposure()] for the cumulative event rate.
#' @examples
#' library(ReliaGrowR)
#' ids <- c("A", "A", "A", "B", "B", "C", "C", "C", "C")
#' times <- c(50, 150, 350, 100, 300, 80, 200, 320, 450)
#' fit <- mcf(id = ids, time = times)
#' plotly_mcf(fit)
#'
#' # Overlay two MCF objects
#' fit2 <- mcf(id = c("X", "X", "Y"), time = c(60, 220, 180))
#' plotly_mcf(list(fit, fit2), cols = c("steelblue", "tomato"))
#' @import ReliaGrowR plotly
#' @importFrom graphics text
#' @importFrom stats runif qnorm
#' @export

plotly_mcf <- function(mcf_obj,
                       showConf = TRUE,
                       showGrid = TRUE,
                       main = "Mean Cumulative Function Plot",
                       xlab = "Time",
                       ylab = "Mean Cumulative Function",
                       fitCol = "black",
                       confCol = "black",
                       gridCol = "lightgray",
                       signif = 3,
                       cols = NULL) {

  # Normalize to list
  if (!is.list(mcf_obj) || inherits(mcf_obj, "mcf")) {
    mcf_obj <- list(mcf_obj)
  }
  n <- length(mcf_obj)

  # Validate inputs
  lapply(mcf_obj, function(obj) {
    if (!inherits(obj, "mcf")) {
      stop("All inputs must be of class 'mcf'.")
    }
  })

  # Set up per-object colors
  default_palette <- c(
    "#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd",
    "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"
  )
  if (is.null(cols)) {
    if (n == 1) {
      obj_colors <- list(list(fit = fitCol, conf = confCol))
    } else {
      base_cols <- rep(default_palette, length.out = n)
      obj_colors <- lapply(base_cols, function(col) {
        list(fit = col, conf = col)
      })
    }
  } else {
    base_cols <- rep(cols, length.out = n)
    obj_colors <- lapply(base_cols, function(col) {
      list(fit = col, conf = col)
    })
  }

  xgrid <- ifelse(is.null(showGrid) || isTRUE(showGrid), TRUE, FALSE)

  # Build plot
  p <- plot_ly()

  for (i in seq_along(mcf_obj)) {
    obj <- mcf_obj[[i]]
    clrs <- obj_colors[[i]]
    fillcolor <- plotly::toRGB(clrs$conf, 0.2)

    label_suffix <- if (n > 1) paste0(" ", i) else ""
    show_in_legend <- n > 1

    p <- p %>%
      add_trace(
        type = "scatter", x = obj$time, y = obj$mcf, mode = "lines",
        line = list(color = clrs$fit, shape = "hv"),
        showlegend = show_in_legend,
        legendgroup = as.character(i),
        name = paste0("MCF", label_suffix),
        text = paste0("MCF: (", round(obj$time, signif), ", ", round(obj$mcf, signif), ")"),
        hoverinfo = "text"
      )

    if (isTRUE(showConf)) {
      p <- p %>%
        add_trace(
          type = "scatter", x = obj$time, y = obj$lower_bounds, mode = "lines",
          line = list(color = clrs$conf, shape = "hv"),
          showlegend = FALSE,
          legendgroup = as.character(i),
          text = paste0("Lower: (", round(obj$time, signif), ", ", round(obj$lower_bounds, signif), ")"),
          hoverinfo = "text"
        ) %>%
        add_trace(
          type = "scatter", x = obj$time, y = obj$upper_bounds, mode = "lines",
          fill = "tonexty", fillcolor = fillcolor,
          line = list(color = clrs$conf, shape = "hv"),
          showlegend = FALSE,
          legendgroup = as.character(i),
          text = paste0("Upper: (", round(obj$time, signif), ", ", round(obj$upper_bounds, signif), ")"),
          hoverinfo = "text"
        )
    }
  }

  p <- p %>%
    layout(
      title = main,
      xaxis = list(
        title = xlab, showline = TRUE, mirror = "ticks",
        showgrid = xgrid, gridcolor = gridCol
      ),
      yaxis = list(
        title = ylab, showline = TRUE, mirror = "ticks",
        showgrid = xgrid, gridcolor = gridCol
      )
    )

  return(p)
}

# Create a mock 'mcf' class object for testing
mock_mcf_obj <- list(
  time         = c(50, 150, 250, 350, 450),
  mcf          = c(0.2, 0.5, 0.9, 1.3, 1.8),
  variance     = c(0.01, 0.03, 0.06, 0.10, 0.15),
  lower_bounds = c(0.1, 0.3, 0.6, 0.9, 1.3),
  upper_bounds = c(0.4, 0.8, 1.3, 1.8, 2.4),
  conf_level   = 0.95,
  n_systems    = 3L,
  n_events     = 9L,
  end_times    = c(A = 500, B = 500, C = 500)
)
class(mock_mcf_obj) <- "mcf"

# Unit tests
test_that("plotly_mcf throws an error for incorrect input class", {
  incorrect_obj <- list(time = 1:5, mcf = 1:5)
  expect_error(plotly_mcf(incorrect_obj), "All inputs must be of class 'mcf'.")
})

test_that("plotly_mcf returns a plotly object", {
  plot <- plotly_mcf(mock_mcf_obj)
  expect_s3_class(plot, "plotly")
})

test_that("plotly_mcf works with default parameters", {
  plot <- plotly_mcf(mock_mcf_obj)
  expect_true(!is.null(plot))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_mcf works with custom color parameters", {
  plot <- plotly_mcf(mock_mcf_obj, fitCol = "blue", confCol = "green", gridCol = "yellow")
  expect_s3_class(plot, "plotly")
})

test_that("plotly_mcf respects the showGrid parameter", {
  plot_with_grid    <- plotly_mcf(mock_mcf_obj, showGrid = TRUE)
  plot_without_grid <- plotly_mcf(mock_mcf_obj, showGrid = FALSE)

  expect_equal(plot_with_grid$x$layoutAttrs[[1]]$xaxis$showgrid, TRUE)
  expect_equal(plot_with_grid$x$layoutAttrs[[1]]$yaxis$showgrid, TRUE)
  expect_equal(plot_without_grid$x$layoutAttrs[[1]]$xaxis$showgrid, FALSE)
  expect_equal(plot_without_grid$x$layoutAttrs[[1]]$yaxis$showgrid, FALSE)
})

test_that("plotly_mcf includes hover text for MCF trace", {
  plot <- plotly_mcf(mock_mcf_obj)
  hover_data <- plot$x$data[[1]]$text
  expect_true(all(grepl("MCF: \\(", hover_data)))
})

test_that("plotly_mcf respects showConf = FALSE", {
  plot_with_conf    <- plotly_mcf(mock_mcf_obj, showConf = TRUE)
  plot_without_conf <- plotly_mcf(mock_mcf_obj, showConf = FALSE)

  # With confidence: MCF + lower + upper = 3 traces
  # Without confidence: MCF = 1 trace
  expect_gt(length(plot_with_conf$x$attrs), length(plot_without_conf$x$attrs))
})

test_that("plotly_mcf accepts a list of mcf objects and returns a plotly object", {
  plot <- plotly_mcf(list(mock_mcf_obj, mock_mcf_obj))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_mcf overlay produces more traces than single object", {
  single  <- plotly_mcf(mock_mcf_obj)
  overlay <- plotly_mcf(list(mock_mcf_obj, mock_mcf_obj))
  expect_gt(length(overlay$x$attrs), length(single$x$attrs))
})

test_that("plotly_mcf overlay respects cols parameter", {
  plot <- plotly_mcf(list(mock_mcf_obj, mock_mcf_obj), cols = c("red", "blue"))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_mcf throws error for non-mcf input in list", {
  expect_error(
    plotly_mcf(list(mock_mcf_obj, list())),
    "All inputs must be of class 'mcf'."
  )
})

#' Interactive Probability Plot.
#'
#' This function creates an interactive probability plot for one or more wblr objects.
#' When a list of objects is provided the models are overlaid on the same plot, each
#' rendered in a distinct color. All objects must use the same distribution family.
#' It can include confidence bounds, suspension data (single object only), and a results table.
#'
#' @param wblr_obj A single object of class 'wblr', or a list of such objects. This is
#' a required argument. Each object must use the same distribution (e.g. all Weibull or
#' all lognormal). Suspension subplots are only shown when a single object is provided.
#' @param susp An optional numeric vector of suspension data. Default is NULL. Ignored
#' when multiple wblr objects are provided.
#' @param showConf Show the confidence bounds (TRUE) or not (FALSE). Default is TRUE if
#' confidence bounds are available in the wblr object.
#' @param showSusp Show the suspensions plot (TRUE) or not (FALSE). Default is TRUE
#' if susp is provided.
#' @param showGrid Show grid (TRUE) or hide grid (FALSE). Default is TRUE.
#' @param main Main title. Default is 'Probability Plot'.
#' @param xlab X-axis label. Default is 'Time to Failure'.
#' @param ylab Y-axis label. Default is 'Probability'.
#' @param probCol Color of the probability values. Default is 'black'. Used only for a
#' single wblr object; ignored when `cols` is provided or multiple objects are supplied.
#' @param fitCol Color of the model fit. Default is 'black'. Used only for a single
#' wblr object; ignored when `cols` is provided or multiple objects are supplied.
#' @param confCol Color of the confidence bounds. Default is 'black'. Used only for a
#' single wblr object; ignored when `cols` is provided or multiple objects are supplied.
#' @param intCol Color of the intervals for interval censored models. Default is 'black'.
#' @param gridCol Color of the grid. Default is 'lightgray'.
#' @param signif Significant digits of results. Default is 3. Must be a positive integer.
#' @param cols Optional character vector of colors, one per wblr object. When provided,
#' each object's data points, fit line, and confidence bounds are all drawn in the
#' corresponding color. Recycled if shorter than the number of objects.
#' @return A `plotly` object representing the interactive probability plot.
#' @details
#' For Weibull distributions the y-axis uses a double-log transformation
#' (\code{log(-log(1 - F))}) so that a straight line represents a perfect
#' Weibull fit. For lognormal distributions the y-axis uses the normal
#' quantile transformation (\code{qnorm(F)}), i.e., standard normal probability
#' paper. Three-parameter Weibull (\code{dist = 'weibull3p'}) shifts the x-axis
#' by the estimated location (gamma) parameter before applying the same
#' transformation.
#'
#' Goodness-of-fit is reported as R² for rank-regression fits or
#' log-likelihood for MLE fits.
#' @seealso [plotly_contour()] for MLE parameter confidence contours.
#' @examples
#' library(WeibullR)
#' library(ReliaPlotR)
#' failures <- c(30, 49, 82, 90, 96)
#' obj <- wblr.conf(wblr.fit(wblr(failures)))
#' plotly_wblr(obj)
#'
#' suspensions <- c(100, 45, 10)
#' obj <- wblr.conf(wblr.fit(wblr(failures, suspensions)))
#' plotly_wblr(obj, suspensions,
#'   fitCol = "blue",
#'   confCol = "blue"
#' )
#' inspection_data <- data.frame(
#'   left = c(0, 6.12, 19.92, 29.64, 35.4, 39.72, 45.32, 52.32),
#'   right = c(6.12, 19.92, 29.64, 35.4, 39.72, 45.32, 52.32, 63.48),
#'   qty = c(5, 16, 12, 18, 18, 2, 6, 17)
#' )
#' suspensions <- data.frame(time = 63.48, event = 0, qty = 73)
#' obj <- wblr(suspensions, interval = inspection_data)
#' obj <- wblr.fit(obj, method.fit = "mle")
#' obj <- wblr.conf(obj, method.conf = "fm", lty = 2)
#' suspensions <- as.vector(suspensions$time)
#' plotly_wblr(obj,
#'   susp = suspensions, fitCol = "red", confCol = "red", intCol = "blue",
#'   main = "Parts Cracking Inspection Interval Analysis",
#'   ylab = "Cumulative % Cracked", xlab = "Inspection Time"
#' )
#' failures <- c(25, 30, 42, 49, 55, 67, 73, 82, 90, 96, 101, 110, 120, 132, 145)
#' fit <- wblr.conf(wblr.fit(wblr(failures), dist = "weibull3p"))
#' plotly_wblr(fit, fitCol = "darkgreen", confCol = "darkgreen")
#'
#' # Overlay two Weibull models
#' obj2 <- wblr.conf(wblr.fit(wblr(c(20, 40, 60, 80, 100)), method.fit = "mle"),
#'                   method.conf = "lrb")
#' obj3 <- wblr.conf(wblr.fit(wblr(c(10, 30, 50, 70, 90)), method.fit = "mle"),
#'                   method.conf = "lrb")
#' plotly_wblr(list(obj2, obj3))
#'
#' @import WeibullR
#' @import plotly
#' @importFrom graphics text
#' @importFrom stats runif qnorm
#' @export

plotly_wblr <- function(wblr_obj,
                        susp = NULL,
                        showConf = TRUE,
                        showSusp = TRUE,
                        showGrid = TRUE,
                        main = "Probability Plot",
                        xlab = "Time to Failure",
                        ylab = "Probability",
                        probCol = "black",
                        fitCol = "black",
                        confCol = "black",
                        intCol = "black",
                        gridCol = "lightgray",
                        signif = 3,
                        cols = NULL) {

  # Normalize to list
  if (!is.list(wblr_obj) || inherits(wblr_obj, "wblr")) {
    wblr_obj <- list(wblr_obj)
  }
  n <- length(wblr_obj)

  if (n == 0) {
    stop("Argument 'wblr_obj' is not of class 'wblr'.")
  }

  # Validate inputs
  lapply(wblr_obj, function(obj) {
    if (!inherits(obj, "wblr")) {
      stop("All inputs must be of class 'wblr'.")
    }
  })
  if (!is.null(susp) && !is.numeric(susp)) {
    stop("Argument 'susp' must be a numeric vector.")
  }
  if (n > 1 && !is.null(susp)) {
    warning("'susp' is ignored when multiple wblr objects are provided.")
    susp <- NULL
  }

  # Require same distribution for overlay
  if (n > 1) {
    dist_types <- sapply(wblr_obj, function(obj) {
      if (is.null(obj$fit)) NA_character_ else obj$fit[[1]]$options$dist
    })
    dist_types <- dist_types[!is.na(dist_types)]
    if (length(unique(dist_types)) > 1) {
      stop("All wblr objects must use the same distribution for overlay.")
    }
  }

  # Per-object colors
  default_palette <- c(
    "#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd",
    "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"
  )
  if (is.null(cols)) {
    if (n == 1) {
      obj_colors <- list(list(prob = probCol, fit = fitCol, conf = confCol))
    } else {
      base_cols <- rep(default_palette, length.out = n)
      obj_colors <- lapply(base_cols, function(col) list(prob = col, fit = col, conf = col))
    }
  } else {
    base_cols <- rep(cols, length.out = n)
    obj_colors <- lapply(base_cols, function(col) list(prob = col, fit = col, conf = col))
  }

  # Get time & probability data
  get_time_data <- function(obj, signif) {
    if (obj$interval == 0) {
      time <- obj$data$dpoints$time
      ints <- NULL
      prob <- obj$data$dpoints$ppp
    } else {
      t1 <- obj$data$dlines$t1
      t2 <- obj$data$dlines$t2
      time <- (t1 + t2) / 2
      ints <- log(t2 - t1)
      prob <- obj$data$dlines$ppp
    }
    list(
      time = time,
      time_sd = round(time, signif),
      ints = ints,
      prob = prob,
      prob_sd = round(prob, signif)
    )
  }

  # Get confidence-bound data
  get_conf_data <- function(obj, signif, showConf) {
    fit <- obj$fit[[1]]
    conf <- fit$conf[[1]]$bounds
    if (is.null(fit) || is.null(conf)) {
      out <- rep(list(NULL), 8)
      names(out) <- c(
        "datum", "unrel", "lower", "upper",
        "datum_sd", "unrel_sd", "lower_sd", "upper_sd"
      )
      return(out)
    }

    datum <- conf$Datum
    unrel <- conf$unrel

    if (!showConf) {
      lower <- upper <- NULL
    } else {
      lower <- conf$Lower
      upper <- conf$Upper
    }

    list(
      datum = datum,
      unrel = unrel,
      lower = lower,
      upper = upper,
      datum_sd = round(datum, signif),
      unrel_sd = round(unrel, signif),
      lower_sd = if (!is.null(lower)) round(lower, signif),
      upper_sd = if (!is.null(upper)) round(upper, signif)
    )
  }

  # Get distribution params & transforms
  get_dist_params <- function(obj, signif, probability, unrel) {
    fit_opts <- obj$fit[[1]]$options
    vec <- as.numeric(obj$fit[[1]]$fit_vec)

    switch(fit_opts$dist,
      lognormal = {
        params <- c("Mulog", "Sigmalog", NULL)
        values <- c(round(vec[1], signif), round(vec[2], signif), NULL)
        prob_tr <- qnorm(probability)
        unrel_tr <- qnorm(unrel)
        list(
          params = params, values = values,
          prob_trans = prob_tr, unrel_trans = unrel_tr
        )
      },
      weibull = ,
      weibull3p = {
        params <- c("Beta", "Eta", if (fit_opts$dist == "weibull3p") "Gamma" else NULL)
        vals <- c(
          round(vec[2], signif), round(vec[1], signif),
          if (fit_opts$dist == "weibull3p") round(vec[3], signif) else NULL
        )
        prob_tr <- log(1 / (1 - probability))
        unrel_tr <- log(1 / (1 - unrel))
        list(
          params = params, values = vals,
          prob_trans = prob_tr, unrel_trans = unrel_tr
        )
      },
      stop("Unsupported distribution: ", fit_opts$dist)
    )
  }

  # Get goodness-of-fit metrics
  get_gof_metrics <- function(obj, signif) {
    fit_opts <- obj$fit[[1]]$options
    gof <- obj$fit[[1]]$gof

    if (is.null(fit_opts$method.fit)) {
      return(list(methlab = NULL, methval = NULL))
    }

    if (fit_opts$method.fit == "rr-xony") {
      list(methlab = "R^2", methval = round(gof$r2, signif))
    } else if (fit_opts$method.fit == "mle") {
      list(methlab = "Loglikelihood", methval = round(gof$loglik, signif))
    } else {
      list(methlab = NULL, methval = NULL)
    }
  }

  # Extract all data from a wblr object
  extract_data <- function(obj, signif = 3, showConf = TRUE) {
    tp <- get_time_data(obj, signif)
    cd <- get_conf_data(obj, signif, showConf)
    dp <- if (is.null(obj$fit)) {
      list(params = NULL, values = NULL, prob_trans = NULL, unrel_trans = NULL)
    } else {
      get_dist_params(obj, signif, tp$prob, cd$unrel)
    }
    gm <- if (is.null(obj$fit)) {
      list(methlab = NULL, methval = NULL)
    } else {
      get_gof_metrics(obj, signif)
    }
    c(tp, cd, dp, gm)
  }

  # Y-axis ticks (in %)
  yticks <- c(
    0.000001, 0.00001, 0.0001, 0.001, 0.01, 0.05, 0.1, 0.2, 0.5, 1,
    2, 5, 10, 20, 50, 90, 99, 99.999
  )

  # Axis transform based on distribution type (use first object)
  first_obj <- wblr_obj[[1]]
  dist_type <- if (!is.null(first_obj$fit)) first_obj$fit[[1]]$options$dist else "weibull"
  if (dist_type == "lognormal") {
    yticks_trans <- qnorm(yticks / 100)
    yaxis_scale <- "linear"
  } else {
    yticks_trans <- log(1 / (1 - yticks / 100))
    yaxis_scale <- "log"
  }

  show_grid <- is.null(showGrid) || isTRUE(showGrid)

  # Extract first object's data now (needed for susp subplot x-range)
  first_data <- extract_data(first_obj, signif, showConf)

  # Build multi-object probability plot
  p <- plot_ly()
  show_in_legend <- n > 1

  for (i in seq_along(wblr_obj)) {
    obj <- wblr_obj[[i]]
    clrs <- obj_colors[[i]]
    data <- extract_data(obj, signif, showConf)
    fillcolor <- plotly::toRGB(clrs$conf, 0.2)
    label_suffix <- if (n > 1) paste0(" ", i) else ""
    grp <- as.character(i)

    p <- p %>%
      add_trace(
        x = data$time, y = data$prob_trans, type = "scatter", mode = "markers",
        marker = list(color = clrs$prob),
        showlegend = show_in_legend,
        legendgroup = grp,
        error_x = list(array = ~ data$ints, color = intCol),
        name = paste0("Data", label_suffix),
        text = ~ paste0("Probability: (", data$time_sd, ", ", data$prob_sd, ")"),
        hoverinfo = "text"
      ) %>%
      add_trace(
        x = data$datum, y = data$unrel_trans, type = "scatter", mode = "markers+lines",
        marker = list(color = "transparent"),
        line = list(color = clrs$fit),
        showlegend = show_in_legend,
        legendgroup = grp,
        name = paste0("Fit", label_suffix),
        text = ~ paste0("Fit: (", data$datum_sd, ", ", data$unrel_sd, ")"),
        hoverinfo = "text"
      )

    if (!is.null(data$lower)) {
      p <- p %>%
        add_trace(
          x = data$lower, y = data$unrel_trans, type = "scatter", mode = "markers+lines",
          marker = list(color = "transparent"),
          line = list(color = clrs$conf),
          showlegend = FALSE,
          legendgroup = grp,
          text = ~ paste0("Lower: (", data$lower_sd, ", ", data$unrel_sd, ")"),
          hoverinfo = "text"
        ) %>%
        add_trace(
          x = data$upper, y = data$unrel_trans, type = "scatter", mode = "markers+lines",
          fill = "tonexty", fillcolor = fillcolor,
          marker = list(color = "transparent"),
          line = list(color = clrs$conf),
          showlegend = FALSE,
          legendgroup = grp,
          text = ~ paste0("Upper: (", data$upper_sd, ", ", data$unrel_sd, ")"),
          hoverinfo = "text"
        )
    }
  }

  # Compute x range from first object's datum (used for both main plot and susp)
  log_datum <- log10(first_data$datum)
  xmin <- min(log_datum)
  xmax <- max(log_datum)

  p <- p %>%
    layout(
      title = main,
      xaxis = list(
        type = "log", title = xlab, showline = TRUE, mirror = "ticks",
        showgrid = show_grid, gridcolor = gridCol,
        range = list(xmin, xmax)
      ),
      yaxis = list(
        type = yaxis_scale, title = ylab, showline = TRUE, mirror = "ticks",
        showgrid = show_grid, gridcolor = gridCol,
        tickvals = yticks_trans, ticktext = yticks
      )
    )

  # Suspension subplot — single object only
  plot_susp <- function() {
    if (!showSusp || is.null(susp)) {
      return(NULL)
    }
    susp_sd <- round(susp, signif)
    rand_y <- stats::runif(length(susp))
    plot_ly(
      x = susp, y = rand_y, type = "scatter", mode = "markers",
      marker = list(color = obj_colors[[1]]$prob), showlegend = FALSE,
      text = ~ paste0("Suspension: ", susp_sd), hoverinfo = "text"
    ) %>%
      layout(
        xaxis = list(
          type = "log", title = "", zeroline = FALSE, showline = TRUE,
          mirror = "ticks", showticklabels = FALSE, showgrid = FALSE,
          range = list(xmin, xmax)
        ),
        yaxis = list(
          title = "", zeroline = FALSE, showline = TRUE,
          mirror = "ticks", showticklabels = FALSE, showgrid = FALSE
        )
      )
  }

  # Combine main plot with optional suspension subplot
  combine_plots <- function(prob_plot, susp_plot) {
    clean_subplot <- function(...) {
      plot <- subplot(...)
      plot$x$layout <- plot$x$layout[grep("NA", names(plot$x$layout), invert = TRUE)]
      plot
    }
    if (is.null(susp_plot)) {
      return(prob_plot)
    }
    clean_subplot(prob_plot, susp_plot, nrows = 2, titleX = TRUE, titleY = TRUE) %>%
      layout(
        xaxis = list(domain = c(0, 1)),
        xaxis2 = list(domain = c(0, 1)),
        yaxis = list(domain = c(0, 0.875)),
        yaxis2 = list(domain = c(0.9, 1))
      )
  }

  p2 <- plot_susp()
  combine_plots(p, p2)
}

test_that("plotly_wblr works with valid wblr object", {
  failures <- c(30, 49, 82, 90, 96)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
  p <- plotly_wblr(obj)
  expect_s3_class(p, "plotly")
})

test_that("plotly_wblr works with missing optional parameters", {
  failures <- c(30, 49, 82, 90, 96)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
  p <- plotly_wblr(obj, showConf = FALSE)
  expect_s3_class(p, "plotly")
})

test_that("plotly_wblr stops with invalid wblr object", {
  expect_error(plotly_wblr(list()), "Argument 'wblr_obj' is not of class 'wblr'.")
})

test_that("plotly_wblr stops with invalid susp argument", {
  failures <- c(30, 49, 82, 90, 96)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
  expect_error(plotly_wblr(obj, susp = "invalid"), "Argument 'susp' must be a numeric vector.")
})

test_that("plotly_wblr works with susp argument", {
  failures <- c(30, 49, 82, 90, 96)
  suspensions <- c(100, 150)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
  p <- plotly_wblr(obj, susp = suspensions)
  expect_s3_class(p, "plotly")
})

test_that("plotly_wblr generates plot with confidence intervals", {
  failures <- c(30, 49, 82, 90, 96)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
  p <- plotly_wblr(obj, showConf = TRUE)
  expect_s3_class(p, "plotly")
})

test_that("plotly_wblr generates plot without confidence intervals", {
  failures <- c(30, 49, 82, 90, 96)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
  p <- plotly_wblr(obj, showConf = FALSE)
  expect_s3_class(p, "plotly")
})

test_that("plotly_wblr works with different colors", {
  failures <- c(30, 49, 82, 90, 96)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
  p <- plotly_wblr(obj, probCol = "red", fitCol = "blue", confCol = "green", intCol = "purple", gridCol = "grey")
  expect_s3_class(p, "plotly")
})

test_that("plotly_wblr works with showGrid parameter", {
  failures <- c(30, 49, 82, 90, 96)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
  p <- plotly_wblr(obj, showGrid = TRUE)
  expect_s3_class(p, "plotly")
})

test_that("plotly_wblr handles subLayouts correctly", {
  failures <- c(30, 49, 82, 90, 96)
  suspensions <- c(100, 150)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")

  p1 <- plotly_wblr(obj, showSusp = TRUE)
  expect_s3_class(p1, "plotly")

  p2 <- plotly_wblr(obj, showSusp = FALSE)
  expect_s3_class(p2, "plotly")
})

test_that("plotly_wblr accepts a list of wblr objects and returns a plotly object", {
  failures1 <- c(30, 49, 82, 90, 96)
  failures2 <- c(20, 40, 60, 80, 100)
  obj1 <- wblr.conf(wblr.fit(wblr(failures1), method.fit = "mle"), method.conf = "lrb")
  obj2 <- wblr.conf(wblr.fit(wblr(failures2), method.fit = "mle"), method.conf = "lrb")
  plot <- plotly_wblr(list(obj1, obj2))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_wblr overlay produces more traces than single object", {
  failures1 <- c(30, 49, 82, 90, 96)
  failures2 <- c(20, 40, 60, 80, 100)
  obj1 <- wblr.conf(wblr.fit(wblr(failures1), method.fit = "mle"), method.conf = "lrb")
  obj2 <- wblr.conf(wblr.fit(wblr(failures2), method.fit = "mle"), method.conf = "lrb")
  single <- plotly_wblr(obj1)
  overlay <- plotly_wblr(list(obj1, obj2))
  expect_gt(length(overlay$x$attrs), length(single$x$attrs))
})

test_that("plotly_wblr overlay respects cols parameter", {
  failures1 <- c(30, 49, 82, 90, 96)
  failures2 <- c(20, 40, 60, 80, 100)
  obj1 <- wblr.conf(wblr.fit(wblr(failures1), method.fit = "mle"), method.conf = "lrb")
  obj2 <- wblr.conf(wblr.fit(wblr(failures2), method.fit = "mle"), method.conf = "lrb")
  plot <- plotly_wblr(list(obj1, obj2), cols = c("red", "blue"))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_wblr throws error when mixing distribution types", {
  failures <- c(30, 49, 82, 90, 96)
  obj_weibull <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
  obj_lognormal <- wblr.conf(
    wblr.fit(wblr(failures), method.fit = "mle", dist = "lognormal"),
    method.conf = "lrb"
  )
  expect_error(
    plotly_wblr(list(obj_weibull, obj_lognormal)),
    "All wblr objects must use the same distribution for overlay."
  )
})

test_that("plotly_wblr throws error for non-wblr input in list", {
  failures <- c(30, 49, 82, 90, 96)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
  expect_error(
    plotly_wblr(list(obj, list())),
    "All inputs must be of class 'wblr'."
  )
})

test_that("plotly_wblr warns and ignores susp when multiple objects provided", {
  failures1 <- c(30, 49, 82, 90, 96)
  failures2 <- c(20, 40, 60, 80, 100)
  obj1 <- wblr.conf(wblr.fit(wblr(failures1), method.fit = "mle"), method.conf = "lrb")
  obj2 <- wblr.conf(wblr.fit(wblr(failures2), method.fit = "mle"), method.conf = "lrb")
  expect_warning(
    plotly_wblr(list(obj1, obj2), susp = c(110, 120)),
    "'susp' is ignored"
  )
})

test_that("plotly_wblr works with lognormal distribution", {
  failures <- c(30, 49, 82, 90, 96)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle", dist = "lognormal"),
                   method.conf = "lrb")
  p <- plotly_wblr(obj)
  expect_s3_class(p, "plotly")
})

test_that("plotly_wblr works with weibull3p distribution", {
  failures <- c(25, 30, 42, 49, 55, 67, 73, 82, 90, 96, 101, 110, 120, 132, 145)
  obj <- wblr.conf(wblr.fit(wblr(failures), dist = "weibull3p"))
  p <- plotly_wblr(obj)
  expect_s3_class(p, "plotly")
})

test_that("plotly_wblr works with interval-censored data", {
  inspection_data <- data.frame(
    left  = c(0,    6.12, 19.92, 29.64, 35.4,  39.72, 45.32, 52.32),
    right = c(6.12, 19.92, 29.64, 35.4, 39.72, 45.32, 52.32, 63.48),
    qty   = c(5L, 16L, 12L, 18L, 18L, 2L, 6L, 17L)
  )
  suspensions <- data.frame(time = 63.48, event = 0, qty = 73)
  obj <- wblr(suspensions, interval = inspection_data)
  obj <- wblr.fit(obj, method.fit = "mle")
  obj <- wblr.conf(obj, method.conf = "fm")
  p <- plotly_wblr(obj)
  expect_s3_class(p, "plotly")
})

test_that("plotly_wblr hover text labels are correct for confidence bounds", {
  failures <- c(30, 49, 82, 90, 96)
  obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
  p <- plotly_wblr(obj, showConf = TRUE)

  # Build to resolve lazy attrs into data
  built <- plotly::plotly_build(p)

  # Trace 3 is the lower bound, trace 4 is the upper bound
  lower_hover <- built$x$data[[3]]$text
  upper_hover <- built$x$data[[4]]$text
  expect_true(all(grepl("^Lower:", lower_hover)))
  expect_true(all(grepl("^Upper:", upper_hover)))
})

#' Interactive NHPP Plot.
#'
#' The function creates an interactive Non-Homogeneous Poisson Process (NHPP)
#' plot for one or more `nhpp` objects. When a list of objects is provided the
#' models are overlaid on the same plot, each rendered in a distinct color.
#' The plot shows the nonparametric Mean Cumulative Function (MCF) alongside
#' the parametric model fit and optional confidence bounds. Vertical lines
#' indicate change points if breakpoints are specified in the nhpp object.
#'
#' @param nhpp_obj An object of class 'nhpp', or a list of such objects. Each
#' object is created using the `nhpp()` function from the `ReliaGrowR` package.
#' @param showConf Show the confidence bounds (TRUE) or not (FALSE). Default is TRUE.
#' @param showGrid Show grid (TRUE) or hide grid (FALSE). Default is TRUE.
#' @param main Main title. Default is "NHPP Plot".
#' @param xlab X-axis label. Default is "Cumulative Time".
#' @param ylab Y-axis label. Default is "Mean Cumulative Function".
#' @param pointCol Color of the MCF data points. Default is "black". Used only
#' for a single nhpp object; ignored when `cols` is provided or multiple objects
#' are supplied.
#' @param fitCol Color of the model fit. Default is "black". Used only for a
#' single nhpp object; ignored when `cols` is provided or multiple objects are
#' supplied.
#' @param confCol Color of the confidence bounds. Default is "black". Used only
#' for a single nhpp object; ignored when `cols` is provided or multiple objects
#' are supplied.
#' @param gridCol Color of the grid. Default is "lightgray".
#' @param breakCol Color of the breakpoints. Default is "black". Used only for a
#' single nhpp object; ignored when `cols` is provided or multiple objects are
#' supplied.
#' @param signif Significant digits of results. Default is 3. Must be a positive integer.
#' @param cols Optional character vector of colors, one per nhpp object. When
#' provided, each object's points, fit line, confidence bounds, and breakpoints
#' are all drawn in the corresponding color. Recycled if shorter than the number
#' of objects.
#' @return A `plotly` object representing the interactive NHPP plot.
#' @details
#' The nonparametric MCF is overlaid with the fitted parametric Power Law
#' NHPP model, \eqn{E[N(t)] = \lambda t^\beta}. Use this plot to assess
#' whether the parametric model fits the observed event history and to
#' identify change points. Confidence bounds are based on the Fisher matrix
#' approximation of the fitted parameters. For piecewise models, vertical
#' dotted lines mark each breakpoint.
#' @seealso [plotly_mcf()] for the nonparametric MCF alone; [plotly_rga()] for
#' a Crow-AMSAA cumulative-failures view.
#' @examples
#' library(ReliaGrowR)
#' times <- c(100, 200, 300, 400, 500)
#' events <- c(1, 2, 1, 3, 2)
#' fit <- nhpp(time = times, event = events)
#' plotly_nhpp(fit)
#'
#' # Piecewise model with a breakpoint
#' times2 <- c(100, 200, 300, 400, 500, 600, 700, 800, 900, 1000)
#' events2 <- c(1, 2, 1, 1, 1, 2, 3, 1, 2, 4)
#' fit2 <- nhpp(time = times2, event = events2, breaks = 500, method = "LS")
#' plotly_nhpp(fit2, breakCol = "red")
#'
#' # Overlay two models
#' plotly_nhpp(list(fit, fit2))
#' @import ReliaGrowR plotly
#' @importFrom graphics text
#' @importFrom stats runif qnorm
#' @export

plotly_nhpp <- function(nhpp_obj,
                        showConf = TRUE,
                        showGrid = TRUE,
                        main = "NHPP Plot",
                        xlab = "Cumulative Time",
                        ylab = "Mean Cumulative Function",
                        pointCol = "black",
                        fitCol = "black",
                        confCol = "black",
                        gridCol = "lightgray",
                        breakCol = "black",
                        signif = 3,
                        cols = NULL) {

  # Normalize to list
  if (!is.list(nhpp_obj) || inherits(nhpp_obj, "nhpp")) {
    nhpp_obj <- list(nhpp_obj)
  }
  n <- length(nhpp_obj)

  # Validate inputs
  lapply(nhpp_obj, function(obj) {
    if (!inherits(obj, "nhpp")) {
      stop("All inputs must be of class 'nhpp'.")
    }
  })

  # Set up per-object colors
  default_palette <- c(
    "#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd",
    "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"
  )
  if (is.null(cols)) {
    if (n == 1) {
      obj_colors <- list(list(point = pointCol, fit = fitCol, conf = confCol, brk = breakCol))
    } else {
      base_cols <- rep(default_palette, length.out = n)
      obj_colors <- lapply(base_cols, function(col) {
        list(point = col, fit = col, conf = col, brk = col)
      })
    }
  } else {
    base_cols <- rep(cols, length.out = n)
    obj_colors <- lapply(base_cols, function(col) {
      list(point = col, fit = col, conf = col, brk = col)
    })
  }

  xgrid <- ifelse(is.null(showGrid) || isTRUE(showGrid), TRUE, FALSE)

  vline <- function(x = 0, color = "black") {
    list(
      type = "line",
      y0 = 0, y1 = 1, yref = "paper",
      x0 = x, x1 = x,
      line = list(color = color, dash = "dot")
    )
  }

  # Build plot
  p <- plot_ly()
  all_shapes <- list()

  for (i in seq_along(nhpp_obj)) {
    obj <- nhpp_obj[[i]]
    clrs <- obj_colors[[i]]
    fillcolor <- plotly::toRGB(clrs$conf, 0.2)

    times <- obj$time
    mcf   <- obj$cum_events
    fitted <- obj$fitted_values

    label_suffix <- if (n > 1) paste0(" ", i) else ""
    show_in_legend <- n > 1

    p <- p %>%
      add_trace(
        x = times, y = mcf, type = "scatter", mode = "markers",
        marker = list(color = clrs$point),
        showlegend = show_in_legend,
        legendgroup = as.character(i),
        name = paste0("MCF", label_suffix),
        text = paste0("MCF: (", round(times, signif), ", ", round(mcf, signif), ")"),
        hoverinfo = "text"
      ) %>%
      add_trace(
        type = "scatter", x = times, y = fitted, mode = "markers+lines",
        marker = list(color = "transparent"), line = list(color = clrs$fit),
        showlegend = show_in_legend,
        legendgroup = as.character(i),
        name = paste0("Fit", label_suffix),
        text = paste0("Fit: (", round(times, signif), ", ", round(fitted, signif), ")"),
        hoverinfo = "text"
      )

    if (isTRUE(showConf)) {
      p <- p %>%
        add_trace(
          type = "scatter", x = times, y = obj$upper_bounds, mode = "markers+lines",
          marker = list(color = "transparent"), line = list(color = clrs$conf),
          showlegend = FALSE,
          legendgroup = as.character(i),
          text = paste0("Upper: (", round(times, signif), ", ", round(obj$upper_bounds, signif), ")"),
          hoverinfo = "text"
        ) %>%
        add_trace(
          type = "scatter", x = times, y = obj$lower_bounds, mode = "markers+lines",
          fill = "tonexty", fillcolor = fillcolor,
          marker = list(color = "transparent"), line = list(color = clrs$conf),
          showlegend = FALSE,
          legendgroup = as.character(i),
          text = paste0("Lower: (", round(times, signif), ", ", round(obj$lower_bounds, signif), ")"),
          hoverinfo = "text"
        )
    }

    # Collect breakpoint shapes
    if (!is.null(obj$breakpoints) && length(obj$breakpoints) > 0) {
      bp_times <- exp(as.numeric(obj$breakpoints))
      for (bp in bp_times) {
        all_shapes <- c(all_shapes, list(vline(bp, color = clrs$brk)))
      }
    }
  }

  p <- p %>%
    layout(
      title = main,
      xaxis = list(
        title = xlab, showline = TRUE, mirror = "ticks",
        showgrid = xgrid, gridcolor = gridCol
      ),
      yaxis = list(
        title = ylab, showline = TRUE, mirror = "ticks",
        showgrid = xgrid, gridcolor = gridCol
      ),
      shapes = all_shapes
    )

  return(p)
}

# Create mock 'nhpp' class objects for testing
mock_nhpp_obj <- list(
  time          = c(100, 200, 300, 400, 500),
  event         = c(1, 2, 1, 3, 2),
  cum_events    = c(1, 3, 4, 7, 9),
  fitted_values = c(1.2, 3.1, 4.5, 7.2, 9.1),
  lower_bounds  = c(0.8, 2.5, 3.8, 6.1, 7.8),
  upper_bounds  = c(1.8, 4.0, 5.5, 8.6, 10.7),
  breakpoints   = NULL,
  model_type    = "Power Law",
  method        = "MLE",
  conf_level    = 0.95
)
class(mock_nhpp_obj) <- "nhpp"

mock_nhpp_bp <- list(
  time          = c(100, 200, 300, 400, 500),
  event         = c(1, 2, 1, 3, 2),
  cum_events    = c(1, 3, 4, 7, 9),
  fitted_values = c(1.2, 3.1, 4.5, 7.2, 9.1),
  lower_bounds  = c(0.8, 2.5, 3.8, 6.1, 7.8),
  upper_bounds  = c(1.8, 4.0, 5.5, 8.6, 10.7),
  breakpoints   = log(c(200, 400)),
  model_type    = "Power Law",
  method        = "MLE",
  conf_level    = 0.95
)
class(mock_nhpp_bp) <- "nhpp"

# Unit tests
test_that("plotly_nhpp throws an error for incorrect input class", {
  incorrect_obj <- list(time = 1:5, cum_events = 1:5)
  expect_error(plotly_nhpp(incorrect_obj), "All inputs must be of class 'nhpp'.")
})

test_that("plotly_nhpp returns a plotly object", {
  plot <- plotly_nhpp(mock_nhpp_obj)
  expect_s3_class(plot, "plotly")
})

test_that("plotly_nhpp works with default parameters", {
  plot <- plotly_nhpp(mock_nhpp_obj)
  expect_true(!is.null(plot))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_nhpp works with custom color parameters", {
  plot <- plotly_nhpp(mock_nhpp_obj,
                      pointCol = "red", fitCol = "blue",
                      confCol = "green", gridCol = "yellow", breakCol = "purple")
  expect_s3_class(plot, "plotly")
})

test_that("plotly_nhpp respects the showGrid parameter", {
  plot_with_grid    <- plotly_nhpp(mock_nhpp_obj, showGrid = TRUE)
  plot_without_grid <- plotly_nhpp(mock_nhpp_obj, showGrid = FALSE)

  expect_equal(plot_with_grid$x$layoutAttrs[[1]]$xaxis$showgrid, TRUE)
  expect_equal(plot_with_grid$x$layoutAttrs[[1]]$yaxis$showgrid, TRUE)
  expect_equal(plot_without_grid$x$layoutAttrs[[1]]$xaxis$showgrid, FALSE)
  expect_equal(plot_without_grid$x$layoutAttrs[[1]]$yaxis$showgrid, FALSE)
})

test_that("plotly_nhpp includes hover text for data points", {
  plot <- plotly_nhpp(mock_nhpp_obj)
  hover_data <- plot$x$data[[1]]$text
  expect_true(all(grepl("MCF: \\(", hover_data)))
})

test_that("plotly_nhpp respects showConf = FALSE", {
  plot_with_conf    <- plotly_nhpp(mock_nhpp_obj, showConf = TRUE)
  plot_without_conf <- plotly_nhpp(mock_nhpp_obj, showConf = FALSE)

  # With confidence: data + fit + lower + upper = 4 traces
  # Without confidence: data + fit = 2 traces
  expect_gt(length(plot_with_conf$x$attrs), length(plot_without_conf$x$attrs))
})

test_that("plotly_nhpp renders breakpoint shapes", {
  plot_no_bp <- plotly_nhpp(mock_nhpp_obj)
  plot_bp    <- plotly_nhpp(mock_nhpp_bp)

  shapes_no_bp <- plot_no_bp$x$layoutAttrs[[1]]$shapes
  shapes_bp    <- plot_bp$x$layoutAttrs[[1]]$shapes

  expect_equal(length(shapes_no_bp), 0)
  expect_equal(length(shapes_bp), 2)
})

test_that("plotly_nhpp accepts a list of nhpp objects and returns a plotly object", {
  plot <- plotly_nhpp(list(mock_nhpp_obj, mock_nhpp_obj))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_nhpp overlay produces more traces than single object", {
  single  <- plotly_nhpp(mock_nhpp_obj)
  overlay <- plotly_nhpp(list(mock_nhpp_obj, mock_nhpp_obj))
  expect_gt(length(overlay$x$attrs), length(single$x$attrs))
})

test_that("plotly_nhpp overlay respects cols parameter", {
  plot <- plotly_nhpp(list(mock_nhpp_obj, mock_nhpp_obj), cols = c("red", "blue"))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_nhpp throws error for non-nhpp input in list", {
  expect_error(
    plotly_nhpp(list(mock_nhpp_obj, list())),
    "All inputs must be of class 'nhpp'."
  )
})

#' Interactive ALT Life-Stress Relationship Plot.
#'
#' Creates an interactive life-stress relationship (Arrhenius or Power Law) plot
#' for an `alt` object. Displays the characteristic-life estimates per stress
#' level, the fitted relationship line, optional percentile bands with shading,
#' and an optional goal-condition marker. The `alt` object must have been
#' processed through [WeibullR.ALT::alt.fit()] before passing to this function.
#'
#' @param alt_obj An object of class `'alt'` created by the `WeibullR.ALT` package
#'   and fitted with `alt.fit()`.
#' @param showPerc Show percentile lines with shading (TRUE) or not (FALSE).
#'   Default is TRUE.
#' @param showGoal Show the goal-condition marker when one is present in `alt_obj`
#'   (TRUE) or not (FALSE). Default is TRUE.
#' @param showGrid Show grid (TRUE) or hide grid (FALSE). Default is TRUE.
#' @param main Main title. Default is "Life-Stress Relationship".
#' @param xlab X-axis label. Default is "Stress".
#' @param ylab Y-axis label. Default is "Time to Failure".
#' @param fitCol Color of the fitted relationship line. Default is "red".
#' @param ptCol Color of the characteristic-life scatter points. Default is "black".
#' @param percCol Color of the percentile lines and fill. Default is "blue".
#' @param goalCol Color of the goal-condition marker. Default is "orange".
#' @param gridCol Color of the grid. Default is "lightgray".
#' @param signif Significant digits for hover text. Default is 3.
#' @param percentiles Numeric vector of percentiles to draw as reference lines
#'   with shading. Sorted internally; shading is filled between adjacent lines.
#'   Default is `c(10, 90)`.
#' @return A `plotly` object representing the interactive life-stress plot.
#' @details
#' Two life-stress models are supported:
#' \itemize{
#'   \item \strong{Arrhenius}: \eqn{\eta = A \exp(E_a / (k T))}, where stress is
#'     temperature in Kelvin and the x-axis uses a reciprocal scale.
#'   \item \strong{Power Law}: \eqn{\eta = A / S^n}, where the x-axis is
#'     log-transformed.
#' }
#' The \code{percentiles} argument controls which quantile lines are drawn
#' with shading between adjacent lines. The goal-condition marker
#' (\code{showGoal}) is drawn when the \code{alt} object contains a
#' \code{$goal} specification from \code{alt.make()}.
#' @seealso [plotly_alt()] for the ALT probability plot using the same \code{alt} object.
#' @examples
#' library(WeibullR.ALT)
#' d1 <- alt.data(c(248, 456, 528, 731, 813, 537), stress = 300)
#' d2 <- alt.data(c(164, 176, 289), stress = 350)
#' d3 <- alt.data(c(88, 112, 152), stress = 400)
#' obj <- alt.fit(
#'   alt.parallel(
#'     alt.make(list(d1, d2, d3), dist = "weibull", alt.model = "arrhenius", view_dist_fits = FALSE),
#'     view_parallel_fits = FALSE
#'   )
#' )
#' plotly_rel(obj)
#' @import WeibullR.ALT
#' @import plotly
#' @importFrom stats qweibull qlnorm
#' @export

plotly_rel <- function(alt_obj,
                       showPerc    = TRUE,
                       showGoal    = TRUE,
                       showGrid    = TRUE,
                       main        = "Life-Stress Relationship",
                       xlab        = "Stress",
                       ylab        = "Time to Failure",
                       fitCol      = "red",
                       ptCol       = "black",
                       percCol     = "blue",
                       goalCol     = "orange",
                       gridCol     = "lightgray",
                       signif      = 3,
                       percentiles = c(10, 90)) {

  if (!inherits(alt_obj, "alt")) {
    stop("Argument 'alt_obj' is not of class 'alt'.")
  }
  if (is.null(alt_obj$alt_coef)) {
    stop("'alt_obj' must be fitted with alt.fit() before plotting.")
  }

  pp          <- alt_obj$parallel_par
  coef        <- alt_obj$alt_coef
  dist_type   <- alt_obj$dist
  model_type  <- alt_obj$alt.model
  P2          <- pp$P2[1]

  show_grid <- is.null(showGrid) || isTRUE(showGrid)

  # Characteristic-life values for the scatter points
  # Weibull: P1 = eta (already in time units)
  # Lognormal: P1 = log(median), convert to median life
  stress_pts <- pp$stress
  life_pts   <- if (dist_type == "lognormal") exp(pp$P1) else pp$P1

  # Fitted relationship line over the stress range
  stress_range <- seq(min(stress_pts), max(stress_pts), length.out = 200)
  fit_y <- if (model_type == "power") {
    exp(coef[2] * log(stress_range) + coef[1])
  } else {
    exp(coef[2] * stress_range + coef[1])
  }

  # X-axis type: log for Power Law, linear for Arrhenius
  xaxis_type <- if (model_type == "power") "log" else "linear"

  # Shading fill color (semi-transparent)
  perc_fill <- plotly::toRGB(percCol, 0.2)

  # Build plot
  p <- plot_ly()

  # Characteristic-life scatter
  p <- p %>%
    add_trace(
      x = stress_pts, y = life_pts,
      type = "scatter", mode = "markers",
      marker = list(color = ptCol, symbol = "x", size = 10),
      name = if (dist_type == "lognormal") "Median Life" else "Char. Life",
      text = paste0(
        "Stress: ", round(stress_pts, signif),
        ", Life: ", round(life_pts, signif)
      ),
      hoverinfo = "text"
    ) %>%
    # Fitted line
    add_trace(
      x = stress_range, y = fit_y,
      type = "scatter", mode = "lines",
      line = list(color = fitCol, width = 2),
      name = "Fit",
      text = paste0(
        "Stress: ", round(stress_range, signif),
        ", Life: ", round(fit_y, signif)
      ),
      hoverinfo = "text"
    )

  # Percentile lines with shading between adjacent pairs
  if (isTRUE(showPerc) && length(percentiles) > 0) {
    percs_sorted <- sort(percentiles)

    for (j in seq_along(percs_sorted)) {
      perc  <- percs_sorted[j]
      quant <- if (dist_type == "weibull") {
        qweibull(perc / 100, shape = P2, scale = 1)
      } else {
        qlnorm(perc / 100, meanlog = 0, sdlog = P2)
      }
      perc_y <- fit_y * quant

      # Fill to the previous trace for all but the first percentile
      use_fill <- j > 1

      p <- p %>%
        add_trace(
          x = stress_range, y = perc_y,
          type = "scatter", mode = "lines",
          fill      = if (use_fill) "tonexty" else "none",
          fillcolor = if (use_fill) perc_fill  else NULL,
          line = list(color = percCol, width = 1),
          name = paste0(perc, "th %ile"),
          text = paste0(
            "Stress: ", round(stress_range, signif),
            ", ", perc, "th %ile: ", round(perc_y, signif)
          ),
          hoverinfo = "text"
        )
    }
  }

  # Goal-condition marker
  if (isTRUE(showGoal) && !is.null(alt_obj$goal)) {
    goal_stress <- alt_obj$goal$stress
    goal_life   <- if (model_type == "power") {
      exp(coef[2] * log(goal_stress) + coef[1])
    } else {
      exp(coef[2] * goal_stress + coef[1])
    }
    if (dist_type == "lognormal") goal_life <- exp(goal_life)
    p <- p %>%
      add_trace(
        x = goal_stress, y = goal_life,
        type = "scatter", mode = "markers",
        marker = list(color = goalCol, symbol = "star", size = 14),
        name = "Goal",
        text = paste0(
          "Goal Stress: ", round(goal_stress, signif),
          ", Life: ", round(goal_life, signif)
        ),
        hoverinfo = "text"
      )
  }

  p %>%
    layout(
      title = main,
      xaxis = list(
        type = xaxis_type, title = xlab, showline = TRUE, mirror = "ticks",
        showgrid = show_grid, gridcolor = gridCol
      ),
      yaxis = list(
        type = "log", title = ylab, showline = TRUE, mirror = "ticks",
        showgrid = show_grid, gridcolor = gridCol
      )
    )
}

# Shared mock 'alt' object for relationship plot tests.
# Must have gone through alt.fit() — needs $alt_coef and $parallel_par.
mock_alt_rel <- list(
  data = list(
    list(
      stress    = 300,
      data      = data.frame(left  = c(248, 456, 528),
                             right = c(248, 456, 528),
                             qty   = c(1L, 1L, 1L)),
      num_fails = 3,
      valid_set = TRUE
    ),
    list(
      stress    = 350,
      data      = data.frame(left  = c(164, 176, 289),
                             right = c(164, 176, 289),
                             qty   = c(1L, 1L, 1L)),
      num_fails = 3,
      valid_set = TRUE
    )
  ),
  dist         = "weibull",
  alt.model    = "arrhenius",
  method.fit   = "mle-rba",
  parallel_par = data.frame(P1     = c(615.97, 231.86),
                             P2     = c(3.94, 3.94),
                             stress = c(300, 350),
                             wt     = c(3L, 3L)),
  alt_coef     = c(11.206, -0.01606),
  goal         = NULL
)
class(mock_alt_rel) <- "alt"

# Power Law variant
mock_alt_power <- mock_alt_rel
mock_alt_power$alt.model <- "power"

# Lognormal variant
mock_alt_lnorm_rel <- mock_alt_rel
mock_alt_lnorm_rel$dist         <- "lognormal"
mock_alt_lnorm_rel$parallel_par <- data.frame(
  P1     = log(c(615.97, 231.86)),
  P2     = c(0.55, 0.55),
  stress = c(300, 350),
  wt     = c(3L, 3L)
)

# Variant with a goal condition
mock_alt_goal <- mock_alt_rel
mock_alt_goal$goal <- list(
  stress = 250,
  data   = data.frame(left = numeric(0), right = numeric(0), qty = integer(0))
)

# --- Tests -------------------------------------------------------------------

test_that("plotly_rel throws error for incorrect input class", {
  expect_error(plotly_rel(list()), "not of class 'alt'")
})

test_that("plotly_rel throws error when alt_coef is missing", {
  obj <- mock_alt_rel
  obj$alt_coef <- NULL
  expect_error(plotly_rel(obj), "alt.fit\\(\\)")
})

test_that("plotly_rel returns a plotly object", {
  p <- plotly_rel(mock_alt_rel)
  expect_s3_class(p, "plotly")
})

test_that("plotly_rel has more traces when showPerc = TRUE than FALSE", {
  p_with  <- plotly_rel(mock_alt_rel, showPerc = TRUE)
  p_without <- plotly_rel(mock_alt_rel, showPerc = FALSE)
  expect_gt(length(p_with$x$attrs), length(p_without$x$attrs))
})

test_that("plotly_rel showPerc = FALSE has fewer traces than showPerc = TRUE", {
  p_with    <- plotly_rel(mock_alt_rel, showPerc = TRUE)
  p_without <- plotly_rel(mock_alt_rel, showPerc = FALSE)
  # showPerc=TRUE adds one trace per percentile vs. showPerc=FALSE
  expect_gt(length(p_with$x$attrs), length(p_without$x$attrs))
})

test_that("plotly_rel custom percentiles change trace count", {
  p_two  <- plotly_rel(mock_alt_rel, percentiles = c(10, 90))
  p_one  <- plotly_rel(mock_alt_rel, percentiles = 50)
  # 2 percentile traces vs 1
  expect_gt(length(p_two$x$attrs), length(p_one$x$attrs))
})

test_that("plotly_rel showGoal adds a trace when goal is present", {
  p_no_goal <- plotly_rel(mock_alt_goal, showGoal = FALSE)
  p_goal    <- plotly_rel(mock_alt_goal, showGoal = TRUE)
  expect_gt(length(p_goal$x$attrs), length(p_no_goal$x$attrs))
})

test_that("plotly_rel uses log x-axis for Power Law model", {
  p <- plotly_rel(mock_alt_power)
  expect_equal(p$x$layoutAttrs[[1]]$xaxis$type, "log")
})

test_that("plotly_rel uses linear x-axis for Arrhenius model", {
  p <- plotly_rel(mock_alt_rel)
  expect_equal(p$x$layoutAttrs[[1]]$xaxis$type, "linear")
})

test_that("plotly_rel y-axis is always log scale", {
  p <- plotly_rel(mock_alt_rel)
  expect_equal(p$x$layoutAttrs[[1]]$yaxis$type, "log")
})

test_that("plotly_rel works with lognormal distribution", {
  p <- plotly_rel(mock_alt_lnorm_rel)
  expect_s3_class(p, "plotly")
})

test_that("plotly_rel respects showGrid = FALSE", {
  p <- plotly_rel(mock_alt_rel, showGrid = FALSE)
  expect_equal(p$x$layoutAttrs[[1]]$xaxis$showgrid, FALSE)
  expect_equal(p$x$layoutAttrs[[1]]$yaxis$showgrid, FALSE)
})

test_that("plotly_rel respects custom main, xlab, ylab", {
  p <- plotly_rel(mock_alt_rel, main = "My Rel", xlab = "Stress", ylab = "Life")
  expect_equal(p$x$layoutAttrs[[1]]$title, "My Rel")
  expect_equal(p$x$layoutAttrs[[1]]$xaxis$title, "Stress")
  expect_equal(p$x$layoutAttrs[[1]]$yaxis$title, "Life")
})

test_that("plotly_rel accepts custom color parameters", {
  p <- plotly_rel(mock_alt_rel,
                  fitCol  = "steelblue",
                  ptCol   = "tomato",
                  percCol = "forestgreen",
                  goalCol = "purple",
                  gridCol = "lightblue")
  expect_s3_class(p, "plotly")
})

#' Interactive Contour Plot
#'
#' This function creates an interactive contour plot for one or more
#' `wblr` objects, each assumed to have confidence contours generated via
#' `method.conf = 'lrb'`. The function overlays all contours in a single plot and displays
#' their respective MLE point estimates.
#'
#' @param wblr_obj A single `wblr` object or a list of `wblr` objects. Each object
#' must have contours generated using `method.conf = 'lrb'`.
#' @param main Main title for the plot.
#' @param xlab X-axis label (typically Eta or Sigmalog).
#' @param ylab Y-axis label (typically Beta or Mulog).
#' @param showGrid Logical; whether to show grid lines (default TRUE).
#' @param cols Optional vector of colors for each contour/estimate pair. If not provided,
#'   colors are chosen from a default palette.
#' @param gridCol Color of the grid lines (default 'lightgray').
#' @param signif Number of significant digits to display for estimates and contour coordinates.
#' Defaults to 3.
#' @return A `plotly` object representing the interactive contour plot.
#' @details
#' Confidence contours require a likelihood-ratio-based (\code{method.conf = 'lrb'})
#' MLE fit (\code{method.fit = 'mle'}). The contour traces the locus of
#' (\eqn{\eta}, \eqn{\beta}) pairs whose log-likelihood falls within a
#' chi-squared critical value of the MLE peak, giving a joint confidence
#' region for the two Weibull parameters. The MLE point estimate is shown as
#' a marker at the center of each contour.
#' @seealso [plotly_wblr()] for probability plots using the same \code{wblr} objects.
#'
#' @examples
#' library(WeibullR)
#' library(ReliaPlotR)
#'
#' failures1 <- c(30, 49, 82, 90, 96)
#' failures2 <- c(20, 40, 60, 80, 100)
#' obj1 <- wblr.conf(wblr.fit(wblr(failures1), method.fit = "mle"), method.conf = "lrb")
#' obj2 <- wblr.conf(wblr.fit(wblr(failures2), method.fit = "mle"), method.conf = "lrb")
#' plotly_contour(list(obj1, obj2), main = "Overlayed Contours")
#'
#' @importFrom plotly plot_ly add_trace layout toRGB
#' @export
plotly_contour <- function(wblr_obj,
                           main = "Contour Plot",
                           xlab = "Eta",
                           ylab = "Beta",
                           showGrid = TRUE,
                           cols = NULL,
                           gridCol = "lightgray",
                           signif = 3) {
  # Handle input as a list if not already
  if (!is.list(wblr_obj) || inherits(wblr_obj, "wblr")) {
    wblr_obj <- list(wblr_obj)
  }

  # Validate all inputs
  lapply(wblr_obj, function(obj) {
    if (!inherits(obj, "wblr")) {
      stop("All inputs must be of class 'wblr'.")
    }
    if (!identical(obj$fit[[1]]$conf[[1]]$options$method.conf, "lrb")) {
      stop("Each wblr object must have contours generated using method.conf='lrb'.")
    }
  })

  # Set colors
  if (is.null(cols)) {
    default_colors <- c(
      "#1f77b4", "#ff7f0e", "#2ca02c", "#d62728",
      "#9467bd", "#8c564b", "#e377c2", "#7f7f7f",
      "#bcbd22", "#17becf"
    )
    cols <- rep(default_colors, length.out = length(wblr_obj))
  }

  # Start plotly object
  p <- plot_ly()

  for (i in seq_along(wblr_obj)) {
    obj <- wblr_obj[[i]]
    col <- cols[i]

    dist_type <- obj$fit[[1]]$options$dist
    fit_vec <- as.numeric(obj$fit[[1]]$fit_vec)
    contour <- obj$fit[[1]]$conf[[1]]$contour

    if (dist_type %in% c("weibull", "weibull3p")) {
      param1 <- "Beta"
      param2 <- "Eta"
      val1 <- round(fit_vec[2], signif)
      val2 <- round(fit_vec[1], signif)
    } else if (dist_type == "lognormal") {
      param1 <- "Mulog"
      param2 <- "Sigmalog"
      val1 <- round(fit_vec[1], signif)
      val2 <- round(fit_vec[2], signif)
    } else {
      next
    }

    fillcolor <- toRGB(col, 0.2)

    # Add contour
    p <- p %>%
      add_trace(
        x = round(contour[[1]], signif),
        y = round(contour[[2]], signif),
        type = "scatter",
        mode = "lines",
        fill = "toself",
        fillcolor = fillcolor,
        line = list(color = col),
        name = paste("Contour", i),
        hoverinfo = "text",
        text = ~ paste0("Contour (", round(contour[[1]], signif), ", ", round(contour[[2]], signif), ")")
      ) %>%
      add_trace(
        x = val2,
        y = val1,
        type = "scatter",
        mode = "markers",
        marker = list(color = col, size = 10, symbol = "x"),
        name = paste("Estimate", i),
        hoverinfo = "text",
        text = ~ paste0("Estimate: (", val2, ", ", val1, ")")
      )
  }

  # Add layout
  p <- p %>%
    layout(
      title = main,
      xaxis = list(title = xlab, showgrid = showGrid, gridcolor = gridCol),
      yaxis = list(title = ylab, showgrid = showGrid, gridcolor = gridCol)
    )

  return(p)
}

# Shared fixtures
failures  <- c(30, 49, 82, 90, 96)
failures2 <- c(20, 40, 60, 80, 100)
wblr_obj  <- wblr.conf(wblr.fit(wblr(failures),  method.fit = "mle"), method.conf = "lrb")
wblr_obj2 <- wblr.conf(wblr.fit(wblr(failures2), method.fit = "mle"), method.conf = "lrb")
wblr_lnorm <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle", dist = "lognormal"),
                         method.conf = "lrb")

test_that("plotly_contour runs without errors", {
  expect_silent(plotly_contour(wblr_obj))
})

test_that("plotly_contour returns a plotly object", {
  plot <- plotly_contour(wblr_obj)
  expect_s3_class(plot, "plotly")
})

test_that("plotly_contour handles invalid wblr_obj", {
  invalid_wblr_obj <- list(a = 1, b = 2)
  expect_error(plotly_contour(invalid_wblr_obj), "All inputs must be of class 'wblr'.")
})

test_that("plotly_contour handles wblr_obj without contours", {
  no_contour_wblr_obj <- wblr(failures)
  expect_error(plotly_contour(no_contour_wblr_obj), "Each wblr object must have contours generated using method.conf='lrb'.")
})

test_that("plotly_contour accepts a list of wblr objects", {
  plot <- plotly_contour(list(wblr_obj, wblr_obj2))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_contour overlay produces more traces than single object", {
  single  <- plotly_contour(wblr_obj)
  overlay <- plotly_contour(list(wblr_obj, wblr_obj2))
  expect_gt(length(overlay$x$attrs), length(single$x$attrs))
})

test_that("plotly_contour overlay respects cols parameter", {
  plot <- plotly_contour(list(wblr_obj, wblr_obj2), cols = c("red", "blue"))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_contour works with lognormal distribution", {
  plot <- plotly_contour(wblr_lnorm)
  expect_s3_class(plot, "plotly")
})

test_that("plotly_contour respects showGrid = FALSE", {
  plot <- plotly_contour(wblr_obj, showGrid = FALSE)
  expect_equal(plot$x$layoutAttrs[[1]]$xaxis$showgrid, FALSE)
  expect_equal(plot$x$layoutAttrs[[1]]$yaxis$showgrid, FALSE)
})

test_that("plotly_contour respects custom main, xlab, ylab", {
  plot <- plotly_contour(wblr_obj, main = "My Contour", xlab = "Scale", ylab = "Shape")
  expect_equal(plot$x$layoutAttrs[[1]]$title, "My Contour")
  expect_equal(plot$x$layoutAttrs[[1]]$xaxis$title, "Scale")
  expect_equal(plot$x$layoutAttrs[[1]]$yaxis$title, "Shape")
})

test_that("plotly_contour signif parameter rounds estimate hover text", {
  plot  <- plotly_contour(wblr_obj, signif = 2)
  built <- plotly::plotly_build(plot)
  # Second trace is the MLE estimate marker
  est_text <- built$x$data[[2]]$text
  expect_true(grepl("^Estimate:", est_text))
})

#' ReliaPlotR: Interactive Reliability Plots with plotly
#'
#' ReliaPlotR creates interactive reliability probability plots using
#' \href{https://plotly.com/r/}{plotly}. It wraps the \pkg{WeibullR},
#' \pkg{WeibullR.ALT}, and \pkg{ReliaGrowR} packages and provides a consistent
#' interface for four reliability analysis domains:
#'
#' \describe{
#'   \item{Life Data Analysis}{[plotly_wblr()], [plotly_contour()]}
#'   \item{Accelerated Life Testing}{[plotly_alt()], [plotly_rel()]}
#'   \item{Reliability Growth}{[plotly_rga()], [plotly_duane()]}
#'   \item{Repairable Systems}{[plotly_nhpp()], [plotly_mcf()], [plotly_exposure()]}
#' }
#'
#' All functions accept a single fitted model object or a list of objects
#' (for overlay plots) and return a \code{plotly} object.
#'
#' @seealso
#' Vignettes:
#' \itemize{
#'   \item \code{vignette("weibull", package = "ReliaPlotR")} — Life Data Analysis
#'   \item \code{vignette("alt", package = "ReliaPlotR")} — Accelerated Life Testing
#'   \item \code{vignette("rga", package = "ReliaPlotR")} — Reliability Growth Analysis
#'   \item \code{vignette("repairable", package = "ReliaPlotR")} — Repairable Systems Analysis
#' }
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

#' Extract Tidy Parameter Estimates from a wblr Object.
#'
#' Returns a tidy data frame of distribution parameter estimates, goodness-of-fit
#' metrics, and (optionally) confidence bound data from one or more fitted `wblr`
#' objects. This makes it straightforward to use ReliaPlotR output in reproducible
#' research workflows — export to CSV, include in tables, or compare multiple fits.
#'
#' @param wblr_obj A single object of class `'wblr'`, or a list of such objects.
#'   Each object must have been fitted with [WeibullR::wblr.fit()].
#' @return A named list with two elements:
#'   \describe{
#'     \item{`estimates`}{A `data.frame` with one row per `wblr` object and
#'       columns `dist`, `method_fit`, `param1_name`, `param1`, `param2_name`,
#'       `param2`, `param3_name`, `param3`, `gof_metric`, `gof_value`,
#'       `method_conf`, `n_failures`.}
#'     \item{`bounds`}{For a single object, a `data.frame` of confidence bound
#'       data (columns `Datum`, `unrel`, `Lower`, `Upper`), or `NULL` if no
#'       confidence bounds were computed. For a list of objects, a list of such
#'       data frames (one per object).}
#'   }
#' @details
#' Parameters are returned on their natural scales. For Weibull and
#' three-parameter Weibull (\code{weibull3p}) models, \code{param1} is the
#' shape parameter \eqn{\beta} and \code{param2} is the scale parameter
#' \eqn{\eta}; \code{param3} is the location parameter \eqn{\gamma} for
#' \code{weibull3p} (NA otherwise). For lognormal models, \code{param1} is
#' \eqn{\mu_{log}} and \code{param2} is \eqn{\sigma_{log}}.
#'
#' Goodness-of-fit is reported as R\eqn{^2} (column value \code{"R2"}) for
#' rank-regression fits, or log-likelihood (\code{"loglikelihood"}) for MLE
#' fits.
#' @references
#' Meeker, W. Q., and Escobar, L. A. (1998). \emph{Statistical Methods for
#' Reliability Data}. Wiley.
#' @seealso [plotly_wblr()] for the corresponding interactive probability plot.
#' @examples
#' library(WeibullR)
#' failures <- c(30, 49, 82, 90, 96)
#' obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
#' result <- tidy_wblr(obj)
#' result$estimates
#' result$bounds
#'
#' # List of objects
#' obj2 <- wblr.conf(wblr.fit(wblr(c(20, 40, 60, 80, 100)), method.fit = "mle"),
#'                   method.conf = "lrb")
#' result2 <- tidy_wblr(list(obj, obj2))
#' result2$estimates
#' @import WeibullR
#' @export

tidy_wblr <- function(wblr_obj) {
  if (!is.list(wblr_obj) || inherits(wblr_obj, "wblr")) {
    wblr_obj <- list(wblr_obj)
  }

  if (length(wblr_obj) == 0) {
    stop("Argument 'wblr_obj' is not of class 'wblr'.")
  }
  lapply(wblr_obj, function(obj) {
    if (!inherits(obj, "wblr")) stop("All inputs must be of class 'wblr'.")
  })

  extract_one <- function(obj) {
    fit <- obj$fit[[1]]
    if (is.null(fit)) stop("wblr object has not been fitted. Run wblr.fit() first.")

    vec        <- as.numeric(fit$fit_vec)
    dist       <- fit$options$dist
    method_fit <- fit$options$method.fit

    if (dist == "lognormal") {
      p1_name <- "Mulog";    p1 <- vec[1]
      p2_name <- "Sigmalog"; p2 <- vec[2]
      p3_name <- NA_character_; p3 <- NA_real_
    } else if (dist %in% c("weibull", "weibull3p")) {
      p1_name <- "Beta"; p1 <- vec[2]
      p2_name <- "Eta";  p2 <- vec[1]
      if (dist == "weibull3p" && length(vec) >= 3) {
        p3_name <- "Gamma"; p3 <- vec[3]
      } else {
        p3_name <- NA_character_; p3 <- NA_real_
      }
    } else {
      stop("Unsupported distribution: ", dist)
    }

    gof <- fit$gof
    if (!is.null(method_fit) && method_fit == "rr-xony") {
      gof_metric <- "R2";            gof_value <- gof$r2
    } else if (!is.null(method_fit) && method_fit == "mle") {
      gof_metric <- "loglikelihood"; gof_value <- gof$loglik
    } else {
      gof_metric <- NA_character_;   gof_value <- NA_real_
    }

    method_conf <- tryCatch(
      fit$conf[[1]]$options$method.conf,
      error = function(e) NA_character_
    )
    if (is.null(method_conf)) method_conf <- NA_character_

    n_failures <- if (isTRUE(obj$interval == 0)) {
      length(obj$data$dpoints$time)
    } else {
      nrow(obj$data$dlines)
    }

    data.frame(
      dist        = dist,
      method_fit  = if (is.null(method_fit)) NA_character_ else method_fit,
      param1_name = p1_name, param1 = p1,
      param2_name = p2_name, param2 = p2,
      param3_name = p3_name, param3 = p3,
      gof_metric  = gof_metric, gof_value = gof_value,
      method_conf = method_conf,
      n_failures  = n_failures,
      stringsAsFactors = FALSE
    )
  }

  estimates <- do.call(rbind, lapply(wblr_obj, extract_one))
  rownames(estimates) <- NULL

  bounds_list <- lapply(wblr_obj, function(obj) {
    tryCatch(obj$fit[[1]]$conf[[1]]$bounds, error = function(e) NULL)
  })

  bounds <- if (length(wblr_obj) == 1) bounds_list[[1]] else bounds_list

  list(estimates = estimates, bounds = bounds)
}

failures <- c(30, 49, 82, 90, 96)
obj_mle <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
obj_rr  <- wblr.conf(wblr.fit(wblr(failures)))

test_that("tidy_wblr stops for non-wblr input", {
  expect_error(tidy_wblr(list()), "not of class 'wblr'")
})

test_that("tidy_wblr stops for unfitted wblr object", {
  expect_error(tidy_wblr(wblr(failures)), "wblr.fit")
})

test_that("tidy_wblr returns a list with estimates and bounds", {
  result <- tidy_wblr(obj_mle)
  expect_type(result, "list")
  expect_named(result, c("estimates", "bounds"))
})

test_that("tidy_wblr estimates is a data frame", {
  result <- tidy_wblr(obj_mle)
  expect_s3_class(result$estimates, "data.frame")
})

test_that("tidy_wblr estimates has correct columns", {
  result <- tidy_wblr(obj_mle)
  expected_cols <- c(
    "dist", "method_fit",
    "param1_name", "param1", "param2_name", "param2",
    "param3_name", "param3",
    "gof_metric", "gof_value", "method_conf", "n_failures"
  )
  expect_true(all(expected_cols %in% names(result$estimates)))
})

test_that("tidy_wblr extracts Weibull Beta and Eta for MLE", {
  result <- tidy_wblr(obj_mle)
  est <- result$estimates
  expect_equal(est$dist, "weibull")
  expect_equal(est$param1_name, "Beta")
  expect_equal(est$param2_name, "Eta")
  expect_true(is.numeric(est$param1) && est$param1 > 0)
  expect_true(is.numeric(est$param2) && est$param2 > 0)
})

test_that("tidy_wblr reports loglikelihood GoF for MLE", {
  result <- tidy_wblr(obj_mle)
  expect_equal(result$estimates$gof_metric, "loglikelihood")
  expect_true(is.numeric(result$estimates$gof_value))
})

test_that("tidy_wblr reports R2 GoF for rank regression", {
  result <- tidy_wblr(obj_rr)
  expect_equal(result$estimates$gof_metric, "R2")
  expect_true(result$estimates$gof_value >= 0 && result$estimates$gof_value <= 1)
})

test_that("tidy_wblr bounds is a data frame for single object", {
  result <- tidy_wblr(obj_mle)
  expect_s3_class(result$bounds, "data.frame")
})

test_that("tidy_wblr n_failures matches input", {
  result <- tidy_wblr(obj_mle)
  expect_equal(result$estimates$n_failures, length(failures))
})

test_that("tidy_wblr works with lognormal distribution", {
  obj_ln <- wblr.conf(wblr.fit(wblr(failures), dist = "lognormal", method.fit = "mle"),
                      method.conf = "fm")
  result  <- tidy_wblr(obj_ln)
  est     <- result$estimates
  expect_equal(est$dist, "lognormal")
  expect_equal(est$param1_name, "Mulog")
  expect_equal(est$param2_name, "Sigmalog")
})

test_that("tidy_wblr works with weibull3p distribution", {
  f3 <- c(25, 30, 42, 49, 55, 67, 73, 82, 90, 96, 101, 110, 120, 132, 145)
  obj3p <- wblr.conf(wblr.fit(wblr(f3), dist = "weibull3p"))
  result  <- tidy_wblr(obj3p)
  est     <- result$estimates
  expect_equal(est$dist, "weibull3p")
  expect_equal(est$param3_name, "Gamma")
  expect_false(is.na(est$param3))
})

test_that("tidy_wblr with list returns one row per object", {
  obj2   <- wblr.conf(wblr.fit(wblr(c(20, 40, 60, 80, 100)), method.fit = "mle"),
                      method.conf = "lrb")
  result <- tidy_wblr(list(obj_mle, obj2))
  expect_equal(nrow(result$estimates), 2)
})

test_that("tidy_wblr with list returns list of bounds", {
  obj2   <- wblr.conf(wblr.fit(wblr(c(20, 40, 60, 80, 100)), method.fit = "mle"),
                      method.conf = "lrb")
  result <- tidy_wblr(list(obj_mle, obj2))
  expect_type(result$bounds, "list")
  expect_length(result$bounds, 2)
})

#' Extract Tidy Fitted Values from an rga Object.
#'
#' Returns a tidy data frame of fitted cumulative failure counts, confidence
#' bounds, and (when available) Crow-AMSAA model coefficients from an `rga`
#' object. Suitable for exporting results for use in reproducible research
#' workflows.
#'
#' @param rga_obj An object of class `'rga'`, or a list of such objects. Each
#'   object is created using the [ReliaGrowR::rga()] function.
#' @return A named list with two elements:
#'   \describe{
#'     \item{`fitted`}{For a single object, a `data.frame` with columns
#'       `time`, `cum_failures`, `fitted`, `lower`, `upper`. For a list, a
#'       list of such data frames (one per object).}
#'     \item{`params`}{For a single object, a one-row `data.frame` with columns
#'       `lambda` and `beta` (Crow-AMSAA Power Law parameters), or `NULL` if
#'       the model coefficients cannot be extracted. For a list, a list of such
#'       data frames.}
#'   }
#' @details
#' The Crow-AMSAA (NHPP Power Law) model gives the expected cumulative
#' failures as \eqn{E[N(t)] = \lambda t^\beta}. A \eqn{\beta < 1} indicates
#' reliability growth (decreasing failure rate); \eqn{\beta > 1} indicates
#' degradation. The parameters are recovered from the fitted log-log linear
#' model via \code{coef()}: \eqn{\lambda = \exp(\text{intercept})} and
#' \eqn{\beta} is the slope coefficient.
#' @references
#' Crow, L. H. (1974). Reliability Analysis for Complex Repairable Systems.
#' In \emph{Reliability and Biometry}, SIAM, pp. 379-410.
#' @seealso [plotly_rga()] for the corresponding interactive reliability growth plot.
#' @examples
#' library(ReliaGrowR)
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' obj <- rga(times, failures)
#' result <- tidy_rga(obj)
#' result$fitted
#' result$params
#' @import ReliaGrowR
#' @importFrom stats coef
#' @export

tidy_rga <- function(rga_obj) {
  if (!is.list(rga_obj) || inherits(rga_obj, "rga")) {
    rga_obj <- list(rga_obj)
  }

  if (length(rga_obj) == 0) stop("Argument 'rga_obj' is not of class 'rga'.")
  lapply(rga_obj, function(obj) {
    if (!inherits(obj, "rga")) stop("All inputs must be of class 'rga'.")
  })

  extract_fitted <- function(obj) {
    data.frame(
      time         = exp(obj$model$model$log_times),
      cum_failures = exp(obj$model$model$log_cum_failures),
      fitted       = obj$fitted_values,
      lower        = obj$lower_bounds,
      upper        = obj$upper_bounds,
      stringsAsFactors = FALSE
    )
  }

  extract_params <- function(obj) {
    tryCatch({
      cf <- coef(obj$model)
      data.frame(lambda = exp(cf[1]), beta = cf[2],
                 stringsAsFactors = FALSE, row.names = NULL)
    }, error = function(e) NULL)
  }

  fitted_list <- lapply(rga_obj, extract_fitted)
  params_list <- lapply(rga_obj, extract_params)

  if (length(rga_obj) == 1) {
    list(fitted = fitted_list[[1]], params = params_list[[1]])
  } else {
    list(fitted = fitted_list, params = params_list)
  }
}

mock_rga_obj <- list(
  model = list(model = list(
    log_times        = log(c(100, 200, 300, 400, 500)),
    log_cum_failures = log(c(1, 3, 6, 10, 15))
  )),
  fitted_values = c(1.5, 3.5, 6.5, 10.5, 15.5),
  lower_bounds  = c(1.2, 3.2, 6.2, 10.2, 15.2),
  upper_bounds  = c(1.8, 3.8, 6.8, 10.8, 15.8),
  breakpoints   = c(200, 400)
)
class(mock_rga_obj) <- "rga"

test_that("tidy_rga stops for non-rga input", {
  expect_error(tidy_rga(list()), "class 'rga'")
})

test_that("tidy_rga returns a list with fitted and params", {
  result <- tidy_rga(mock_rga_obj)
  expect_type(result, "list")
  expect_named(result, c("fitted", "params"))
})

test_that("tidy_rga fitted is a data frame", {
  result <- tidy_rga(mock_rga_obj)
  expect_s3_class(result$fitted, "data.frame")
})

test_that("tidy_rga fitted has correct columns", {
  result <- tidy_rga(mock_rga_obj)
  expect_true(all(c("time", "cum_failures", "fitted", "lower", "upper") %in%
                    names(result$fitted)))
})

test_that("tidy_rga fitted has correct number of rows", {
  result <- tidy_rga(mock_rga_obj)
  expect_equal(nrow(result$fitted), 5)
})

test_that("tidy_rga recovers time on original scale", {
  result <- tidy_rga(mock_rga_obj)
  expect_equal(result$fitted$time, c(100, 200, 300, 400, 500))
})

test_that("tidy_rga recovers cum_failures on original scale", {
  result <- tidy_rga(mock_rga_obj)
  expect_equal(result$fitted$cum_failures, c(1, 3, 6, 10, 15))
})

test_that("tidy_rga fitted values match mock", {
  result <- tidy_rga(mock_rga_obj)
  expect_equal(result$fitted$fitted, c(1.5, 3.5, 6.5, 10.5, 15.5))
})

test_that("tidy_rga bounds match mock", {
  result <- tidy_rga(mock_rga_obj)
  expect_equal(result$fitted$lower, c(1.2, 3.2, 6.2, 10.2, 15.2))
  expect_equal(result$fitted$upper, c(1.8, 3.8, 6.8, 10.8, 15.8))
})

test_that("tidy_rga params is NULL or data frame", {
  result <- tidy_rga(mock_rga_obj)
  expect_true(is.null(result$params) || is.data.frame(result$params))
})

test_that("tidy_rga with list returns list of fitted data frames", {
  result <- tidy_rga(list(mock_rga_obj, mock_rga_obj))
  expect_type(result$fitted, "list")
  expect_length(result$fitted, 2)
  expect_s3_class(result$fitted[[1]], "data.frame")
})

test_that("tidy_rga with list returns list of params", {
  result <- tidy_rga(list(mock_rga_obj, mock_rga_obj))
  expect_type(result$params, "list")
  expect_length(result$params, 2)
})

test_that("tidy_rga throws error for non-rga input in list", {
  expect_error(tidy_rga(list(mock_rga_obj, list())), "class 'rga'")
})

#' Interactive Exposure Plot.
#'
#' The function creates an interactive exposure plot for one or more \code{exposure}
#' objects. When a list of objects is provided the estimates are overlaid on the
#' same plot, each rendered in a distinct color. The plot shows the instantaneous
#' event rate (events per unit time per system at risk) as a step function,
#' calculated from recurrence data by dividing the event count in each interval
#' by the total system-time at risk during that interval.
#'
#' @param exposure_obj An object of class 'exposure', or a list of such objects.
#' Each object is created using the `exposure()` function from the `ReliaGrowR`
#' package.
#' @param showGrid Show grid (TRUE) or hide grid (FALSE). Default is TRUE.
#' @param main Main title. Default is "Exposure Plot".
#' @param xlab X-axis label. Default is "Time".
#' @param ylab Y-axis label. Default is "Event Rate".
#' @param fitCol Color of the event rate step function. Default is "black". Used
#' only for a single exposure object; ignored when `cols` is provided or multiple
#' objects are supplied.
#' @param gridCol Color of the grid. Default is "lightgray".
#' @param signif Significant digits of results. Default is 3. Must be a positive integer.
#' @param cols Optional character vector of colors, one per exposure object. When
#' provided, each object's step function is drawn in the corresponding color.
#' Recycled if shorter than the number of objects.
#' @return A `plotly` object representing the interactive exposure plot.
#' @details
#' Unlike [plotly_mcf()] which shows cumulative events, the exposure plot
#' shows the instantaneous event rate: the number of events per unit time
#' per system at risk in each interval. A flat exposure plot suggests a
#' constant event rate (homogeneous Poisson process); a declining rate
#' suggests improvement; a rising rate suggests degradation. Use this plot
#' alongside [plotly_mcf()] to diagnose time-dependence in the event process.
#' @seealso [plotly_mcf()] for the cumulative view; [plotly_nhpp()] for a
#' parametric NHPP model.
#' @examples
#' library(ReliaGrowR)
#' ids <- c("A", "A", "A", "B", "B", "C", "C", "C", "C")
#' times <- c(50, 150, 350, 100, 300, 80, 200, 320, 450)
#' fit <- exposure(id = ids, time = times)
#' plotly_exposure(fit)
#'
#' # Overlay two exposure objects
#' fit2 <- exposure(id = c("X", "X", "Y"), time = c(60, 220, 180))
#' plotly_exposure(list(fit, fit2), cols = c("steelblue", "tomato"))
#' @import ReliaGrowR plotly
#' @importFrom graphics text
#' @importFrom stats runif qnorm
#' @export

plotly_exposure <- function(exposure_obj,
                            showGrid = TRUE,
                            main = "Exposure Plot",
                            xlab = "Time",
                            ylab = "Event Rate",
                            fitCol = "black",
                            gridCol = "lightgray",
                            signif = 3,
                            cols = NULL) {

  # Normalize to list
  if (!is.list(exposure_obj) || inherits(exposure_obj, "exposure")) {
    exposure_obj <- list(exposure_obj)
  }
  n <- length(exposure_obj)

  # Validate inputs
  lapply(exposure_obj, function(obj) {
    if (!inherits(obj, "exposure")) {
      stop("All inputs must be of class 'exposure'.")
    }
  })

  # Set up per-object colors
  default_palette <- c(
    "#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd",
    "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"
  )
  if (is.null(cols)) {
    if (n == 1) {
      obj_colors <- list(fitCol)
    } else {
      obj_colors <- as.list(rep(default_palette, length.out = n))
    }
  } else {
    obj_colors <- as.list(rep(cols, length.out = n))
  }

  xgrid <- ifelse(is.null(showGrid) || isTRUE(showGrid), TRUE, FALSE)

  # Build plot
  p <- plot_ly()

  for (i in seq_along(exposure_obj)) {
    obj <- exposure_obj[[i]]
    col <- obj_colors[[i]]

    label_suffix <- if (n > 1) paste0(" ", i) else ""
    show_in_legend <- n > 1

    p <- p %>%
      add_trace(
        type = "scatter", x = obj$time, y = obj$event_rate, mode = "lines",
        line = list(color = col, shape = "hv"),
        showlegend = show_in_legend,
        legendgroup = as.character(i),
        name = paste0("Event Rate", label_suffix),
        text = paste0("Rate: (", round(obj$time, signif), ", ", round(obj$event_rate, signif), ")"),
        hoverinfo = "text"
      )
  }

  p <- p %>%
    layout(
      title = main,
      xaxis = list(
        title = xlab, showline = TRUE, mirror = "ticks",
        showgrid = xgrid, gridcolor = gridCol
      ),
      yaxis = list(
        title = ylab, showline = TRUE, mirror = "ticks",
        showgrid = xgrid, gridcolor = gridCol
      )
    )

  return(p)
}

# Create a mock 'exposure' class object for testing
mock_exposure_obj <- list(
  time           = c(50, 150, 250, 350, 450),
  n_at_risk      = c(5L, 5L, 4L, 3L, 3L),
  cum_exposure   = c(250, 750, 1150, 1450, 1750),
  cum_events     = c(1, 3, 5, 7, 9),
  event_rate     = c(0.004, 0.004, 0.00435, 0.00483, 0.00514),
  total_exposure = 1750,
  total_events   = 9L,
  n_systems      = 5L,
  end_times      = c(A = 500, B = 500, C = 500, D = 400, E = 500)
)
class(mock_exposure_obj) <- "exposure"

# Unit tests
test_that("plotly_exposure throws an error for incorrect input class", {
  incorrect_obj <- list(time = 1:5, event_rate = 1:5)
  expect_error(plotly_exposure(incorrect_obj), "All inputs must be of class 'exposure'.")
})

test_that("plotly_exposure returns a plotly object", {
  plot <- plotly_exposure(mock_exposure_obj)
  expect_s3_class(plot, "plotly")
})

test_that("plotly_exposure works with default parameters", {
  plot <- plotly_exposure(mock_exposure_obj)
  expect_true(!is.null(plot))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_exposure works with custom color parameters", {
  plot <- plotly_exposure(mock_exposure_obj, fitCol = "blue", gridCol = "yellow")
  expect_s3_class(plot, "plotly")
})

test_that("plotly_exposure respects the showGrid parameter", {
  plot_with_grid    <- plotly_exposure(mock_exposure_obj, showGrid = TRUE)
  plot_without_grid <- plotly_exposure(mock_exposure_obj, showGrid = FALSE)

  expect_equal(plot_with_grid$x$layoutAttrs[[1]]$xaxis$showgrid, TRUE)
  expect_equal(plot_with_grid$x$layoutAttrs[[1]]$yaxis$showgrid, TRUE)
  expect_equal(plot_without_grid$x$layoutAttrs[[1]]$xaxis$showgrid, FALSE)
  expect_equal(plot_without_grid$x$layoutAttrs[[1]]$yaxis$showgrid, FALSE)
})

test_that("plotly_exposure includes hover text for event rate trace", {
  plot <- plotly_exposure(mock_exposure_obj)
  hover_data <- plot$x$data[[1]]$text
  expect_true(all(grepl("Rate: \\(", hover_data)))
})

test_that("plotly_exposure accepts a list of exposure objects and returns a plotly object", {
  plot <- plotly_exposure(list(mock_exposure_obj, mock_exposure_obj))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_exposure overlay produces more traces than single object", {
  single  <- plotly_exposure(mock_exposure_obj)
  overlay <- plotly_exposure(list(mock_exposure_obj, mock_exposure_obj))
  expect_gt(length(overlay$x$attrs), length(single$x$attrs))
})

test_that("plotly_exposure overlay respects cols parameter", {
  plot <- plotly_exposure(list(mock_exposure_obj, mock_exposure_obj), cols = c("red", "blue"))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_exposure throws error for non-exposure input in list", {
  expect_error(
    plotly_exposure(list(mock_exposure_obj, list())),
    "All inputs must be of class 'exposure'."
  )
})

#' Launch a ReliaPlotR MCP Server.
#'
#' Starts a Model Context Protocol (MCP) server that exposes five reliability
#' analysis tools to MCP clients such as Claude Code and Claude Desktop.
#'
#' \describe{
#'   \item{`fit_weibull`}{Fits a Weibull or lognormal distribution to
#'     time-to-failure data (optionally with right-censored suspensions) and
#'     returns parameter estimates and goodness-of-fit metrics.}
#'   \item{`fit_alt`}{Fits an Accelerated Life Test (ALT) model across
#'     multiple stress levels and returns per-stress-level parameters and the
#'     global life-stress relationship coefficients.}
#'   \item{`plot_weibull`}{Fits a Weibull or lognormal distribution and returns
#'     a Weibull probability plot as a plotly JSON string.}
#'   \item{`plot_alt`}{Fits an ALT model and returns probability and life-stress
#'     relationship plots as plotly JSON strings.}
#'   \item{`plot_rga`}{Fits a Crow-AMSAA reliability growth model and returns a
#'     cumulative failure plot as a plotly JSON string.}
#' }
#'
#' @param ... Additional arguments passed to [mcptools::mcp_server()], such as
#'   `type = "stdio"` (default) or `type = "http"`.
#' @return Called for its side effect of launching a blocking MCP server
#'   process.
#' @details
#' Both `mcptools` (>= 0.2.0) and `ellmer` must be installed. They are listed
#' in `Suggests` and are not automatically installed with ReliaPlotR.
#' `jsonlite` is also required for the `fit_alt` tool.
#'
#' **Registering with Claude Code:**
#' ```bash
#' claude mcp add -s user reliapltr -- \
#'   Rscript -e "ReliaPlotR::reliapltr_mcp_server()"
#' ```
#'
#' Alternatively, use the bundled launcher script:
#' ```bash
#' claude mcp add -s user reliapltr -- \
#'   Rscript /path/to/ReliaPlotR/mcp/server.R
#' ```
#' @export
reliapltr_mcp_server <- function(...) {
  .check_mcp_deps()
  tools <- list(
    .make_wblr_tool(),
    .make_alt_tool(),
    .make_plot_wblr_tool(),
    .make_plot_alt_tool(),
    .make_plot_rga_tool()
  )
  mcptools::mcp_server(tools = tools, ...)
}

# --------------------------------------------------------------------------- #
# Internal helpers                                                             #
# --------------------------------------------------------------------------- #

.check_mcp_deps <- function() {
  missing <- character(0)
  if (!requireNamespace("mcptools", quietly = TRUE)) missing <- c(missing, "mcptools")
  if (!requireNamespace("ellmer",   quietly = TRUE)) missing <- c(missing, "ellmer")
  if (length(missing) > 0) {
    stop(
      "The following packages are required to run the MCP server but are not installed: ",
      paste(missing, collapse = ", "), ". ",
      "Install them with: pak::pak(c('mcptools', 'ellmer'))",
      call. = FALSE
    )
  }
  invisible(NULL)
}

.make_wblr_tool <- function() {
  ellmer::tool(
    fun = function(failures,
                   suspensions = NULL,
                   dist        = "weibull",
                   method_fit  = "mle",
                   method_conf = "lrb") {

      # lrb confidence bounds require MLE; fall back to Fisher-matrix for RR
      if (!identical(method_fit, "mle") && identical(method_conf, "lrb")) {
        method_conf <- "fm"
      }

      obj <- WeibullR::wblr(failures, suspensions)
      obj <- WeibullR::wblr.fit(obj, dist = dist, method.fit = method_fit)
      obj <- tryCatch(
        WeibullR::wblr.conf(obj, method.conf = method_conf),
        error = function(e) obj   # return un-conf'd object if bounds fail
      )

      est <- tidy_wblr(obj)$estimates
      as.list(est[1L, , drop = FALSE])
    },
    name = "fit_weibull",
    description = paste(
      "Fit a Weibull or lognormal distribution to time-to-failure data.",
      "Returns distribution parameters (Beta and Eta for Weibull;",
      "Mulog and Sigmalog for lognormal), a goodness-of-fit metric (R-squared",
      "for rank regression, log-likelihood for MLE), and the number of failures.",
      "Optionally accepts right-censored suspension times."
    ),
    arguments = list(
      failures    = ellmer::type_array(
        ellmer::type_number(),
        description = "Failure times as positive numbers, e.g. [30, 49, 82, 90, 96]."
      ),
      suspensions = ellmer::type_array(
        ellmer::type_number(),
        description = paste(
          "Right-censored (suspension) times -- units still running at end of test.",
          "Optional. Omit if there are no suspensions."
        ),
        required = FALSE
      ),
      dist = ellmer::type_string(
        description = paste(
          "Distribution to fit:",
          "'weibull' (2-parameter Weibull, default),",
          "'lognormal', or 'weibull3p' (3-parameter Weibull)."
        )
      ),
      method_fit = ellmer::type_string(
        description = paste(
          "Parameter estimation method:",
          "'mle' (maximum likelihood, default) or 'rr-xony' (rank regression).",
          "MLE is preferred for censored data."
        )
      ),
      method_conf = ellmer::type_string(
        description = paste(
          "Confidence bound method:",
          "'lrb' (likelihood-ratio bounds, default, requires MLE) or",
          "'fm' (Fisher-matrix bounds).",
          "Automatically set to 'fm' if rank regression is selected."
        )
      )
    )
  )
}

.make_alt_tool <- function() {
  ellmer::tool(
    fun = function(stresses,
                   failures_json,
                   dist      = "weibull",
                   alt_model = "arrhenius") {

      if (!requireNamespace("jsonlite", quietly = TRUE)) {
        stop("Package 'jsonlite' is required for fit_alt. ",
             "Install with: install.packages('jsonlite')", call. = FALSE)
      }

      failures_list <- jsonlite::fromJSON(failures_json)
      if (!is.list(failures_list)) {
        # fromJSON may return a matrix for regular-length arrays; coerce to list
        failures_list <- lapply(seq_len(nrow(failures_list)),
                                function(i) as.numeric(failures_list[i, ]))
      }

      if (length(stresses) != length(failures_list)) {
        stop("'stresses' and 'failures_json' must have the same number of entries.",
             call. = FALSE)
      }

      data_list <- mapply(
        function(s, f) WeibullR.ALT::alt.data(as.numeric(f), stress = s),
        stresses, failures_list,
        SIMPLIFY = FALSE
      )

      obj <- WeibullR.ALT::alt.make(data_list, dist = dist,
                                    alt.model = alt_model,
                                    view_dist_fits = FALSE)
      obj <- WeibullR.ALT::alt.parallel(obj, view_parallel_fits = FALSE)
      obj <- WeibullR.ALT::alt.fit(obj)

      result <- tidy_alt(obj)
      list(
        parallel     = result$parallel,
        relationship = as.list(result$relationship[1L, ])
      )
    },
    name = "fit_alt",
    description = paste(
      "Fit an Accelerated Life Test (ALT) model across multiple stress levels.",
      "Returns per-stress-level distribution parameters (P1 = characteristic life,",
      "P2 = shape parameter) and the global life-stress relationship coefficients",
      "(Arrhenius or Power Law).",
      "Requires at least two stress levels with failure data."
    ),
    arguments = list(
      stresses = ellmer::type_array(
        ellmer::type_number(),
        description = paste(
          "Stress values, one per stress level.",
          "Example: [300, 350, 400] for three temperature stress levels in Kelvin."
        )
      ),
      failures_json = ellmer::type_string(
        description = paste(
          "JSON string encoding an array of failure-time arrays, one inner array",
          "per stress level (must match the length of stresses).",
          "Example: '[[248,456,528,731],[164,176,289],[88,112,152]]'",
          "for three stress levels with 4, 3, and 3 failures respectively."
        )
      ),
      dist = ellmer::type_string(
        description = "Life distribution: 'weibull' (default) or 'lognormal'."
      ),
      alt_model = ellmer::type_string(
        description = paste(
          "Life-stress relationship model:",
          "'arrhenius' (default, for temperature stress) or",
          "'power' (inverse power law, for non-thermal stress)."
        )
      )
    )
  )
}

.make_plot_wblr_tool <- function() {
  ellmer::tool(
    fun = function(failures,
                   suspensions = NULL,
                   dist        = "weibull",
                   method_fit  = "mle",
                   method_conf = "lrb",
                   show_conf   = TRUE) {

      if (!identical(method_fit, "mle") && identical(method_conf, "lrb")) {
        method_conf <- "fm"
      }

      obj <- WeibullR::wblr(failures, suspensions)
      obj <- WeibullR::wblr.fit(obj, dist = dist, method.fit = method_fit)
      obj <- tryCatch(
        WeibullR::wblr.conf(obj, method.conf = method_conf),
        error = function(e) obj
      )

      fig <- plotly_wblr(obj, showConf = show_conf)
      jsonlite::toJSON(plotly::plotly_build(fig)$x, digits = 50,
                       auto_unbox = TRUE, force = TRUE, null = "null",
                       na = "null", time_format = "%Y-%m-%d %H:%M:%OS6")
    },
    name = "plot_weibull",
    description = paste(
      "Fit a Weibull or lognormal distribution and return an interactive",
      "Weibull probability plot as a plotly JSON string.",
      "The JSON can be saved as HTML or rendered in any plotly-compatible viewer.",
      "Optionally accepts right-censored suspension times."
    ),
    arguments = list(
      failures    = ellmer::type_array(
        ellmer::type_number(),
        description = "Failure times as positive numbers, e.g. [30, 49, 82, 90, 96]."
      ),
      suspensions = ellmer::type_array(
        ellmer::type_number(),
        description = paste(
          "Right-censored (suspension) times -- units still running at end of test.",
          "Optional. Omit if there are no suspensions."
        ),
        required = FALSE
      ),
      dist = ellmer::type_string(
        description = paste(
          "Distribution to fit:",
          "'weibull' (2-parameter Weibull, default),",
          "'lognormal', or 'weibull3p' (3-parameter Weibull)."
        )
      ),
      method_fit = ellmer::type_string(
        description = paste(
          "Parameter estimation method:",
          "'mle' (maximum likelihood, default) or 'rr-xony' (rank regression)."
        )
      ),
      method_conf = ellmer::type_string(
        description = paste(
          "Confidence bound method:",
          "'lrb' (likelihood-ratio bounds, default, requires MLE) or",
          "'fm' (Fisher-matrix bounds)."
        )
      ),
      show_conf = ellmer::type_boolean(
        description = "Whether to show confidence bounds on the plot. Default TRUE."
      )
    )
  )
}

.make_plot_alt_tool <- function() {
  ellmer::tool(
    fun = function(stresses,
                   failures_json,
                   dist      = "weibull",
                   alt_model = "arrhenius") {

      failures_list <- jsonlite::fromJSON(failures_json)
      if (!is.list(failures_list)) {
        failures_list <- lapply(seq_len(nrow(failures_list)),
                                function(i) as.numeric(failures_list[i, ]))
      }

      if (length(stresses) != length(failures_list)) {
        stop("'stresses' and 'failures_json' must have the same number of entries.",
             call. = FALSE)
      }

      data_list <- mapply(
        function(s, f) WeibullR.ALT::alt.data(as.numeric(f), stress = s),
        stresses, failures_list,
        SIMPLIFY = FALSE
      )

      obj <- WeibullR.ALT::alt.make(data_list, dist = dist,
                                    alt.model = alt_model,
                                    view_dist_fits = FALSE)
      obj <- WeibullR.ALT::alt.parallel(obj, view_parallel_fits = FALSE)
      obj <- WeibullR.ALT::alt.fit(obj)

      fig_prob <- plotly_alt(obj)
      fig_rel  <- plotly_rel(obj)

      to_json <- function(fig) {
        jsonlite::toJSON(plotly::plotly_build(fig)$x, digits = 50,
                       auto_unbox = TRUE, force = TRUE, null = "null",
                       na = "null", time_format = "%Y-%m-%d %H:%M:%OS6")
      }

      list(
        probability_plot = to_json(fig_prob),
        life_stress_plot = to_json(fig_rel)
      )
    },
    name = "plot_alt",
    description = paste(
      "Fit an Accelerated Life Test (ALT) model and return two interactive plots",
      "as plotly JSON strings: a probability plot (one fitted line per stress level)",
      "and a life-stress relationship plot.",
      "Both JSON strings can be saved as HTML or rendered in a plotly-compatible viewer."
    ),
    arguments = list(
      stresses = ellmer::type_array(
        ellmer::type_number(),
        description = paste(
          "Stress values, one per stress level.",
          "Example: [300, 350, 400] for three temperature stress levels in Kelvin."
        )
      ),
      failures_json = ellmer::type_string(
        description = paste(
          "JSON string encoding an array of failure-time arrays, one inner array",
          "per stress level (must match the length of stresses).",
          "Example: '[[248,456,528,731],[164,176,289],[88,112,152]]'"
        )
      ),
      dist = ellmer::type_string(
        description = "Life distribution: 'weibull' (default) or 'lognormal'."
      ),
      alt_model = ellmer::type_string(
        description = paste(
          "Life-stress relationship model:",
          "'arrhenius' (default, for temperature stress) or",
          "'power' (inverse power law, for non-thermal stress)."
        )
      )
    )
  )
}

.make_plot_rga_tool <- function() {
  ellmer::tool(
    fun = function(times,
                   failures,
                   times_type = "failure_times",
                   method     = "LS",
                   show_conf  = TRUE) {

      obj <- ReliaGrowR::rga(times, failures,
                              times_type = times_type,
                              method     = method)
      fig <- plotly_rga(obj, showConf = show_conf)
      jsonlite::toJSON(plotly::plotly_build(fig)$x, digits = 50,
                       auto_unbox = TRUE, force = TRUE, null = "null",
                       na = "null", time_format = "%Y-%m-%d %H:%M:%OS6")
    },
    name = "plot_rga",
    description = paste(
      "Fit a Crow-AMSAA reliability growth model (Power Law NHPP) and return",
      "an interactive cumulative failure plot as a plotly JSON string.",
      "The JSON can be saved as HTML or rendered in any plotly-compatible viewer."
    ),
    arguments = list(
      times = ellmer::type_array(
        ellmer::type_number(),
        description = paste(
          "Failure times or cumulative failure times (positive numbers).",
          "Interpretation depends on times_type."
        )
      ),
      failures = ellmer::type_array(
        ellmer::type_number(),
        description = "Number of failures at each corresponding time in 'times'."
      ),
      times_type = ellmer::type_string(
        description = paste(
          "Interpretation of 'times':",
          "'failure_times' (default, individual failure times) or",
          "'cumulative_failure_times' (cumulative time at each failure)."
        )
      ),
      method = ellmer::type_string(
        description = paste(
          "Parameter estimation method:",
          "'LS' (least squares, default) or 'MLE' (maximum likelihood)."
        )
      ),
      show_conf = ellmer::type_boolean(
        description = "Whether to show confidence bounds on the plot. Default TRUE."
      )
    )
  )
}

#' Extract Tidy Parameter Estimates from an alt Object.
#'
#' Returns tidy data frames of per-stress-level parameter estimates and (when
#' available) global life-stress relationship coefficients from a fitted `alt`
#' object. Useful for reporting ALT results in reproducible research workflows.
#'
#' @param alt_obj An object of class `'alt'` created by the `WeibullR.ALT`
#'   package. Must have been processed through [WeibullR.ALT::alt.parallel()].
#'   For `$relationship` to be non-NULL, [WeibullR.ALT::alt.fit()] must also
#'   have been called.
#' @return A named list with two elements:
#'   \describe{
#'     \item{`parallel`}{A `data.frame` with one row per stress level,
#'       containing columns `stress`, `P1`, `P2`, `wt`, and `n_failures`.
#'       For Weibull models, `P1` is the scale parameter \eqn{\eta} and `P2`
#'       is the shape parameter \eqn{\beta}. For lognormal models, `P1` is
#'       \eqn{\mu_{log}} and `P2` is \eqn{\sigma_{log}}.}
#'     \item{`relationship`}{A one-row `data.frame` with columns `model`,
#'       `coef1`, and `coef2` (the global life-stress relationship
#'       coefficients), or `NULL` with a message if `alt.fit()` has not been
#'       called.}
#'   }
#' @details
#' Two life-stress models are supported. For the Arrhenius model,
#' \eqn{\eta = \exp(C_1 + C_2 \cdot S)} where \eqn{S} is stress (typically
#' reciprocal temperature in 1/K). For the Power Law model,
#' \eqn{\eta = \exp(C_1) / S^{|C_2|}}. Both \code{coef1} and \code{coef2}
#' are on the log scale as returned by \code{alt.fit()}.
#' @references
#' Nelson, W. B. (1990). \emph{Accelerated Testing: Statistical Models,
#' Test Plans, and Data Analysis}. Wiley.
#' @seealso [plotly_alt()] for the ALT probability plot,
#'   [plotly_rel()] for the life-stress relationship plot.
#' @examples
#' library(WeibullR.ALT)
#' d1 <- alt.data(c(248, 456, 528, 731, 813, 537), stress = 300)
#' d2 <- alt.data(c(164, 176, 289), stress = 350)
#' d3 <- alt.data(c(88, 112, 152), stress = 400)
#' obj <- alt.fit(
#'   alt.parallel(
#'     alt.make(list(d1, d2, d3), dist = "weibull", alt.model = "arrhenius",
#'              view_dist_fits = FALSE),
#'     view_parallel_fits = FALSE
#'   )
#' )
#' result <- tidy_alt(obj)
#' result$parallel
#' result$relationship
#' @import WeibullR.ALT
#' @export

tidy_alt <- function(alt_obj) {
  if (!inherits(alt_obj, "alt")) {
    stop("Argument 'alt_obj' is not of class 'alt'.")
  }
  if (is.null(alt_obj$parallel_par)) {
    stop("'alt_obj' must be fitted with alt.parallel() before calling tidy_alt().")
  }

  pp <- alt_obj$parallel_par

  n_failures <- vapply(seq_len(nrow(pp)), function(i) {
    stress_i  <- pp$stress[i]
    data_idx  <- which(vapply(alt_obj$data, function(d) d$stress, numeric(1)) == stress_i)
    if (length(data_idx) == 0) return(NA_integer_)
    as.integer(alt_obj$data[[data_idx[1]]]$num_fails)
  }, integer(1))

  parallel_df <- data.frame(
    stress     = pp$stress,
    P1         = pp$P1,
    P2         = pp$P2,
    wt         = pp$wt,
    n_failures = n_failures,
    stringsAsFactors = FALSE
  )

  if (is.null(alt_obj$alt_coef)) {
    message("'alt_obj' has not been fitted with alt.fit(). Returning relationship = NULL.")
    relationship_df <- NULL
  } else {
    coef <- alt_obj$alt_coef
    relationship_df <- data.frame(
      model = alt_obj$alt.model,
      coef1 = coef[1],
      coef2 = coef[2],
      stringsAsFactors = FALSE,
      row.names = NULL
    )
  }

  list(parallel = parallel_df, relationship = relationship_df)
}

mock_alt_full <- list(
  data = list(
    list(stress = 300, data = data.frame(left  = c(248, 456, 528),
                                         right = c(248, 456, 528),
                                         qty   = c(1L, 1L, 1L)),
         num_fails = 3L, valid_set = TRUE),
    list(stress = 350, data = data.frame(left  = c(164, 176, 289),
                                         right = c(164, 176, 289),
                                         qty   = c(1L, 1L, 1L)),
         num_fails = 3L, valid_set = TRUE)
  ),
  dist         = "weibull",
  alt.model    = "arrhenius",
  method.fit   = "mle-rba",
  parallel_par = data.frame(P1     = c(615.97, 231.86),
                             P2     = c(3.94, 3.94),
                             stress = c(300, 350),
                             wt     = c(3L, 3L)),
  alt_coef     = c(11.206, -0.01606)
)
class(mock_alt_full) <- "alt"

mock_alt_no_fit <- mock_alt_full
mock_alt_no_fit$alt_coef <- NULL

test_that("tidy_alt stops for non-alt input", {
  expect_error(tidy_alt(list()), "not of class 'alt'")
})

test_that("tidy_alt stops when parallel_par is missing", {
  obj <- mock_alt_full
  obj$parallel_par <- NULL
  expect_error(tidy_alt(obj), "alt.parallel\\(\\)")
})

test_that("tidy_alt returns a list with parallel and relationship", {
  result <- tidy_alt(mock_alt_full)
  expect_type(result, "list")
  expect_named(result, c("parallel", "relationship"))
})

test_that("tidy_alt parallel is a data frame", {
  result <- tidy_alt(mock_alt_full)
  expect_s3_class(result$parallel, "data.frame")
})

test_that("tidy_alt parallel has correct columns", {
  result <- tidy_alt(mock_alt_full)
  expect_true(all(c("stress", "P1", "P2", "wt", "n_failures") %in%
                    names(result$parallel)))
})

test_that("tidy_alt parallel has one row per stress level", {
  result <- tidy_alt(mock_alt_full)
  expect_equal(nrow(result$parallel), 2)
})

test_that("tidy_alt parallel stress values are correct", {
  result <- tidy_alt(mock_alt_full)
  expect_equal(result$parallel$stress, c(300, 350))
})

test_that("tidy_alt parallel P1 values are correct", {
  result <- tidy_alt(mock_alt_full)
  expect_equal(result$parallel$P1, c(615.97, 231.86))
})

test_that("tidy_alt parallel n_failures matches mock", {
  result <- tidy_alt(mock_alt_full)
  expect_equal(result$parallel$n_failures, c(3L, 3L))
})

test_that("tidy_alt relationship is a data frame when alt_coef present", {
  result <- tidy_alt(mock_alt_full)
  expect_s3_class(result$relationship, "data.frame")
})

test_that("tidy_alt relationship has correct columns", {
  result <- tidy_alt(mock_alt_full)
  expect_true(all(c("model", "coef1", "coef2") %in% names(result$relationship)))
})

test_that("tidy_alt relationship coef values are correct", {
  result <- tidy_alt(mock_alt_full)
  expect_equal(result$relationship$coef1, 11.206)
  expect_equal(result$relationship$coef2, -0.01606)
})

test_that("tidy_alt relationship is NULL when alt.fit not called", {
  expect_message(result <- tidy_alt(mock_alt_no_fit), "alt.fit")
  expect_null(result$relationship)
})

test_that("tidy_alt parallel still returned when relationship is NULL", {
  suppressMessages(result <- tidy_alt(mock_alt_no_fit))
  expect_s3_class(result$parallel, "data.frame")
  expect_equal(nrow(result$parallel), 2)
})

#' Interactive ALT Probability Plot.
#'
#' Creates an interactive probability plot for an `alt` object, overlaying one
#' Weibull or lognormal fit line per stress level on a shared probability paper.
#' The `alt` object must have been processed through [WeibullR.ALT::alt.parallel()]
#' before passing to this function.
#'
#' @param alt_obj An object of class `'alt'` created by the `WeibullR.ALT` package
#'   and fitted with `alt.parallel()`.
#' @param showConf Show Fisher-matrix confidence bounds (TRUE) or not (FALSE).
#'   Default is TRUE.
#' @param showGrid Show grid (TRUE) or hide grid (FALSE). Default is TRUE.
#' @param main Main title. Default is "ALT Probability Plot".
#' @param xlab X-axis label. Default is "Time to Failure".
#' @param ylab Y-axis label. Default is "Probability".
#' @param gridCol Color of the grid. Default is "lightgray".
#' @param signif Significant digits for hover text. Default is 3.
#' @param cols Optional character vector of colors, one per stress level. Recycled
#'   to match the number of stress levels. When NULL a 10-color default palette is used.
#' @return A `plotly` object representing the interactive ALT probability plot.
#' @details
#' Building an \code{alt} object requires three steps:
#' \enumerate{
#'   \item \code{alt.make()} — bundle the per-stress data sets and choose a
#'     distribution (\code{"weibull"} or \code{"lognormal"}) and life-stress
#'     model (\code{"arrhenius"} or \code{"power"}).
#'   \item \code{alt.parallel()} — fit independent Weibull/lognormal models
#'     at each stress level (required before calling \code{plotly_alt()}).
#'   \item \code{alt.fit()} — fit the global life-stress relationship
#'     (required for \code{plotly_rel()}).
#' }
#' The probability paper transformation (log y-axis for Weibull, normal
#' quantile for lognormal) is the same as in [plotly_wblr()].
#' @seealso [plotly_rel()] for the life-stress relationship plot using the same \code{alt} object.
#' @examples
#' library(WeibullR.ALT)
#' d1 <- alt.data(c(248, 456, 528, 731, 813, 537), stress = 300)
#' d2 <- alt.data(c(164, 176, 289), stress = 350)
#' d3 <- alt.data(c(88, 112, 152), stress = 400)
#' obj <- alt.fit(
#'   alt.parallel(
#'     alt.make(list(d1, d2, d3), dist = "weibull", alt.model = "arrhenius", view_dist_fits = FALSE),
#'     view_parallel_fits = FALSE
#'   )
#' )
#' plotly_alt(obj)
#' @import WeibullR.ALT
#' @import WeibullR
#' @import plotly
#' @importFrom stats pweibull plnorm qnorm qweibull qlnorm
#' @export

plotly_alt <- function(alt_obj,
                       showConf = TRUE,
                       showGrid = TRUE,
                       main     = "ALT Probability Plot",
                       xlab     = "Time to Failure",
                       ylab     = "Probability",
                       gridCol  = "lightgray",
                       signif   = 3,
                       cols     = NULL) {

  if (!inherits(alt_obj, "alt")) {
    stop("Argument 'alt_obj' is not of class 'alt'.")
  }
  if (is.null(alt_obj$parallel_par)) {
    stop("'alt_obj' must be fitted with alt.parallel() before plotting.")
  }

  pp        <- alt_obj$parallel_par
  n         <- nrow(pp)
  dist_type <- alt_obj$dist

  # Per-stress-level colors
  default_palette <- c(
    "#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd",
    "#8c564b", "#e377c2", "#7f7f7f", "#bcbd22", "#17becf"
  )
  cols <- rep(if (is.null(cols)) default_palette else cols, length.out = n)

  # Y-axis probability ticks (identical to plotly_wblr)
  yticks <- c(
    0.000001, 0.00001, 0.0001, 0.001, 0.01, 0.05, 0.1, 0.2, 0.5, 1,
    2, 5, 10, 20, 50, 90, 99, 99.999
  )
  if (dist_type == "lognormal") {
    yticks_trans <- qnorm(yticks / 100)
    yaxis_scale  <- "linear"
  } else {
    yticks_trans <- log(1 / (1 - yticks / 100))
    yaxis_scale  <- "log"
  }

  show_grid <- is.null(showGrid) || isTRUE(showGrid)

  # Collect all failure times for global x range (extended by fit line extremes)
  all_failures <- unlist(lapply(alt_obj$data, function(d) {
    dat <- d$data
    rep(dat$left[dat$right >= 0], dat$qty[dat$right >= 0])
  }))

  p <- plot_ly()

  # Track x-range across all stress levels
  all_t_seq <- numeric(0)

  for (i in seq_len(n)) {
    stress_i <- pp$stress[i]
    P1_i     <- pp$P1[i]
    P2_i     <- pp$P2[i]
    col_i    <- cols[i]
    grp      <- as.character(i)

    # Match stress level to raw data entry
    data_idx <- which(sapply(alt_obj$data, function(d) d$stress) == stress_i)
    dat      <- alt_obj$data[[data_idx]]$data

    fail_rows <- dat[dat$right >= 0, ]
    susp_rows <- dat[dat$right <  0, ]
    failures_i    <- rep(fail_rows$left, fail_rows$qty)
    suspensions_i <- rep(susp_rows$left, susp_rows$qty)

    # Empirical plotting positions via WeibullR
    wblr_i <- if (length(suspensions_i) > 0) {
      WeibullR::wblr(failures_i, suspensions_i)
    } else {
      WeibullR::wblr(failures_i)
    }
    time_pts <- wblr_i$data$dpoints$time
    ppp_pts  <- wblr_i$data$dpoints$ppp

    # Transform to probability-paper coordinates
    prob_trans <- if (dist_type == "lognormal") {
      qnorm(ppp_pts)
    } else {
      log(1 / (1 - ppp_pts))
    }

    # Theoretical fit line spanning ~0.1% to 99.9% of the fitted CDF
    if (dist_type == "lognormal") {
      t_lo <- exp(P1_i + qnorm(0.001) * P2_i)
      t_hi <- exp(P1_i + qnorm(0.999) * P2_i)
    } else {
      t_lo <- P1_i * qweibull(0.001, shape = P2_i)
      t_hi <- P1_i * qweibull(0.999, shape = P2_i)
    }
    t_seq <- exp(seq(log(t_lo), log(t_hi), length.out = 200))
    all_t_seq <- c(all_t_seq, t_seq)

    if (dist_type == "lognormal") {
      F_seq   <- plnorm(t_seq, meanlog = P1_i, sdlog = P2_i)
      F_trans <- qnorm(F_seq)
    } else {
      F_seq   <- pweibull(t_seq, shape = P2_i, scale = P1_i)
      F_trans <- log(1 / (1 - F_seq))
    }

    # Remove non-finite values at CDF extremes
    keep    <- is.finite(F_trans)
    t_seq   <- t_seq[keep]
    F_trans <- F_trans[keep]

    p <- p %>%
      add_trace(
        x = time_pts, y = prob_trans,
        type = "scatter", mode = "markers",
        marker = list(color = col_i),
        name = paste0("Stress ", stress_i),
        legendgroup = grp,
        text = paste0(
          "Stress: ", stress_i,
          ", Time: ", round(time_pts, signif),
          ", Prob: ", round(ppp_pts * 100, signif), "%"
        ),
        hoverinfo = "text"
      ) %>%
      add_trace(
        x = t_seq, y = F_trans,
        type = "scatter", mode = "lines",
        line = list(color = col_i),
        name = paste0("Fit ", stress_i),
        legendgroup = grp,
        showlegend = FALSE,
        hoverinfo = "skip"
      )

    # Optional Fisher-matrix confidence bounds
    if (isTRUE(showConf)) {
      tryCatch({
        wblr_fit_i <- WeibullR::wblr.fit(wblr_i, method.fit = "mle")
        wblr_fit_i <- WeibullR::wblr.conf(wblr_fit_i, method.conf = "fm")
        conf_data  <- wblr_fit_i$fit[[1]]$conf[[1]]$bounds

        if (!is.null(conf_data) &&
            !is.null(conf_data$Lower) && !is.null(conf_data$Upper)) {
          conf_unrel  <- conf_data$unrel
          conf_lower  <- conf_data$Lower
          conf_upper  <- conf_data$Upper

          conf_trans <- if (dist_type == "lognormal") {
            qnorm(conf_unrel)
          } else {
            log(1 / (1 - conf_unrel))
          }

          keep_conf   <- is.finite(conf_trans)
          conf_trans  <- conf_trans[keep_conf]
          conf_lower  <- conf_lower[keep_conf]
          conf_upper  <- conf_upper[keep_conf]

          fillcolor <- plotly::toRGB(col_i, 0.2)

          p <- p %>%
            add_trace(
              x = conf_lower, y = conf_trans,
              type = "scatter", mode = "lines",
              line = list(color = col_i, width = 1),
              showlegend = FALSE, legendgroup = grp,
              hoverinfo = "skip"
            ) %>%
            add_trace(
              x = conf_upper, y = conf_trans,
              type = "scatter", mode = "lines",
              fill = "tonexty", fillcolor = fillcolor,
              line = list(color = col_i, width = 1),
              showlegend = FALSE, legendgroup = grp,
              hoverinfo = "skip"
            )
        }
      }, error = function(e) NULL)
    }
  }

  # X-axis range from fit line extremes (covers the full 0.1%–99.9% range)
  xmin <- log10(min(all_t_seq, all_failures))
  xmax <- log10(max(all_t_seq, all_failures))

  p %>%
    layout(
      title = main,
      xaxis = list(
        type = "log", title = xlab, showline = TRUE, mirror = "ticks",
        showgrid = show_grid, gridcolor = gridCol,
        range = list(xmin, xmax)
      ),
      yaxis = list(
        type = yaxis_scale, title = ylab, showline = TRUE, mirror = "ticks",
        showgrid = show_grid, gridcolor = gridCol,
        tickvals = yticks_trans, ticktext = yticks
      )
    )
}

# Shared mock 'alt' object for probability plot tests.
# Must have gone through alt.parallel() — needs $parallel_par.
mock_alt_prob <- list(
  data = list(
    list(
      stress    = 300,
      data      = data.frame(left  = c(248, 456, 528),
                             right = c(248, 456, 528),
                             qty   = c(1L, 1L, 1L)),
      num_fails = 3,
      valid_set = TRUE
    ),
    list(
      stress    = 350,
      data      = data.frame(left  = c(164, 176, 289),
                             right = c(164, 176, 289),
                             qty   = c(1L, 1L, 1L)),
      num_fails = 3,
      valid_set = TRUE
    )
  ),
  dist         = "weibull",
  alt.model    = "arrhenius",
  method.fit   = "mle-rba",
  parallel_par = data.frame(P1     = c(615.97, 231.86),
                             P2     = c(3.94, 3.94),
                             stress = c(300, 350),
                             wt     = c(3L, 3L)),
  alt_coef     = c(11.206, -0.01606)
)
class(mock_alt_prob) <- "alt"

# Lognormal variant (P1 = log(median))
mock_alt_lnorm <- mock_alt_prob
mock_alt_lnorm$dist         <- "lognormal"
mock_alt_lnorm$parallel_par <- data.frame(P1     = log(c(615.97, 231.86)),
                                           P2     = c(0.55, 0.55),
                                           stress = c(300, 350),
                                           wt     = c(3L, 3L))

# --- Tests -------------------------------------------------------------------

test_that("plotly_alt throws error for incorrect input class", {
  expect_error(plotly_alt(list()), "not of class 'alt'")
})

test_that("plotly_alt throws error when parallel_par is missing", {
  obj <- mock_alt_prob
  obj$parallel_par <- NULL
  expect_error(plotly_alt(obj), "alt.parallel\\(\\)")
})

test_that("plotly_alt showConf = TRUE adds more traces than showConf = FALSE", {
  p_with    <- plotly_alt(mock_alt_prob, showConf = TRUE)
  p_without <- plotly_alt(mock_alt_prob, showConf = FALSE)
  expect_gte(length(p_with$x$attrs), length(p_without$x$attrs))
})

test_that("plotly_alt returns a plotly object", {
  p <- plotly_alt(mock_alt_prob)
  expect_s3_class(p, "plotly")
})

test_that("plotly_alt produces more traces for more stress levels", {
  # Two stress levels should produce more traces than a single stress level would
  p2 <- plotly_alt(mock_alt_prob)                     # 2 stress levels

  single_stress        <- mock_alt_prob
  single_stress$parallel_par <- mock_alt_prob$parallel_par[1, ]
  single_stress$data   <- mock_alt_prob$data[1]
  p1 <- plotly_alt(single_stress)                     # 1 stress level

  expect_gt(length(p2$x$attrs), length(p1$x$attrs))
})

test_that("plotly_alt respects custom cols parameter", {
  p <- plotly_alt(mock_alt_prob, cols = c("steelblue", "tomato"))
  expect_s3_class(p, "plotly")
})

test_that("plotly_alt works with showGrid = FALSE", {
  p <- plotly_alt(mock_alt_prob, showGrid = FALSE)
  expect_equal(p$x$layoutAttrs[[1]]$xaxis$showgrid, FALSE)
  expect_equal(p$x$layoutAttrs[[1]]$yaxis$showgrid, FALSE)
})

test_that("plotly_alt works for lognormal distribution", {
  p <- plotly_alt(mock_alt_lnorm)
  expect_s3_class(p, "plotly")
})

test_that("plotly_alt respects custom main, xlab, ylab", {
  p <- plotly_alt(mock_alt_prob, main = "My ALT", xlab = "Time", ylab = "Probability")
  expect_equal(p$x$layoutAttrs[[1]]$title, "My ALT")
  expect_equal(p$x$layoutAttrs[[1]]$xaxis$title, "Time")
  expect_equal(p$x$layoutAttrs[[1]]$yaxis$title, "Probability")
})

test_that("plotly_alt data scatter has correct hover text prefix", {
  p     <- plotly_alt(mock_alt_prob)
  built <- plotly::plotly_build(p)
  # First trace is the stress-300 scatter
  expect_true(all(grepl("^Stress: 300", built$x$data[[1]]$text)))
})

#' Interactive Duane Plot.
#'
#' This function creates an interactive Duane plot for a \code{duane} object.
#' The plot displays observed cumulative MTBF against cumulative test time on
#' a log-log scale alongside the fitted Duane line and optional confidence
#' bounds. Positive slope on the log-log scale indicates reliability growth.
#'
#' @param duane_obj An object of class 'duane'. This object is created
#' using the `duane` function from the ReliaGrowR package.
#' @param showConf Show the confidence bounds (TRUE) or not (FALSE). Default is TRUE.
#' @param showGrid Show grid (TRUE) or hide grid (FALSE). Default is TRUE.
#' @param main Main title. Default is "Duane Plot".
#' @param xlab X-axis label. Default is "Cumulative Time".
#' @param ylab Y-axis label. Default is "Cumulative MTBF".
#' @param pointCol Color of the point values. Default is "black".
#' @param fitCol Color of the model fit. Default is "black".
#' @param confCol Color of the confidence bounds. Default is "black".
#' @param gridCol Color of the grid. Default is "lightgray".
#' @param signif Significant digits of results. Default is 3. Must be a positive integer.
#' @return A `plotly` object representing the interactive Duane plot.
#' @details
#' The Duane model fits a power-law relationship between cumulative MTBF and
#' cumulative test time: \eqn{\text{MTBF}_c = K \cdot T^\alpha}. On a
#' log-log plot this appears as a straight line with slope \eqn{\alpha} (the
#' growth rate) and intercept \eqn{\log(K)}. A slope near 0 indicates no
#' growth; a slope near 0.5 is typical for a managed development program.
#' @seealso [plotly_rga()] for the Crow-AMSAA NHPP growth model on the same data.
#' @examples
#' library(ReliaGrowR)
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' fit <- duane(times, failures)
#' plotly_duane(fit)
#' @import ReliaGrowR plotly
#' @importFrom graphics text
#' @importFrom stats runif qnorm
#' @export

plotly_duane <- function(duane_obj,
                         showConf = TRUE,
                         showGrid = TRUE,
                         main = "Duane Plot",
                         xlab = "Cumulative Time",
                         ylab = "Cumulative MTBF",
                         pointCol = "black",
                         fitCol = "black",
                         confCol = "black",
                         gridCol = "lightgray",
                         signif = 3) {
  # Validate inputs
  if (!inherits(duane_obj, "duane")) {
    stop("Argument 'duane_obj' is not of class 'duane'.")
  }

  # Extract data from the duane_obj
  times <- duane_obj$Cumulative_Time
  mtbf <- duane_obj$Cumulative_MTBF
  fitted <- duane_obj$Fitted_Values

  # Create the plot
  plot_duane <- function() {

    # Set up the plot layout
    fillcolor <- plotly::toRGB(confCol, 0.2)
    xgrid <- ifelse(is.null(showGrid) || isTRUE(showGrid), TRUE, FALSE)
    ygrid <- xgrid

    duane_plot <- plot_ly(
      x = times, y = mtbf, type = "scatter", mode = "markers",
      marker = list(color = pointCol), showlegend = FALSE, name = "",
      text = ~ paste0("MTBF: (", round(times, signif), ", ", round(mtbf, signif), ")"),
      hoverinfo = "text"
    ) %>%
      # Set up the main plot layout
      layout(
        title = main,
        xaxis = list(
          type = "log", title = xlab, showline = TRUE, mirror = "ticks",
          showgrid = xgrid, gridcolor = gridCol
        ),
        yaxis = list(
          type = "log", title = ylab, showline = TRUE, mirror = "ticks",
          showgrid = ygrid, gridcolor = gridCol
        )
      ) %>%
      # Add best fit
      add_trace(
        x = times, y = fitted, mode = "markers+lines",
        marker = list(color = "transparent"), line = list(color = fitCol),
        text = ~ paste0("Fit: (", round(times, signif), ", ", round(fitted, signif), ")"),
        hoverinfo = "text"
      )

    if (isTRUE(showConf)) {
      duane_plot <- duane_plot %>%
        # Add lower confidence bound
        add_trace(
          x = times, y = duane_obj$lower_bounds, mode = "markers+lines",
          marker = list(color = "transparent"), line = list(color = confCol),
          text = ~ paste0("Lower: (", round(times, signif), ", ", round(duane_obj$lower_bounds, signif), ")"),
          hoverinfo = "text"
        ) %>%
        # Add upper confidence bound
        add_trace(
          x = times, y = duane_obj$upper_bounds, mode = "markers+lines",
          fill = "tonexty",
          fillcolor = fillcolor,
          marker = list(color = "transparent"), line = list(color = confCol),
          text = ~ paste0("Upper: (", round(times, signif), ", ", round(duane_obj$upper_bounds, signif), ")"),
          hoverinfo = "text"
        )
    }

    return(duane_plot)
  }
  final_plot <- plot_duane()

  return(final_plot)
}

# Create a mock 'duane' class object for testing
mock_duane_obj <- list(
  Cumulative_Time = c(100, 200, 300, 400, 500),
  Cumulative_MTBF = c(10, 20, 30, 40, 50),
  Fitted_Values = c(12, 22, 32, 42, 52),
  lower_bounds = c(8, 18, 28, 38, 48),
  upper_bounds = c(12, 22, 32, 42, 52)
)
class(mock_duane_obj) <- "duane"

# Unit tests
test_that("plotly_duane throws an error for incorrect input class", {
  incorrect_obj <- list(Cumulative_Time = 1:5, Cumulative_MTBF = 1:5)
  expect_error(plotly_duane(incorrect_obj), "Argument 'duane_obj' is not of class 'duane'.")
})

test_that("plotly_duane returns a plotly object", {
  plot <- plotly_duane(mock_duane_obj)
  expect_s3_class(plot, "plotly")
})

test_that("plotly_duane works with default parameters", {
  plot <- plotly_duane(mock_duane_obj)
  expect_true(!is.null(plot))
  expect_s3_class(plot, "plotly")
})

test_that("plotly_duane works with custom color parameters", {
  plot <- plotly_duane(mock_duane_obj, pointCol = "red", fitCol = "blue", gridCol = "green")
  expect_s3_class(plot, "plotly")
})

test_that("plotly_duane includes hover text for data points", {
  plot <- plotly_duane(mock_duane_obj)
  hover_data <- plot$x$data[[1]]$text
  expect_true(all(grepl("MTBF: \\(", hover_data)))
})

test_that("plotly_duane respects the showGrid parameter", {
  plot_with_grid <- plotly_duane(mock_duane_obj, showGrid = TRUE)
  plot_without_grid <- plotly_duane(mock_duane_obj, showGrid = FALSE)

  expect_equal(plot_with_grid$x$layoutAttrs[[1]]$xaxis$showgrid, TRUE)
  expect_equal(plot_with_grid$x$layoutAttrs[[1]]$yaxis$showgrid, TRUE)
  expect_equal(plot_without_grid$x$layoutAttrs[[1]]$xaxis$showgrid, FALSE)
  expect_equal(plot_without_grid$x$layoutAttrs[[1]]$yaxis$showgrid, FALSE)
})

test_that("plotly_duane respects showConf = FALSE", {
  plot_with_conf <- plotly_duane(mock_duane_obj, showConf = TRUE)
  plot_without_conf <- plotly_duane(mock_duane_obj, showConf = FALSE)

  # With confidence: point + fit + lower + upper = 4 traces
  # Without confidence: point + fit = 2 traces
  expect_gt(length(plot_with_conf$x$attrs), length(plot_without_conf$x$attrs))
})

test_that("plotly_duane signif parameter affects hover text precision", {
  plot <- plotly_duane(mock_duane_obj, signif = 1)
  hover_data <- plot$x$data[[1]]$text
  # With signif=1, values should be rounded to 1 significant digit
  expect_true(all(grepl("MTBF: \\(", hover_data)))
})

test_that("plotly_duane respects custom main, xlab, ylab", {
  plot <- plotly_duane(mock_duane_obj, main = "My Duane", xlab = "Time", ylab = "MTBF")
  expect_equal(plot$x$layoutAttrs[[1]]$title, "My Duane")
  expect_equal(plot$x$layoutAttrs[[1]]$xaxis$title, "Time")
  expect_equal(plot$x$layoutAttrs[[1]]$yaxis$title, "MTBF")
})

test_that("plotly_duane accepts confCol parameter", {
  plot <- plotly_duane(mock_duane_obj, confCol = "orange")
  expect_s3_class(plot, "plotly")
})

test_that("plotly_duane signif rounds hover text values", {
  plot <- plotly_duane(mock_duane_obj, signif = 2)
  built <- plotly::plotly_build(plot)
  hover_data <- built$x$data[[1]]$text
  # signif=2 applied to Cumulative_Time (100,200,...) and MTBF (10,20,...)
  expect_true(all(grepl("MTBF: \\(1e\\+02|MTBF: \\(100", hover_data[1])))
})

---
title: "MCP Tools"
output:
  rmarkdown::html_vignette:
    fig_width: 7
    fig_height: 5
    self_contained: false
bibliography: ../inst/REFERENCES.bib
vignette: >
  %\VignetteIndexEntry{MCP Tools}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
has_ellmer <- requireNamespace("ellmer", quietly = TRUE)
```

```{r setup}
library(ReliaPlotR)
```

## Overview

The [Model Context Protocol](https://modelcontextprotocol.io) (MCP) is an open standard that lets AI assistants call external tools as part of a conversation. Two MCP servers together cover the full ReliaPlotR analysis pipeline:

**ReliaPlotR server** — `reliapltr_mcp_server()` — life data and ALT analysis:

| Tool | What it does |
|---|---|
| `fit_weibull` | Fit a Weibull or lognormal distribution to time-to-failure data and return parameter estimates |
| `fit_alt` | Fit an ALT model across multiple stress levels and return life-stress relationship coefficients |
| `plot_weibull` | Fit a distribution and return a Weibull probability plot as a plotly JSON string |
| `plot_alt` | Fit an ALT model and return probability and life-stress plots as plotly JSON strings |
| `plot_rga` | Fit a Crow-AMSAA model and return a cumulative failure plot as a plotly JSON string |

**ReliaGrowR server** — `rga_mcp_server()` — reliability growth and repairable systems:

| Tool | What it does |
|---|---|
| `rga` | Fit Crow-AMSAA reliability growth model; returns λ, β, and goodness-of-fit stats |
| `nhpp` | Fit a Power Law or Log-Linear NHPP process; returns model parameters |
| `duane` | Fit a Duane log-log growth model; returns slope, intercept, and GOF stats |
| `mcf` | Compute Nelson-Aalen MCF for repairable systems; returns time, MCF, and bounds |
| `predict_rga` | Forecast cumulative failures from a fitted Crow-AMSAA model |
| `predict_duane` | Forecast MTBF from a fitted Duane model |
| `rdt` | Reliability demonstration test planning — solve for required sample size or test time |
| `gof_rga` | Goodness-of-fit statistics (CvM, KS) for a Crow-AMSAA model |

## Installation

`mcptools` and `ellmer` are optional dependencies listed in `Suggests`. Install them before starting the server:

```{r install, eval = FALSE}
pak::pak(c("mcptools", "ellmer"))
```

## Registering with Claude Code

Once the packages are installed, register both servers in your Claude Code session.

**ReliaPlotR server** (life data and ALT analysis):

```{bash mcp-add-inline, eval = FALSE}
# Option 1: inline (works with any R installation)
claude mcp add -s user reliapltr -- \
  Rscript -e "ReliaPlotR::reliapltr_mcp_server()"
```

```{bash mcp-add-script, eval = FALSE}
# Option 2: bundled launcher script (after package installation)
claude mcp add -s user reliapltr -- \
  Rscript $(Rscript -e "cat(system.file('mcp/server.R', package='ReliaPlotR'))")
```

**ReliaGrowR server** (reliability growth and repairable systems):

```{bash mcp-add-reliagrow, eval = FALSE}
claude mcp add -s user reliagrowR -- \
  Rscript -e "ReliaGrowR::rga_mcp_server()"
```

Confirm both servers are registered:

```{bash mcp-list, eval = FALSE}
claude mcp list
```

Once registered, Claude Code will offer all tools automatically whenever it detects a reliability analysis task.

## The `fit_weibull` Tool

`fit_weibull` fits a Weibull or lognormal distribution to a vector of failure times, optionally with right-censored (suspension) data, and returns parameter estimates and a goodness-of-fit metric.

### Arguments

| Argument | Type | Default | Description |
|---|---|---|---|
| `failures` | numeric array | *required* | Failure times (positive numbers) |
| `suspensions` | numeric array | `NULL` | Right-censored times -- units still operating at end of test |
| `dist` | string | `"weibull"` | Distribution: `"weibull"`, `"lognormal"`, or `"weibull3p"` |
| `method_fit` | string | `"mle"` | Estimation method: `"mle"` (recommended for censored data) or `"rr-xony"` (rank regression) |
| `method_conf` | string | `"lrb"` | Confidence bounds: `"lrb"` (likelihood-ratio, requires MLE) or `"fm"` (Fisher-matrix). Automatically switches to `"fm"` when rank regression is used. |

### Example

The following shows what the tool returns when called from an MCP client. The same function can be invoked directly in R for testing:

```{r wblr-example, eval = has_ellmer}
# Five machines that failed at these times (hours)
tool <- ReliaPlotR:::.make_wblr_tool()
result <- tool(failures = c(30, 49, 82, 90, 96))
str(result)
```

The flat named list is serialized to JSON when returned over MCP. `param1` is the Weibull shape parameter $\beta$; `param2` is the characteristic life $\eta$.

Adding right-censored suspensions shifts the estimates to account for units that did not fail:

```{r wblr-susp, eval = has_ellmer}
result_susp <- tool(
  failures    = c(30, 49, 82, 90, 96),
  suspensions = c(100, 120, 150)
)
cat("Beta:", result_susp$param1, "\n")
cat("Eta: ", result_susp$param2, "\n")
```

## The `fit_alt` Tool

`fit_alt` runs the full ALT pipeline (`alt.make` → `alt.parallel` → `alt.fit`) across multiple stress levels and returns per-stress-level parameters alongside the fitted global life-stress relationship.

### Arguments

| Argument | Type | Default | Description |
|---|---|---|---|
| `stresses` | numeric array | *required* | One stress value per stress level, e.g. `[300, 350, 400]` |
| `failures_json` | string | *required* | JSON array of failure-time arrays, one inner array per stress level. Example: `'[[248,456,528],[164,176,289],[88,112,152]]'` |
| `dist` | string | `"weibull"` | Life distribution: `"weibull"` or `"lognormal"` |
| `alt_model` | string | `"arrhenius"` | Life-stress model: `"arrhenius"` (temperature) or `"power"` (non-thermal) |

### Example

```{r alt-example, eval = has_ellmer, results = "hide"}
tool_alt <- ReliaPlotR:::.make_alt_tool()
result_alt <- tool_alt(
  stresses      = c(300, 350, 400),
  failures_json = "[[248,456,528,731,813,537],[164,176,289],[88,112,152]]",
  dist          = "weibull",
  alt_model     = "arrhenius"
)
```

```{r alt-show, eval = has_ellmer}
# Per-stress-level parameter estimates
result_alt$parallel

# Global Arrhenius life-stress relationship coefficients
result_alt$relationship
```

The `P1` column in `$parallel` is the characteristic life $\eta$ at each stress level (for Weibull) or $\exp(\mu_{\log})$ (for lognormal). The `P2` column is the shape parameter $\beta$, constrained to be equal across stress levels. The `$relationship` coefficients `coef1` and `coef2` define the log-linear life-stress model [@Nelson1990].

## Plot Tools

The three `plot_*` tools run the full fitting pipeline and return an interactive plot as a [plotly](https://plotly.com/r/) JSON string. Claude Code (or any MCP client) can save the string to an `.html` file that opens directly in a browser.

### The `plot_weibull` Tool

`plot_weibull` accepts the same arguments as `fit_weibull` plus an optional `show_conf` flag, and returns the probability plot as a JSON string.

```{r plot-wblr-example, eval = has_ellmer}
ptool <- ReliaPlotR:::.make_plot_wblr_tool()
json_str <- ptool(failures = c(30, 49, 82, 90, 96))
nchar(json_str)  # plotly JSON string length
```

### The `plot_alt` Tool

`plot_alt` accepts the same arguments as `fit_alt` and returns a named list with two JSON strings: `probability_plot` (one fitted line per stress level) and `life_stress_plot` (Arrhenius or Power Law relationship).

```{r plot-alt-example, eval = has_ellmer, results = "hide"}
ptool_alt <- ReliaPlotR:::.make_plot_alt_tool()
result_plots <- ptool_alt(
  stresses      = c(300, 350, 400),
  failures_json = "[[248,456,528,731,813,537],[164,176,289],[88,112,152]]",
  dist          = "weibull",
  alt_model     = "arrhenius"
)
```

```{r plot-alt-show, eval = has_ellmer}
nchar(result_plots$probability_plot)
nchar(result_plots$life_stress_plot)
```

### The `plot_rga` Tool

`plot_rga` fits a Crow-AMSAA model to failure count data and returns the cumulative failure plot.

```{r plot-rga-example, eval = has_ellmer}
ptool_rga <- ReliaPlotR:::.make_plot_rga_tool()
json_rga <- ptool_rga(
  times    = c(100, 200, 300, 400, 500),
  failures = c(  2,   1,   3,   1,   2)
)
nchar(json_rga)
```

## How the Tools Work

Each MCP tool wraps the same fitting pipeline used by the corresponding `plotly_*` function:

```
fit_weibull:    wblr() -> wblr.fit() -> wblr.conf() -> tidy_wblr()    -> named list
fit_alt:     alt.make() -> alt.parallel() -> alt.fit() -> tidy_alt()  -> named list
plot_weibull:   wblr() -> wblr.fit() -> wblr.conf() -> plotly_wblr()  -> plotly JSON
plot_alt:    alt.make() -> alt.parallel() -> alt.fit() -> plotly_alt()
                                                       -> plotly_rel() -> plotly JSON × 2
plot_rga:       rga()                                 -> plotly_rga()  -> plotly JSON
```

The accessor functions (`tidy_wblr()`, `tidy_alt()`, `tidy_rga()`) can also be called directly from R to extract parameter estimates from already-fitted model objects without going through MCP.

## ReliaGrowR MCP Tools

The ReliaGrowR server exposes eight tools for reliability growth analysis and repairable systems. Register it with `claude mcp add -s user reliagrowR -- Rscript -e "ReliaGrowR::rga_mcp_server()"`.

| Tool | Arguments | Returns |
|---|---|---|
| `rga` | `times`, `failures`, `times_type`, `method` | λ, β, growth rate, log-likelihood, AIC, BIC |
| `nhpp` | `time`, `event`, `model_type`, `method` | model type, method, fitted parameters, GOF stats |
| `duane` | `times`, `failures`, `conf_level` | slope, intercept, GOF stats |
| `mcf` | `id`, `time`, `event`, `conf_level` | time points, MCF values, confidence bounds |
| `predict_rga` | `times`, `failures`, `forecast_times`, `method`, `conf_level` | forecast cumulative failures with bounds |
| `predict_duane` | `times`, `failures`, `forecast_times`, `conf_level` | forecast MTBF with bounds |
| `rdt` | `target`, `mission_time`, `conf_level`, `beta`, `f`, `n`, `test_time` | required sample size or test duration |
| `gof_rga` | `times`, `failures`, `method` | Cramér-von Mises (W²) and Kolmogorov-Smirnov (D) statistics |

## References

## See Also

- [Life Data Analysis](weibull.html) -- fit Weibull/lognormal models interactively
- [Accelerated Life Testing](alt.html) -- full ALT workflow with probability and life-stress plots
- [Reliability Growth Analysis](rga.html) -- Crow-AMSAA and Duane models
- [Repairable Systems Analysis](repairable.html) -- MCF and NHPP models

---
title: "Accelerated Life Testing (ALT)"
output:
  rmarkdown::html_vignette:
    fig_width: 7
    fig_height: 5
    self_contained: false
bibliography: ../inst/REFERENCES.bib
vignette: >
  %\VignetteIndexEntry{Accelerated Life Testing (ALT)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(ReliaPlotR)
library(WeibullR)
library(WeibullR.ALT)
```

## Statistical Background

**Accelerated Life Testing (ALT)** subjects units to stress levels (temperature, voltage, vibration, etc.) higher than use conditions in order to induce failures faster. Life data collected at multiple stress levels are then extrapolated to the use-condition stress using a physics-motivated life-stress model [@Nelson1990].

**Life distributions.** At each stress level, failure times are modeled with a Weibull or lognormal distribution. The *shape* parameter ($\beta$ for Weibull; $\sigma_{\log}$ for lognormal) is assumed constant across stress levels (the distributions are parallel on probability paper), while the *scale* parameter ($\eta$ for Weibull; $\exp(\mu_{\log})$ for lognormal) varies with stress according to the life-stress model.

**Life-stress models.** Two models are supported:

- **Arrhenius**: $\eta(S) = A \exp(E_a / (k S))$, where $S$ is temperature in Kelvin and $E_a / k$ is the activation energy divided by Boltzmann's constant. The x-axis uses a reciprocal scale so the relationship is linear.
- **Power Law (Inverse Power)**: $\eta(S) = A / S^n$, where $S$ is a non-thermal stress. Both axes are log-transformed so the relationship is linear.

**Fitting.** `alt.parallel()` fits independent Weibull/lognormal models at each stress level; `alt.fit()` then fits the global life-stress relationship by constraining the shape parameter to be equal across stress levels [@Nelson1990; @MeekerEscobar1998].

## Building an ALT Model

The `WeibullR.ALT` package uses a three-step pipeline to create an ALT model.

**Step 1 — Create data sets** for each stress level using `alt.data()`:

```{r data}
d1 <- alt.data(c(248, 456, 528, 731, 813, 537), stress = 300)
d2 <- alt.data(c(164, 176, 289), stress = 350)
d3 <- alt.data(c(88, 112, 152), stress = 400)
```

**Step 2 — Fit parallel models** across stress levels using `alt.make()` and `alt.parallel()`:

```{r parallel, results = "hide"}
obj <- alt.parallel(
  alt.make(list(d1, d2, d3), dist = "weibull", alt.model = "arrhenius", view_dist_fits = FALSE),
  view_parallel_fits = FALSE
)
```

**Step 3 — Fit the life-stress relationship** using `alt.fit()`:

```{r fit, results = "hide"}
obj <- alt.fit(obj)
```

## ALT Probability Plot

`plotly_alt()` overlays one probability-paper fit line per stress level. Data points show empirical plotting positions; lines show the theoretical Weibull (or lognormal) fit. Click a legend entry to toggle a stress level on or off.

```{r prob-plot}
plotly_alt(obj)
```

### Customization

The plot accepts several optional arguments:

```{r prob-custom}
plotly_alt(
  obj,
  main    = "Reliability Test Results",
  xlab    = "Hours to Failure",
  cols    = c("#1f77b4", "#ff7f0e", "#2ca02c"),
  showGrid = FALSE
)
```

## Life-Stress Relationship Plot

`plotly_rel()` displays how characteristic life (eta for Weibull, median life for lognormal) changes with stress level, along with the fitted Arrhenius or Power Law relationship.

```{r rel-plot}
plotly_rel(obj)
```

The plot includes:

- **Points** (×) — characteristic-life estimates from parallel fits at each stress level
- **Red line** — fitted life-stress relationship (Arrhenius or Power Law)
- **Dashed blue lines** — percentile bands (10th and 90th by default)

### Percentile Bands

Use the `percentiles` argument to change which percentile bands are shown:

```{r rel-percentiles}
plotly_rel(obj, percentiles = c(5, 50, 95))
```

### Hiding Percentile Lines

Set `showPerc = FALSE` to show only the fitted relationship line:

```{r rel-no-perc}
plotly_rel(obj, showPerc = FALSE)
```

### Customization

```{r rel-custom}
plotly_rel(
  obj,
  main    = "Arrhenius Life-Stress Relationship",
  fitCol  = "darkgreen",
  percCol = "steelblue",
  signif  = 4
)
```

## References

## See Also

- [Life Data Analysis](weibull.html) — fit Weibull/lognormal models at a single stress level
- [Reliability Growth Analysis](rga.html) — track reliability improvement over time
- [Repairable Systems Analysis](repairable.html) — analyze recurrent events with MCF and NHPP models

---
title: "Repairable Systems Analysis"
output:
  rmarkdown::html_vignette:
    fig_width: 7
    fig_height: 5
    self_contained: false
bibliography: ../inst/REFERENCES.bib
vignette: >
  %\VignetteIndexEntry{repairable}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

To run a repairable systems analysis, start by loading `ReliaGrowR` and `ReliaPlotR`:

```{r setup, warning=FALSE}
library(ReliaGrowR)
library(ReliaPlotR)
```

## Statistical Background

**Repairable systems** experience multiple failure events over their lifetime. Standard survival analysis (Kaplan-Meier, Weibull) applies to the *first* failure; repairable-systems methods model the entire *recurrence* process [@Nelson2003].

The **Mean Cumulative Function (MCF)**, $M(t) = E[N(t)]$, is the expected cumulative number of repairs per system by time $t$. The nonparametric Nelson-Aalen estimator

$$\hat{M}(t) = \sum_{t_i \leq t} \frac{d_i}{r_i},$$

where $d_i$ is the number of events at time $t_i$ and $r_i$ is the number of systems at risk (still under observation), provides an unbiased estimate that properly accounts for systems observed only up to a censoring time [@Nelson2003]. This differs fundamentally from the Kaplan-Meier estimator: Kaplan-Meier estimates survival to *first* event; MCF sums expected *recurrences* and can exceed 1.

The **NHPP Power Law** (Crow-AMSAA) model fits a parametric intensity function $\lambda(t) = \lambda \beta t^{\beta - 1}$ to the recurrent process. It is the repairable-systems analog of the Crow-AMSAA model for development testing: $\beta < 1$ indicates improving reliability; $\beta > 1$ indicates deterioration; $\beta = 1$ is a homogeneous Poisson process [@Crow1974].

The examples below use a fleet of five field units. Each row records which unit failed and when:

```{r echo=TRUE}
id    <- c("U1","U1","U1","U2","U2","U3","U3","U3","U4","U5","U5")
time  <- c( 120, 310, 560, 200, 480, 95, 280, 430, 370, 150, 490)
```

## Mean Cumulative Function

The Mean Cumulative Function (MCF) is a non-parametric estimate of the expected cumulative number of failures per system by time *t*. It is computed using the Nelson-Aalen estimator, which accounts for systems that are still operating (censored) at the end of the observation window.

```{r echo=TRUE}
end_times <- c(U1 = 600, U2 = 600, U3 = 600, U4 = 600, U5 = 600)
fit_mcf   <- mcf(id = id, time = time, end_time = end_times)
plotly_mcf(fit_mcf)
```

The shaded band shows the pointwise confidence interval. Hover over the step function to read exact MCF values at each event time.

## Exposure Analysis

The exposure plot tracks the cumulative event rate — total failures divided by total system-time — over the observation period. A rising rate indicates a deteriorating fleet; a flat or falling rate indicates stability or improvement.

```{r}
fit_exp <- exposure(id = id, time = time)
plotly_exposure(fit_exp)
```

## NHPP — Power Law Model

The Non-Homogeneous Poisson Process (NHPP) fits a parametric model to the recurrent failure process. The Power Law model (also called the Crow-AMSAA model for repairable systems) describes how the failure intensity changes over time:

$$\lambda(t) = \lambda \beta \, t^{\beta - 1}$$

A shape parameter $\beta < 1$ indicates reliability improvement; $\beta > 1$ indicates deterioration; $\beta = 1$ reduces to a homogeneous Poisson process.

```{r}
times  <- c(120, 200, 310, 370, 430, 480, 490, 560)
events <- c(  1,   1,   1,   1,   1,   1,   1,   1)
fit_nhpp <- nhpp(time = times, event = events, model_type = "Power Law")
plotly_nhpp(fit_nhpp)
```

## NHPP — Piecewise Model

When a corrective action or design change is introduced mid-program, the failure process may shift. A piecewise NHPP fits separate Power Law segments on either side of a specified breakpoint, shown as a vertical dotted line:

```{r}
# Dense failures before the design change (t < 500), sparse afterwards
times2  <- c(30, 65, 105, 150, 200, 255, 315, 380, 450, 800, 1300, 1950, 2800)
events2 <- rep(1, 13)
fit_nhpp2 <- nhpp(time = times2, event = events2, breaks = 500, method = "LS")
plotly_nhpp(fit_nhpp2, fitCol = "steelblue", confCol = "steelblue", breakCol = "red")
```

## Overlay

Multiple MCF or NHPP estimates can be overlaid on the same plot by passing a list of objects. This is useful for comparing two fleets, two time periods, or a before/after scenario.

```{r}
id2   <- c("A","A","B","B","B","C")
time2 <- c(180, 420, 90, 310, 530, 260)
end2  <- c(A = 600, B = 600, C = 600)
fit_mcf2 <- mcf(id = id2, time = time2, end_time = end2)

plotly_mcf(list(fit_mcf, fit_mcf2), cols = c("steelblue", "tomato"))
```

## References

## See Also

- [Life Data Analysis](weibull.html) — fit Weibull/lognormal models to non-repairable failure-time data
- [Reliability Growth Analysis](rga.html) — track cumulative failures over development test time
- [Accelerated Life Testing](alt.html) — extrapolate life from accelerated stress conditions

---
title: "Reliability Growth Analysis"
output:
  rmarkdown::html_vignette:
    fig_width: 7
    fig_height: 5
    self_contained: false
bibliography: ../inst/REFERENCES.bib
vignette: >
  %\VignetteIndexEntry{rga}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

To run a Reliability Growth Analysis, start by loading `ReliaGrowR` and `ReliaPlotR`

```{r setup, warning=FALSE}
library(ReliaGrowR)
library(ReliaPlotR)
```

## Statistical Background

The **Crow-AMSAA** model (Non-Homogeneous Poisson Process, Power Law) describes the expected cumulative failures as

$$E[N(t)] = \lambda t^{\beta},$$

where $\lambda > 0$ is a scale parameter and $\beta > 0$ is the shape parameter. A value $\beta < 1$ indicates **reliability growth** (the failure rate $\lambda\beta t^{\beta-1}$ is decreasing over time); $\beta > 1$ indicates degradation; $\beta = 1$ gives a homogeneous Poisson process (constant rate). Parameters are estimated by fitting a log-log linear regression to cumulative failures versus cumulative test time [@Crow1974].

The **Duane model** predates the NHPP formulation and postulates that the cumulative mean time between failures follows a power law in cumulative test time: $\text{CMTBF}(T) = K T^\alpha$, where $\alpha$ is the growth rate. Empirically, managed reliability growth programs tend to achieve $\alpha \approx 0.3$–$0.5$ [@Duane1964].

**Piecewise NHPP.** When a design change or corrective action divides the test program into distinct phases, separate Power Law parameters are fit within each interval defined by the specified `breaks`. A vertical dotted line marks each change point.

## Crow-AMSAA Model

To run a Crow-AMSAA model, first set up some cumulative time and failure data:
```{r}
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
```

Then run the rga and plot the results:
```{r}
result <- rga(times, failures)
plotly_rga(result)
```

## Piecewise NHPP Model

To run a Piecewise NHPP, first set up some cumulative time/failure data and specify the breakpoint:
```{r}
times <- c(25, 55, 97, 146, 201, 268, 341, 423, 513, 609, 710, 820, 940, 1072, 1217)
failures <- c(1, 1, 2, 4, 4, 1, 1, 2, 1, 4, 1, 1, 3, 3, 4)
breaks <- 500
```

Then run the rga and plot the results:
```{r}
result <- rga(times, failures, model_type = "Piecewise NHPP", breaks = breaks)
plotly_rga(result, fitCol = "blue", confCol = "blue", breakCol = "red")
```

## Duane Model

To run a Duane Model, first set up some cumulative time and failure data:
```{r}
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
```

Then plot the results:
```{r}
fit <- duane(times, failures)
plotly_duane(fit, fitCol = "darkgreen", confCol = "darkgreen")
```

## Overlay Models

Multiple `rga` models can be overlaid on the same plot by passing a list of objects. Each model is rendered in a distinct color, and clicking a legend entry toggles all traces for that model.

```{r}
times1 <- c(100, 200, 300, 400, 500)
failures1 <- c(1, 2, 1, 3, 2)
result1 <- rga(times1, failures1)

times2 <- c(50, 150, 250, 350, 450)
failures2 <- c(2, 1, 3, 1, 2)
result2 <- rga(times2, failures2)

plotly_rga(list(result1, result2), cols = c("steelblue", "tomato"))
```

## References

## See Also

- [Life Data Analysis](weibull.html) — fit Weibull/lognormal models to failure-time data
- [Repairable Systems Analysis](repairable.html) — analyze recurrent events with MCF and NHPP models
- [Accelerated Life Testing](alt.html) — extrapolate life from accelerated stress conditions

---
title: "Life Data Analysis"
output:
  rmarkdown::html_vignette:
    fig_width: 7
    fig_height: 5
    self_contained: false
bibliography: ../inst/REFERENCES.bib
vignette: >
  %\VignetteIndexEntry{weibull}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

To run a Weibull Analysis, start by loading `WeibullR` and `ReliaPlotR`

```{r setup}
library(WeibullR)
library(ReliaPlotR)
```

## Statistical Background

The two-parameter Weibull distribution has CDF

$$F(t) = 1 - \exp\!\left[-\left(\frac{t}{\eta}\right)^{\!\beta}\right], \quad t > 0,$$

where $\beta > 0$ is the shape (slope on probability paper) and $\eta > 0$ is the scale (characteristic life, the time at which 63.2% of units have failed). A shape $\beta < 1$ indicates infant-mortality failure (decreasing hazard rate); $\beta = 1$ reduces to the exponential distribution (constant hazard); $\beta > 1$ indicates wearout (increasing hazard). The three-parameter extension adds a location parameter $\gamma$ (failure-free period): $F(t) = 1 - \exp[-((t - \gamma)/\eta)^\beta]$.

For the lognormal distribution, $\ln(T) \sim \mathcal{N}(\mu_{\log}, \sigma_{\log}^2)$; the probability paper transformation uses the standard normal quantile $\Phi^{-1}(F)$.

**Fitting methods.** Rank regression (default in WeibullR, `method.fit = "rr-xony"`) linearizes the CDF on probability paper and minimizes squared deviations. Maximum likelihood estimation (`method.fit = "mle"`) maximizes the log-likelihood and is preferred for heavily censored data. Goodness-of-fit is reported as R² for rank regression and log-likelihood for MLE [@MeekerEscobar1998].

**Confidence intervals.** The Fisher matrix method (`method.conf = "fm"`) approximates confidence bounds via the asymptotic normal distribution of MLE estimates. Likelihood ratio bounds (`method.conf = "lrb"`) invert a chi-squared test and are generally more accurate for small samples [@MeekerEscobar1998].

## A Basic Example

Next, create some failure data for 5 different machines that fail at time 30, 49, 82, 90, and 96 respectively. 

```{r echo=TRUE}
failures <- c(30, 49, 82, 90, 96)
```

Then use the `WeibullR` package to fit a Weibull distribution to the data, and the `plotly_wblr()` function to create a probability plot.

```{r echo=TRUE}
obj <- wblr.conf(wblr.fit(wblr(failures)))
plotly_wblr(obj)
```

## Right Censored Model

Let's add right censored data to the previous example. In addition to the 5 machines that failed, add suspensions for 3 machines that did not fail (right censored) at times 100, 45, and 10 respectively.

```{r echo=TRUE}
suspensions <- c(100, 45, 10)
obj <- wblr.conf(wblr.fit(wblr(failures, suspensions)))
plotly_wblr(obj, suspensions, fitCol = "blue", confCol = "blue")
```

## Interval Censored Model

To create an interval censored model, let's use the inspection data from Silkworth, 2020.

```{r echo=TRUE}
inspection_data <- data.frame(
  left = c(0, 6.12, 19.92, 29.64, 35.4, 39.72, 45.32, 52.32),
  right = c(6.12, 19.92, 29.64, 35.4, 39.72, 45.32, 52.32, 63.48),
  qty = c(5, 16, 12, 18, 18, 2, 6, 17)
)
```

Then add suspension data for units surviving until the last inspection date.

```{r echo=TRUE}
suspensions <- data.frame(time = 63.48, event = 0, qty = 73)
```

Finally, add a fit and plot the results.

```{r echo=TRUE, warning=FALSE}
obj <- wblr(suspensions, interval = inspection_data)
obj <- wblr.fit(obj, method.fit = "mle")
obj <- wblr.conf(obj, method.conf = "fm", lty = 2)
suspensions <- as.vector(suspensions$time)
plotly_wblr(obj,
  susp = suspensions, fitCol = "red", confCol = "red", intCol = "blue",
  main = "Parts Cracking Inspection Interval Analysis",
  ylab = "Cumulative % Cracked", xlab = "Inspection Time"
)
```

## 3-Parameter Weibull Model

To fit a 3P Weibull, let's create some new failure data and plot the results.

```{r echo=TRUE}
failures <- c(25, 30, 42, 49, 55, 67, 73, 82, 90, 96, 101, 110, 120, 132, 145)
fit <- wblr.conf(wblr.fit(wblr(failures), dist = "weibull3p"))
plotly_wblr(fit, fitCol = "darkgreen", confCol = "darkgreen")
```

## Contour Plots

To build a contour plot, let's rerun the first example and use the `plotly_contour()` function to create a plot.

```{r echo=TRUE}
failures <- c(30, 49, 82, 90, 96)
obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
plotly_contour(obj, col = "blue")
```

## Overlay Models

Multiple `wblr` models can be overlaid on the same probability plot by passing a list of objects. All objects must use the same distribution family. Each model is rendered in a distinct color, and clicking a legend entry toggles all traces for that model.

```{r echo=TRUE}
failures1 <- c(30, 49, 82, 90, 96)
failures2 <- c(20, 40, 60, 80, 100)
obj1 <- wblr.conf(wblr.fit(wblr(failures1), method.fit = "mle"), method.conf = "lrb")
obj2 <- wblr.conf(wblr.fit(wblr(failures2), method.fit = "mle"), method.conf = "lrb")
plotly_wblr(list(obj1, obj2), cols = c("steelblue", "tomato"))
```

## References

## See Also

- [Accelerated Life Testing](alt.html) — apply Weibull/lognormal models across multiple stress levels
- [Reliability Growth Analysis](rga.html) — track cumulative failures over development test time
- [Repairable Systems Analysis](repairable.html) — analyze recurrent events with MCF and NHPP models

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotly_rel.R
\name{plotly_rel}
\alias{plotly_rel}
\title{Interactive ALT Life-Stress Relationship Plot.}
\usage{
plotly_rel(
  alt_obj,
  showPerc = TRUE,
  showGoal = TRUE,
  showGrid = TRUE,
  main = "Life-Stress Relationship",
  xlab = "Stress",
  ylab = "Time to Failure",
  fitCol = "red",
  ptCol = "black",
  percCol = "blue",
  goalCol = "orange",
  gridCol = "lightgray",
  signif = 3,
  percentiles = c(10, 90)
)
}
\arguments{
\item{alt_obj}{An object of class `'alt'` created by the `WeibullR.ALT` package
and fitted with `alt.fit()`.}

\item{showPerc}{Show percentile lines with shading (TRUE) or not (FALSE).
Default is TRUE.}

\item{showGoal}{Show the goal-condition marker when one is present in `alt_obj`
(TRUE) or not (FALSE). Default is TRUE.}

\item{showGrid}{Show grid (TRUE) or hide grid (FALSE). Default is TRUE.}

\item{main}{Main title. Default is "Life-Stress Relationship".}

\item{xlab}{X-axis label. Default is "Stress".}

\item{ylab}{Y-axis label. Default is "Time to Failure".}

\item{fitCol}{Color of the fitted relationship line. Default is "red".}

\item{ptCol}{Color of the characteristic-life scatter points. Default is "black".}

\item{percCol}{Color of the percentile lines and fill. Default is "blue".}

\item{goalCol}{Color of the goal-condition marker. Default is "orange".}

\item{gridCol}{Color of the grid. Default is "lightgray".}

\item{signif}{Significant digits for hover text. Default is 3.}

\item{percentiles}{Numeric vector of percentiles to draw as reference lines
with shading. Sorted internally; shading is filled between adjacent lines.
Default is `c(10, 90)`.}
}
\value{
A `plotly` object representing the interactive life-stress plot.
}
\description{
Creates an interactive life-stress relationship (Arrhenius or Power Law) plot
for an `alt` object. Displays the characteristic-life estimates per stress
level, the fitted relationship line, optional percentile bands with shading,
and an optional goal-condition marker. The `alt` object must have been
processed through [WeibullR.ALT::alt.fit()] before passing to this function.
}
\details{
Two life-stress models are supported:
\itemize{
  \item \strong{Arrhenius}: \eqn{\eta = A \exp(E_a / (k T))}, where stress is
    temperature in Kelvin and the x-axis uses a reciprocal scale.
  \item \strong{Power Law}: \eqn{\eta = A / S^n}, where the x-axis is
    log-transformed.
}
The \code{percentiles} argument controls which quantile lines are drawn
with shading between adjacent lines. The goal-condition marker
(\code{showGoal}) is drawn when the \code{alt} object contains a
\code{$goal} specification from \code{alt.make()}.
}
\examples{
library(WeibullR.ALT)
d1 <- alt.data(c(248, 456, 528, 731, 813, 537), stress = 300)
d2 <- alt.data(c(164, 176, 289), stress = 350)
d3 <- alt.data(c(88, 112, 152), stress = 400)
obj <- alt.fit(
  alt.parallel(
    alt.make(list(d1, d2, d3), dist = "weibull", alt.model = "arrhenius", view_dist_fits = FALSE),
    view_parallel_fits = FALSE
  )
)
plotly_rel(obj)
}
\seealso{
[plotly_alt()] for the ALT probability plot using the same \code{alt} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotly_alt.R
\name{plotly_alt}
\alias{plotly_alt}
\title{Interactive ALT Probability Plot.}
\usage{
plotly_alt(
  alt_obj,
  showConf = TRUE,
  showGrid = TRUE,
  main = "ALT Probability Plot",
  xlab = "Time to Failure",
  ylab = "Probability",
  gridCol = "lightgray",
  signif = 3,
  cols = NULL
)
}
\arguments{
\item{alt_obj}{An object of class `'alt'` created by the `WeibullR.ALT` package
and fitted with `alt.parallel()`.}

\item{showConf}{Show Fisher-matrix confidence bounds (TRUE) or not (FALSE).
Default is TRUE.}

\item{showGrid}{Show grid (TRUE) or hide grid (FALSE). Default is TRUE.}

\item{main}{Main title. Default is "ALT Probability Plot".}

\item{xlab}{X-axis label. Default is "Time to Failure".}

\item{ylab}{Y-axis label. Default is "Probability".}

\item{gridCol}{Color of the grid. Default is "lightgray".}

\item{signif}{Significant digits for hover text. Default is 3.}

\item{cols}{Optional character vector of colors, one per stress level. Recycled
to match the number of stress levels. When NULL a 10-color default palette is used.}
}
\value{
A `plotly` object representing the interactive ALT probability plot.
}
\description{
Creates an interactive probability plot for an `alt` object, overlaying one
Weibull or lognormal fit line per stress level on a shared probability paper.
The `alt` object must have been processed through [WeibullR.ALT::alt.parallel()]
before passing to this function.
}
\details{
Building an \code{alt} object requires three steps:
\enumerate{
  \item \code{alt.make()} — bundle the per-stress data sets and choose a
    distribution (\code{"weibull"} or \code{"lognormal"}) and life-stress
    model (\code{"arrhenius"} or \code{"power"}).
  \item \code{alt.parallel()} — fit independent Weibull/lognormal models
    at each stress level (required before calling \code{plotly_alt()}).
  \item \code{alt.fit()} — fit the global life-stress relationship
    (required for \code{plotly_rel()}).
}
The probability paper transformation (log y-axis for Weibull, normal
quantile for lognormal) is the same as in [plotly_wblr()].
}
\examples{
library(WeibullR.ALT)
d1 <- alt.data(c(248, 456, 528, 731, 813, 537), stress = 300)
d2 <- alt.data(c(164, 176, 289), stress = 350)
d3 <- alt.data(c(88, 112, 152), stress = 400)
obj <- alt.fit(
  alt.parallel(
    alt.make(list(d1, d2, d3), dist = "weibull", alt.model = "arrhenius", view_dist_fits = FALSE),
    view_parallel_fits = FALSE
  )
)
plotly_alt(obj)
}
\seealso{
[plotly_rel()] for the life-stress relationship plot using the same \code{alt} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReliaPlotR-package.R
\docType{package}
\name{ReliaPlotR-package}
\alias{ReliaPlotR}
\alias{ReliaPlotR-package}
\title{ReliaPlotR: Interactive Reliability Plots with plotly}
\description{
ReliaPlotR creates interactive reliability probability plots using
\href{https://plotly.com/r/}{plotly}. It wraps the \pkg{WeibullR},
\pkg{WeibullR.ALT}, and \pkg{ReliaGrowR} packages and provides a consistent
interface for four reliability analysis domains:
}
\details{
\describe{
  \item{Life Data Analysis}{[plotly_wblr()], [plotly_contour()]}
  \item{Accelerated Life Testing}{[plotly_alt()], [plotly_rel()]}
  \item{Reliability Growth}{[plotly_rga()], [plotly_duane()]}
  \item{Repairable Systems}{[plotly_nhpp()], [plotly_mcf()], [plotly_exposure()]}
}

All functions accept a single fitted model object or a list of objects
(for overlay plots) and return a \code{plotly} object.
}
\seealso{
Vignettes:
\itemize{
  \item \code{vignette("weibull", package = "ReliaPlotR")} — Life Data Analysis
  \item \code{vignette("alt", package = "ReliaPlotR")} — Accelerated Life Testing
  \item \code{vignette("rga", package = "ReliaPlotR")} — Reliability Growth Analysis
  \item \code{vignette("repairable", package = "ReliaPlotR")} — Repairable Systems Analysis
}
}
\author{
\strong{Maintainer}: Paul Govan \email{paul.govan2@gmail.com} (\href{https://orcid.org/0000-0002-1821-8492}{ORCID}) [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcp_tools.R
\name{reliapltr_mcp_server}
\alias{reliapltr_mcp_server}
\title{Launch a ReliaPlotR MCP Server.}
\usage{
reliapltr_mcp_server(...)
}
\arguments{
\item{...}{Additional arguments passed to [mcptools::mcp_server()], such as
`type = "stdio"` (default) or `type = "http"`.}
}
\value{
Called for its side effect of launching a blocking MCP server
  process.
}
\description{
Starts a Model Context Protocol (MCP) server that exposes five reliability
analysis tools to MCP clients such as Claude Code and Claude Desktop.
}
\details{
\describe{
  \item{`fit_weibull`}{Fits a Weibull or lognormal distribution to
    time-to-failure data (optionally with right-censored suspensions) and
    returns parameter estimates and goodness-of-fit metrics.}
  \item{`fit_alt`}{Fits an Accelerated Life Test (ALT) model across
    multiple stress levels and returns per-stress-level parameters and the
    global life-stress relationship coefficients.}
  \item{`plot_weibull`}{Fits a Weibull or lognormal distribution and returns
    a Weibull probability plot as a plotly JSON string.}
  \item{`plot_alt`}{Fits an ALT model and returns probability and life-stress
    relationship plots as plotly JSON strings.}
  \item{`plot_rga`}{Fits a Crow-AMSAA reliability growth model and returns a
    cumulative failure plot as a plotly JSON string.}
}


Both `mcptools` (>= 0.2.0) and `ellmer` must be installed. They are listed
in `Suggests` and are not automatically installed with ReliaPlotR.
`jsonlite` is also required for the `fit_alt` tool.

**Registering with Claude Code:**
```bash
claude mcp add -s user reliapltr -- \
  Rscript -e "ReliaPlotR::reliapltr_mcp_server()"
```

Alternatively, use the bundled launcher script:
```bash
claude mcp add -s user reliapltr -- \
  Rscript /path/to/ReliaPlotR/mcp/server.R
```
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotly_duane.R
\name{plotly_duane}
\alias{plotly_duane}
\title{Interactive Duane Plot.}
\usage{
plotly_duane(
  duane_obj,
  showConf = TRUE,
  showGrid = TRUE,
  main = "Duane Plot",
  xlab = "Cumulative Time",
  ylab = "Cumulative MTBF",
  pointCol = "black",
  fitCol = "black",
  confCol = "black",
  gridCol = "lightgray",
  signif = 3
)
}
\arguments{
\item{duane_obj}{An object of class 'duane'. This object is created
using the `duane` function from the ReliaGrowR package.}

\item{showConf}{Show the confidence bounds (TRUE) or not (FALSE). Default is TRUE.}

\item{showGrid}{Show grid (TRUE) or hide grid (FALSE). Default is TRUE.}

\item{main}{Main title. Default is "Duane Plot".}

\item{xlab}{X-axis label. Default is "Cumulative Time".}

\item{ylab}{Y-axis label. Default is "Cumulative MTBF".}

\item{pointCol}{Color of the point values. Default is "black".}

\item{fitCol}{Color of the model fit. Default is "black".}

\item{confCol}{Color of the confidence bounds. Default is "black".}

\item{gridCol}{Color of the grid. Default is "lightgray".}

\item{signif}{Significant digits of results. Default is 3. Must be a positive integer.}
}
\value{
A `plotly` object representing the interactive Duane plot.
}
\description{
This function creates an interactive Duane plot for a \code{duane} object.
The plot displays observed cumulative MTBF against cumulative test time on
a log-log scale alongside the fitted Duane line and optional confidence
bounds. Positive slope on the log-log scale indicates reliability growth.
}
\details{
The Duane model fits a power-law relationship between cumulative MTBF and
cumulative test time: \eqn{\text{MTBF}_c = K \cdot T^\alpha}. On a
log-log plot this appears as a straight line with slope \eqn{\alpha} (the
growth rate) and intercept \eqn{\log(K)}. A slope near 0 indicates no
growth; a slope near 0.5 is typical for a managed development program.
}
\examples{
library(ReliaGrowR)
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
fit <- duane(times, failures)
plotly_duane(fit)
}
\seealso{
[plotly_rga()] for the Crow-AMSAA NHPP growth model on the same data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_rga.R
\name{tidy_rga}
\alias{tidy_rga}
\title{Extract Tidy Fitted Values from an rga Object.}
\usage{
tidy_rga(rga_obj)
}
\arguments{
\item{rga_obj}{An object of class `'rga'`, or a list of such objects. Each
object is created using the [ReliaGrowR::rga()] function.}
}
\value{
A named list with two elements:
  \describe{
    \item{`fitted`}{For a single object, a `data.frame` with columns
      `time`, `cum_failures`, `fitted`, `lower`, `upper`. For a list, a
      list of such data frames (one per object).}
    \item{`params`}{For a single object, a one-row `data.frame` with columns
      `lambda` and `beta` (Crow-AMSAA Power Law parameters), or `NULL` if
      the model coefficients cannot be extracted. For a list, a list of such
      data frames.}
  }
}
\description{
Returns a tidy data frame of fitted cumulative failure counts, confidence
bounds, and (when available) Crow-AMSAA model coefficients from an `rga`
object. Suitable for exporting results for use in reproducible research
workflows.
}
\details{
The Crow-AMSAA (NHPP Power Law) model gives the expected cumulative
failures as \eqn{E[N(t)] = \lambda t^\beta}. A \eqn{\beta < 1} indicates
reliability growth (decreasing failure rate); \eqn{\beta > 1} indicates
degradation. The parameters are recovered from the fitted log-log linear
model via \code{coef()}: \eqn{\lambda = \exp(\text{intercept})} and
\eqn{\beta} is the slope coefficient.
}
\examples{
library(ReliaGrowR)
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
obj <- rga(times, failures)
result <- tidy_rga(obj)
result$fitted
result$params
}
\references{
Crow, L. H. (1974). Reliability Analysis for Complex Repairable Systems.
In \emph{Reliability and Biometry}, SIAM, pp. 379-410.
}
\seealso{
[plotly_rga()] for the corresponding interactive reliability growth plot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_alt.R
\name{tidy_alt}
\alias{tidy_alt}
\title{Extract Tidy Parameter Estimates from an alt Object.}
\usage{
tidy_alt(alt_obj)
}
\arguments{
\item{alt_obj}{An object of class `'alt'` created by the `WeibullR.ALT`
package. Must have been processed through [WeibullR.ALT::alt.parallel()].
For `$relationship` to be non-NULL, [WeibullR.ALT::alt.fit()] must also
have been called.}
}
\value{
A named list with two elements:
  \describe{
    \item{`parallel`}{A `data.frame` with one row per stress level,
      containing columns `stress`, `P1`, `P2`, `wt`, and `n_failures`.
      For Weibull models, `P1` is the scale parameter \eqn{\eta} and `P2`
      is the shape parameter \eqn{\beta}. For lognormal models, `P1` is
      \eqn{\mu_{log}} and `P2` is \eqn{\sigma_{log}}.}
    \item{`relationship`}{A one-row `data.frame` with columns `model`,
      `coef1`, and `coef2` (the global life-stress relationship
      coefficients), or `NULL` with a message if `alt.fit()` has not been
      called.}
  }
}
\description{
Returns tidy data frames of per-stress-level parameter estimates and (when
available) global life-stress relationship coefficients from a fitted `alt`
object. Useful for reporting ALT results in reproducible research workflows.
}
\details{
Two life-stress models are supported. For the Arrhenius model,
\eqn{\eta = \exp(C_1 + C_2 \cdot S)} where \eqn{S} is stress (typically
reciprocal temperature in 1/K). For the Power Law model,
\eqn{\eta = \exp(C_1) / S^{|C_2|}}. Both \code{coef1} and \code{coef2}
are on the log scale as returned by \code{alt.fit()}.
}
\examples{
library(WeibullR.ALT)
d1 <- alt.data(c(248, 456, 528, 731, 813, 537), stress = 300)
d2 <- alt.data(c(164, 176, 289), stress = 350)
d3 <- alt.data(c(88, 112, 152), stress = 400)
obj <- alt.fit(
  alt.parallel(
    alt.make(list(d1, d2, d3), dist = "weibull", alt.model = "arrhenius",
             view_dist_fits = FALSE),
    view_parallel_fits = FALSE
  )
)
result <- tidy_alt(obj)
result$parallel
result$relationship
}
\references{
Nelson, W. B. (1990). \emph{Accelerated Testing: Statistical Models,
Test Plans, and Data Analysis}. Wiley.
}
\seealso{
[plotly_alt()] for the ALT probability plot,
  [plotly_rel()] for the life-stress relationship plot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotly_mcf.R
\name{plotly_mcf}
\alias{plotly_mcf}
\title{Interactive Mean Cumulative Function Plot.}
\usage{
plotly_mcf(
  mcf_obj,
  showConf = TRUE,
  showGrid = TRUE,
  main = "Mean Cumulative Function Plot",
  xlab = "Time",
  ylab = "Mean Cumulative Function",
  fitCol = "black",
  confCol = "black",
  gridCol = "lightgray",
  signif = 3,
  cols = NULL
)
}
\arguments{
\item{mcf_obj}{An object of class 'mcf', or a list of such objects. Each
object is created using the `mcf()` function from the `ReliaGrowR` package.}

\item{showConf}{Show the confidence bounds (TRUE) or not (FALSE). Default is TRUE.}

\item{showGrid}{Show grid (TRUE) or hide grid (FALSE). Default is TRUE.}

\item{main}{Main title. Default is "Mean Cumulative Function Plot".}

\item{xlab}{X-axis label. Default is "Time".}

\item{ylab}{Y-axis label. Default is "Mean Cumulative Function".}

\item{fitCol}{Color of the MCF step function. Default is "black". Used only
for a single mcf object; ignored when `cols` is provided or multiple objects
are supplied.}

\item{confCol}{Color of the confidence bounds. Default is "black". Used only
for a single mcf object; ignored when `cols` is provided or multiple objects
are supplied.}

\item{gridCol}{Color of the grid. Default is "lightgray".}

\item{signif}{Significant digits of results. Default is 3. Must be a positive integer.}

\item{cols}{Optional character vector of colors, one per mcf object. When
provided, each object's step function and confidence bounds are drawn in the
corresponding color. Recycled if shorter than the number of objects.}
}
\value{
A `plotly` object representing the interactive MCF plot.
}
\description{
The function creates an interactive Mean Cumulative Function (MCF) plot for
one or more `mcf` objects. When a list of objects is provided the models are
overlaid on the same plot, each rendered in a distinct color.
The MCF is rendered as a step function. Optional confidence bounds are shown
as a shaded band around the estimate.
}
\details{
The MCF is a nonparametric estimate of the expected cumulative number of
events per system by time \eqn{t}. It is computed from recurrence data
(multiple events per system) and rendered as a step function. The slope
of the MCF at any point approximates the current event rate (repairs per
unit time). Confidence bounds are computed using the Nelson variance
estimator.
}
\examples{
library(ReliaGrowR)
ids <- c("A", "A", "A", "B", "B", "C", "C", "C", "C")
times <- c(50, 150, 350, 100, 300, 80, 200, 320, 450)
fit <- mcf(id = ids, time = times)
plotly_mcf(fit)

# Overlay two MCF objects
fit2 <- mcf(id = c("X", "X", "Y"), time = c(60, 220, 180))
plotly_mcf(list(fit, fit2), cols = c("steelblue", "tomato"))
}
\seealso{
[plotly_nhpp()] to overlay a parametric Power Law model on the MCF;
[plotly_exposure()] for the cumulative event rate.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotly_contour.R
\name{plotly_contour}
\alias{plotly_contour}
\title{Interactive Contour Plot}
\usage{
plotly_contour(
  wblr_obj,
  main = "Contour Plot",
  xlab = "Eta",
  ylab = "Beta",
  showGrid = TRUE,
  cols = NULL,
  gridCol = "lightgray",
  signif = 3
)
}
\arguments{
\item{wblr_obj}{A single `wblr` object or a list of `wblr` objects. Each object
must have contours generated using `method.conf = 'lrb'`.}

\item{main}{Main title for the plot.}

\item{xlab}{X-axis label (typically Eta or Sigmalog).}

\item{ylab}{Y-axis label (typically Beta or Mulog).}

\item{showGrid}{Logical; whether to show grid lines (default TRUE).}

\item{cols}{Optional vector of colors for each contour/estimate pair. If not provided,
colors are chosen from a default palette.}

\item{gridCol}{Color of the grid lines (default 'lightgray').}

\item{signif}{Number of significant digits to display for estimates and contour coordinates.
Defaults to 3.}
}
\value{
A `plotly` object representing the interactive contour plot.
}
\description{
This function creates an interactive contour plot for one or more
`wblr` objects, each assumed to have confidence contours generated via
`method.conf = 'lrb'`. The function overlays all contours in a single plot and displays
their respective MLE point estimates.
}
\details{
Confidence contours require a likelihood-ratio-based (\code{method.conf = 'lrb'})
MLE fit (\code{method.fit = 'mle'}). The contour traces the locus of
(\eqn{\eta}, \eqn{\beta}) pairs whose log-likelihood falls within a
chi-squared critical value of the MLE peak, giving a joint confidence
region for the two Weibull parameters. The MLE point estimate is shown as
a marker at the center of each contour.
}
\examples{
library(WeibullR)
library(ReliaPlotR)

failures1 <- c(30, 49, 82, 90, 96)
failures2 <- c(20, 40, 60, 80, 100)
obj1 <- wblr.conf(wblr.fit(wblr(failures1), method.fit = "mle"), method.conf = "lrb")
obj2 <- wblr.conf(wblr.fit(wblr(failures2), method.fit = "mle"), method.conf = "lrb")
plotly_contour(list(obj1, obj2), main = "Overlayed Contours")

}
\seealso{
[plotly_wblr()] for probability plots using the same \code{wblr} objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_wblr.R
\name{tidy_wblr}
\alias{tidy_wblr}
\title{Extract Tidy Parameter Estimates from a wblr Object.}
\usage{
tidy_wblr(wblr_obj)
}
\arguments{
\item{wblr_obj}{A single object of class `'wblr'`, or a list of such objects.
Each object must have been fitted with [WeibullR::wblr.fit()].}
}
\value{
A named list with two elements:
  \describe{
    \item{`estimates`}{A `data.frame` with one row per `wblr` object and
      columns `dist`, `method_fit`, `param1_name`, `param1`, `param2_name`,
      `param2`, `param3_name`, `param3`, `gof_metric`, `gof_value`,
      `method_conf`, `n_failures`.}
    \item{`bounds`}{For a single object, a `data.frame` of confidence bound
      data (columns `Datum`, `unrel`, `Lower`, `Upper`), or `NULL` if no
      confidence bounds were computed. For a list of objects, a list of such
      data frames (one per object).}
  }
}
\description{
Returns a tidy data frame of distribution parameter estimates, goodness-of-fit
metrics, and (optionally) confidence bound data from one or more fitted `wblr`
objects. This makes it straightforward to use ReliaPlotR output in reproducible
research workflows — export to CSV, include in tables, or compare multiple fits.
}
\details{
Parameters are returned on their natural scales. For Weibull and
three-parameter Weibull (\code{weibull3p}) models, \code{param1} is the
shape parameter \eqn{\beta} and \code{param2} is the scale parameter
\eqn{\eta}; \code{param3} is the location parameter \eqn{\gamma} for
\code{weibull3p} (NA otherwise). For lognormal models, \code{param1} is
\eqn{\mu_{log}} and \code{param2} is \eqn{\sigma_{log}}.

Goodness-of-fit is reported as R\eqn{^2} (column value \code{"R2"}) for
rank-regression fits, or log-likelihood (\code{"loglikelihood"}) for MLE
fits.
}
\examples{
library(WeibullR)
failures <- c(30, 49, 82, 90, 96)
obj <- wblr.conf(wblr.fit(wblr(failures), method.fit = "mle"), method.conf = "lrb")
result <- tidy_wblr(obj)
result$estimates
result$bounds

# List of objects
obj2 <- wblr.conf(wblr.fit(wblr(c(20, 40, 60, 80, 100)), method.fit = "mle"),
                  method.conf = "lrb")
result2 <- tidy_wblr(list(obj, obj2))
result2$estimates
}
\references{
Meeker, W. Q., and Escobar, L. A. (1998). \emph{Statistical Methods for
Reliability Data}. Wiley.
}
\seealso{
[plotly_wblr()] for the corresponding interactive probability plot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotly_rga.R
\name{plotly_rga}
\alias{plotly_rga}
\title{Interactive Reliability Growth Plot.}
\usage{
plotly_rga(
  rga_obj,
  showConf = TRUE,
  showGrid = TRUE,
  main = "Reliability Growth Plot",
  xlab = "Cumulative Time",
  ylab = "Cumulative Failures",
  pointCol = "black",
  fitCol = "black",
  confCol = "black",
  gridCol = "lightgray",
  breakCol = "black",
  signif = 3,
  cols = NULL
)
}
\arguments{
\item{rga_obj}{An object of class 'rga', or a list of such objects. Each object
is created using the `rga()` function from the `ReliaGrowR` package.}

\item{showConf}{Show the confidence bounds (TRUE) or not (FALSE). Default is TRUE.}

\item{showGrid}{Show grid (TRUE) or hide grid (FALSE). Default is TRUE.}

\item{main}{Main title. Default is "Reliability Growth Plot".}

\item{xlab}{X-axis label. Default is "Cumulative Time".}

\item{ylab}{Y-axis label. Default is "Cumulative Failures".}

\item{pointCol}{Color of the point values. Default is "black". Used only for a
single rga object; ignored when `cols` is provided or multiple objects are supplied.}

\item{fitCol}{Color of the model fit. Default is "black". Used only for a single
rga object; ignored when `cols` is provided or multiple objects are supplied.}

\item{confCol}{Color of the confidence bounds. Default is "black". Used only for a
single rga object; ignored when `cols` is provided or multiple objects are supplied.}

\item{gridCol}{Color of the grid. Default is "lightgray".}

\item{breakCol}{Color of the breakpoints. Default is "black". Used only for a
single rga object; ignored when `cols` is provided or multiple objects are supplied.}

\item{signif}{Significant digits of results. Default is 3. Must be a positive integer.}

\item{cols}{Optional character vector of colors, one per rga object. When provided,
each object's points, fit line, confidence bounds, and breakpoints are all drawn in
the corresponding color. Recycled if shorter than the number of objects.}
}
\value{
A `plotly` object representing the interactive reliability growth plot.
}
\description{
The function creates an interactive reliability growth plot for one or more `rga`
objects. When a list of objects is provided the models are overlaid on the same
plot, each rendered in a distinct color.
The plot includes cumulative failures over time, the model fit, and optional confidence bounds.
Vertical lines indicate change points if breakpoints are specified in the rga object.
}
\details{
The Crow-AMSAA (NHPP Power Law) model fits a Non-Homogeneous Poisson
Process where the expected cumulative failures follow
\eqn{E[N(t)] = \lambda t^\beta}. A \eqn{\beta < 1} indicates reliability
growth (decreasing failure rate), \eqn{\beta > 1} indicates reliability
degradation. For piecewise models, a separate set of parameters is fit
within each interval defined by \code{breaks}, and vertical lines mark
the change-point times.
}
\examples{
library(ReliaGrowR)
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
rga <- rga(times, failures)
plotly_rga(rga)

times <- c(100, 200, 300, 400, 500, 600, 700, 800, 900, 1000)
failures <- c(1, 2, 1, 1, 1, 2, 3, 1, 2, 4)
breakpoints <- 400
rga2 <- rga(times, failures, model_type = "Piecewise NHPP", breaks = breakpoints)
plotly_rga(rga2, fitCol = "blue", confCol = "blue", breakCol = "red")

# Overlay two models
rga3 <- rga(c(50, 150, 250, 350, 450), c(2, 1, 3, 1, 2))
plotly_rga(list(rga, rga3))
}
\seealso{
[plotly_duane()] for the Duane MTBF growth model on the same data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotly_wblr.R
\name{plotly_wblr}
\alias{plotly_wblr}
\title{Interactive Probability Plot.}
\usage{
plotly_wblr(
  wblr_obj,
  susp = NULL,
  showConf = TRUE,
  showSusp = TRUE,
  showGrid = TRUE,
  main = "Probability Plot",
  xlab = "Time to Failure",
  ylab = "Probability",
  probCol = "black",
  fitCol = "black",
  confCol = "black",
  intCol = "black",
  gridCol = "lightgray",
  signif = 3,
  cols = NULL
)
}
\arguments{
\item{wblr_obj}{A single object of class 'wblr', or a list of such objects. This is
a required argument. Each object must use the same distribution (e.g. all Weibull or
all lognormal). Suspension subplots are only shown when a single object is provided.}

\item{susp}{An optional numeric vector of suspension data. Default is NULL. Ignored
when multiple wblr objects are provided.}

\item{showConf}{Show the confidence bounds (TRUE) or not (FALSE). Default is TRUE if
confidence bounds are available in the wblr object.}

\item{showSusp}{Show the suspensions plot (TRUE) or not (FALSE). Default is TRUE
if susp is provided.}

\item{showGrid}{Show grid (TRUE) or hide grid (FALSE). Default is TRUE.}

\item{main}{Main title. Default is 'Probability Plot'.}

\item{xlab}{X-axis label. Default is 'Time to Failure'.}

\item{ylab}{Y-axis label. Default is 'Probability'.}

\item{probCol}{Color of the probability values. Default is 'black'. Used only for a
single wblr object; ignored when `cols` is provided or multiple objects are supplied.}

\item{fitCol}{Color of the model fit. Default is 'black'. Used only for a single
wblr object; ignored when `cols` is provided or multiple objects are supplied.}

\item{confCol}{Color of the confidence bounds. Default is 'black'. Used only for a
single wblr object; ignored when `cols` is provided or multiple objects are supplied.}

\item{intCol}{Color of the intervals for interval censored models. Default is 'black'.}

\item{gridCol}{Color of the grid. Default is 'lightgray'.}

\item{signif}{Significant digits of results. Default is 3. Must be a positive integer.}

\item{cols}{Optional character vector of colors, one per wblr object. When provided,
each object's data points, fit line, and confidence bounds are all drawn in the
corresponding color. Recycled if shorter than the number of objects.}
}
\value{
A `plotly` object representing the interactive probability plot.
}
\description{
This function creates an interactive probability plot for one or more wblr objects.
When a list of objects is provided the models are overlaid on the same plot, each
rendered in a distinct color. All objects must use the same distribution family.
It can include confidence bounds, suspension data (single object only), and a results table.
}
\details{
For Weibull distributions the y-axis uses a double-log transformation
(\code{log(-log(1 - F))}) so that a straight line represents a perfect
Weibull fit. For lognormal distributions the y-axis uses the normal
quantile transformation (\code{qnorm(F)}), i.e., standard normal probability
paper. Three-parameter Weibull (\code{dist = 'weibull3p'}) shifts the x-axis
by the estimated location (gamma) parameter before applying the same
transformation.

Goodness-of-fit is reported as R² for rank-regression fits or
log-likelihood for MLE fits.
}
\examples{
library(WeibullR)
library(ReliaPlotR)
failures <- c(30, 49, 82, 90, 96)
obj <- wblr.conf(wblr.fit(wblr(failures)))
plotly_wblr(obj)

suspensions <- c(100, 45, 10)
obj <- wblr.conf(wblr.fit(wblr(failures, suspensions)))
plotly_wblr(obj, suspensions,
  fitCol = "blue",
  confCol = "blue"
)
inspection_data <- data.frame(
  left = c(0, 6.12, 19.92, 29.64, 35.4, 39.72, 45.32, 52.32),
  right = c(6.12, 19.92, 29.64, 35.4, 39.72, 45.32, 52.32, 63.48),
  qty = c(5, 16, 12, 18, 18, 2, 6, 17)
)
suspensions <- data.frame(time = 63.48, event = 0, qty = 73)
obj <- wblr(suspensions, interval = inspection_data)
obj <- wblr.fit(obj, method.fit = "mle")
obj <- wblr.conf(obj, method.conf = "fm", lty = 2)
suspensions <- as.vector(suspensions$time)
plotly_wblr(obj,
  susp = suspensions, fitCol = "red", confCol = "red", intCol = "blue",
  main = "Parts Cracking Inspection Interval Analysis",
  ylab = "Cumulative \% Cracked", xlab = "Inspection Time"
)
failures <- c(25, 30, 42, 49, 55, 67, 73, 82, 90, 96, 101, 110, 120, 132, 145)
fit <- wblr.conf(wblr.fit(wblr(failures), dist = "weibull3p"))
plotly_wblr(fit, fitCol = "darkgreen", confCol = "darkgreen")

# Overlay two Weibull models
obj2 <- wblr.conf(wblr.fit(wblr(c(20, 40, 60, 80, 100)), method.fit = "mle"),
                  method.conf = "lrb")
obj3 <- wblr.conf(wblr.fit(wblr(c(10, 30, 50, 70, 90)), method.fit = "mle"),
                  method.conf = "lrb")
plotly_wblr(list(obj2, obj3))

}
\seealso{
[plotly_contour()] for MLE parameter confidence contours.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotly_nhpp.R
\name{plotly_nhpp}
\alias{plotly_nhpp}
\title{Interactive NHPP Plot.}
\usage{
plotly_nhpp(
  nhpp_obj,
  showConf = TRUE,
  showGrid = TRUE,
  main = "NHPP Plot",
  xlab = "Cumulative Time",
  ylab = "Mean Cumulative Function",
  pointCol = "black",
  fitCol = "black",
  confCol = "black",
  gridCol = "lightgray",
  breakCol = "black",
  signif = 3,
  cols = NULL
)
}
\arguments{
\item{nhpp_obj}{An object of class 'nhpp', or a list of such objects. Each
object is created using the `nhpp()` function from the `ReliaGrowR` package.}

\item{showConf}{Show the confidence bounds (TRUE) or not (FALSE). Default is TRUE.}

\item{showGrid}{Show grid (TRUE) or hide grid (FALSE). Default is TRUE.}

\item{main}{Main title. Default is "NHPP Plot".}

\item{xlab}{X-axis label. Default is "Cumulative Time".}

\item{ylab}{Y-axis label. Default is "Mean Cumulative Function".}

\item{pointCol}{Color of the MCF data points. Default is "black". Used only
for a single nhpp object; ignored when `cols` is provided or multiple objects
are supplied.}

\item{fitCol}{Color of the model fit. Default is "black". Used only for a
single nhpp object; ignored when `cols` is provided or multiple objects are
supplied.}

\item{confCol}{Color of the confidence bounds. Default is "black". Used only
for a single nhpp object; ignored when `cols` is provided or multiple objects
are supplied.}

\item{gridCol}{Color of the grid. Default is "lightgray".}

\item{breakCol}{Color of the breakpoints. Default is "black". Used only for a
single nhpp object; ignored when `cols` is provided or multiple objects are
supplied.}

\item{signif}{Significant digits of results. Default is 3. Must be a positive integer.}

\item{cols}{Optional character vector of colors, one per nhpp object. When
provided, each object's points, fit line, confidence bounds, and breakpoints
are all drawn in the corresponding color. Recycled if shorter than the number
of objects.}
}
\value{
A `plotly` object representing the interactive NHPP plot.
}
\description{
The function creates an interactive Non-Homogeneous Poisson Process (NHPP)
plot for one or more `nhpp` objects. When a list of objects is provided the
models are overlaid on the same plot, each rendered in a distinct color.
The plot shows the nonparametric Mean Cumulative Function (MCF) alongside
the parametric model fit and optional confidence bounds. Vertical lines
indicate change points if breakpoints are specified in the nhpp object.
}
\details{
The nonparametric MCF is overlaid with the fitted parametric Power Law
NHPP model, \eqn{E[N(t)] = \lambda t^\beta}. Use this plot to assess
whether the parametric model fits the observed event history and to
identify change points. Confidence bounds are based on the Fisher matrix
approximation of the fitted parameters. For piecewise models, vertical
dotted lines mark each breakpoint.
}
\examples{
library(ReliaGrowR)
times <- c(100, 200, 300, 400, 500)
events <- c(1, 2, 1, 3, 2)
fit <- nhpp(time = times, event = events)
plotly_nhpp(fit)

# Piecewise model with a breakpoint
times2 <- c(100, 200, 300, 400, 500, 600, 700, 800, 900, 1000)
events2 <- c(1, 2, 1, 1, 1, 2, 3, 1, 2, 4)
fit2 <- nhpp(time = times2, event = events2, breaks = 500, method = "LS")
plotly_nhpp(fit2, breakCol = "red")

# Overlay two models
plotly_nhpp(list(fit, fit2))
}
\seealso{
[plotly_mcf()] for the nonparametric MCF alone; [plotly_rga()] for
a Crow-AMSAA cumulative-failures view.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotly_exposure.R
\name{plotly_exposure}
\alias{plotly_exposure}
\title{Interactive Exposure Plot.}
\usage{
plotly_exposure(
  exposure_obj,
  showGrid = TRUE,
  main = "Exposure Plot",
  xlab = "Time",
  ylab = "Event Rate",
  fitCol = "black",
  gridCol = "lightgray",
  signif = 3,
  cols = NULL
)
}
\arguments{
\item{exposure_obj}{An object of class 'exposure', or a list of such objects.
Each object is created using the `exposure()` function from the `ReliaGrowR`
package.}

\item{showGrid}{Show grid (TRUE) or hide grid (FALSE). Default is TRUE.}

\item{main}{Main title. Default is "Exposure Plot".}

\item{xlab}{X-axis label. Default is "Time".}

\item{ylab}{Y-axis label. Default is "Event Rate".}

\item{fitCol}{Color of the event rate step function. Default is "black". Used
only for a single exposure object; ignored when `cols` is provided or multiple
objects are supplied.}

\item{gridCol}{Color of the grid. Default is "lightgray".}

\item{signif}{Significant digits of results. Default is 3. Must be a positive integer.}

\item{cols}{Optional character vector of colors, one per exposure object. When
provided, each object's step function is drawn in the corresponding color.
Recycled if shorter than the number of objects.}
}
\value{
A `plotly` object representing the interactive exposure plot.
}
\description{
The function creates an interactive exposure plot for one or more \code{exposure}
objects. When a list of objects is provided the estimates are overlaid on the
same plot, each rendered in a distinct color. The plot shows the instantaneous
event rate (events per unit time per system at risk) as a step function,
calculated from recurrence data by dividing the event count in each interval
by the total system-time at risk during that interval.
}
\details{
Unlike [plotly_mcf()] which shows cumulative events, the exposure plot
shows the instantaneous event rate: the number of events per unit time
per system at risk in each interval. A flat exposure plot suggests a
constant event rate (homogeneous Poisson process); a declining rate
suggests improvement; a rising rate suggests degradation. Use this plot
alongside [plotly_mcf()] to diagnose time-dependence in the event process.
}
\examples{
library(ReliaGrowR)
ids <- c("A", "A", "A", "B", "B", "C", "C", "C", "C")
times <- c(50, 150, 350, 100, 300, 80, 200, 320, 450)
fit <- exposure(id = ids, time = times)
plotly_exposure(fit)

# Overlay two exposure objects
fit2 <- exposure(id = c("X", "X", "Y"), time = c(60, 220, 180))
plotly_exposure(list(fit, fit2), cols = c("steelblue", "tomato"))
}
\seealso{
[plotly_mcf()] for the cumulative view; [plotly_nhpp()] for a
parametric NHPP model.
}
